SystemOrganization addCategory: #'Pgi-Setup'!
SystemOrganization addCategory: #'Pgi-Resources'!
SystemOrganization addCategory: #'Pgi-Managing'!
SystemOrganization addCategory: #'Pgi-LevelMorphic'!
SystemOrganization addCategory: #'Pgi-LevelLogic'!
SystemOrganization addCategory: #'Pgi-LevelLogic-States'!
SystemOrganization addCategory: #'Pgi-Test'!

PasteUpMorph subclass: #PgiGame
	instanceVariableNames: 'screenSize state highestReachedLevel highestPossibleLevel ressourceManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiGame commentStamp: 'RS 2/11/2011 00:14:57.941' prior: 0!
A PgiGame represents the most top level Canvas on which all the Objects in the Game are actually drawn. Additionaly it saves the general state of the game, such has how far the user has played through the game and which maps he has already opened up.

Instance Variables
	highestLevel:		<Object>
	reachedLevel:		<Object>
	ressourceManager:		<Object>
	screenSize:		<Object>
	state:		<Object>

highestLevel
	- The highest possible level. Cana lso be seen as the total number of possible levels.

reachedLevel
	- the highest level that the user has yet reached.

ressourceManager
	- the unit, from which all the components of the game recieve their needed data, such as images or music.

screenSize
	- size of the Canvas; extent of the game.

state
	- the state in which the game temporary resides such as: home, loading or level.
!

!PgiGame class methodsFor: 'parts bin' stamp: 'rs 2/18/2011 20:53:39.352'!
descriptionForPartsBin 

	^ self partName: 'Penguins' 
	categories: #('SWA11') 
	documentation: 'A little Lemmings clone with cute penguins.'
	sampleImageForm: ((PgiResource retrieve: #Pgicon) first)! !

!PgiGame methodsFor: 'game state' stamp: 'RS 2/10/2011 20:20:32.281'!
changeState: aState
	
	self state abandon.
	
	self state: aState.
	
	self state launch.! !

!PgiGame methodsFor: 'game state' stamp: 'kh 2/11/2011 17:19:06.274'!
closeGame

	self state abandon.
	self abandon.! !

!PgiGame methodsFor: 'game state' stamp: 'kh 2/18/2011 10:57:09.191'!
endLevel: aLevelNumber successful: aBoolean

	(aBoolean and: (aLevelNumber = self highestReachedLevel)) ifTrue:  [
			(self highestReachedLevel < self highestPossibleLevel) ifTrue: [
						self highestReachedLevel: (self highestReachedLevel + 1)]].
				
	
	self changeState: (PgiGameStateFinalize newFinalizeStateFor: self with: aLevelNumber successful: aBoolean).
! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:28:56.65'!
highestPossibleLevel

	^ highestPossibleLevel! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:29:05.519'!
highestPossibleLevel: aNumber

	highestPossibleLevel := aNumber! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:29:09.067'!
highestReachedLevel

	^ highestReachedLevel! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:29:13.75'!
highestReachedLevel: aNumber

	highestReachedLevel := aNumber! !

!PgiGame methodsFor: 'initialization' stamp: 'pre 2/19/2011 11:39:13.839'!
initialize

	"We are making use of a state for the purpose of switching between multiple game-modes (home, level, ...). "

	super initialize.
	self 
		ressourceManager: PgiRessourceManager new;
		color: (Color transparent);
		screenSize: 800@600;
		extent: ((self screenSize x)@(self screenSize y));
		highestReachedLevel: 4;
		highestPossibleLevel:  7;
		openInHand;
		state: (PgiGameStateHome newHomeStateFor: self with: highestReachedLevel).
	
	self state launch.
! !

!PgiGame methodsFor: 'initialization' stamp: 'kh 1/19/2011 01:28:07.352'!
initializeToStandAlone

	self initialize.! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:07:17.688'!
openHomeScreen 

	self changeState: (PgiGameStateHome newHomeStateFor: self with: self highestReachedLevel).! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:21:50.754'!
ressourceManager

	^ ressourceManager! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:22:04.197'!
ressourceManager: aPgiRessourceManager

	ressourceManager := aPgiRessourceManager! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:22:13.711'!
screenSize

	^ screenSize! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:22:22.899'!
screenSize: aPoint

	screenSize := aPoint! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 2/18/2011 00:22:00.557'!
scrollablePart

	^ self state scrollablePart! !

!PgiGame methodsFor: 'game state' stamp: 'kh 2/11/2011 13:29:32.34'!
startLevel: aLevelNumber
	
	| resultState |
	
	self changeState: (PgiGameStateLoading newLoadingStateFor: self).
	
	[resultState := PgiGameStateLevel newLevelStateFor: self with: aLevelNumber.
	
	self changeState: resultState.] fork.! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:22:29.944'!
state

	^ state! !

!PgiGame methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:22:39.686'!
state: aPgiState

	state := aPgiState! !

Morph subclass: #PgiScrollSensor
	instanceVariableNames: 'isActive movementVector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiScrollSensor commentStamp: 'RS 2/11/2011 11:14:39.405' prior: 0!
A PgiScrollSensor is a rectangular field that senses if the users sourcer hovers above it and notifies its parent scroller object to scroll in the appropriate direction.

Instance Variables
	active:		<Boolean>
	direction:		<Point>

active
	- sets wether the Sensor is responsive or not

direction
	- the direction the game is moved , when the sensor is hovered.
!

!PgiScrollSensor methodsFor: 'event handling' stamp: 'pre 1/18/2011 17:47'!
handlesMouseOver: anEvent

	^true! !

!PgiScrollSensor methodsFor: 'initialization' stamp: 'kh 2/17/2011 18:04:42.214'!
initialize 

	super initialize.
	self color: Color transparent;
		  isActive: false.! !

!PgiScrollSensor methodsFor: 'accessing' stamp: 'kh 2/17/2011 18:02:26.673'!
isActive

	^ isActive! !

!PgiScrollSensor methodsFor: 'accessing' stamp: 'kh 2/17/2011 18:02:34.921'!
isActive: aBoolean

	isActive := aBoolean! !

!PgiScrollSensor methodsFor: 'event handling' stamp: 'kh 2/17/2011 18:05:30.995'!
mouseEnter: anEvent

	self isActive: true;
		 color: (Color white alpha: 0.5).! !

!PgiScrollSensor methodsFor: 'event handling' stamp: 'kh 2/17/2011 18:05:37.924'!
mouseLeave: anEvent

	self isActive: false;
		 color: Color transparent.! !

!PgiScrollSensor methodsFor: 'accessing' stamp: 'kh 2/18/2011 00:19:55.839'!
movement

	(self isActive) ifTrue: [^ self movementVector].
	^ 0@0! !

!PgiScrollSensor methodsFor: 'accessing' stamp: 'kh 2/17/2011 22:49:59.957'!
movementVector

	^ movementVector! !

!PgiScrollSensor methodsFor: 'accessing' stamp: 'kh 2/17/2011 22:50:06.126'!
movementVector: anObject

	movementVector := anObject! !

Morph subclass: #PgiTexturedMorph
	instanceVariableNames: 'graphic shape baseImage texturePart texturing'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelMorphic'!

!PgiTexturedMorph commentStamp: 'RS 2/11/2011 11:44:41.837' prior: 0!
A PgiTexturedMorph is a PolygonMorph that can be filled with a Texture which consists out of Forms that are supplied by the graphic Object.

Instance Variables
	baseImage:		<Form>
	graphic:		<PgiGraphic>
	shape:		<PolygonMorph>
	textureParts:		<OrderedCollection>

baseImage
	- xxxxx

graphic
	- xxxxx

shape
	- xxxxx

textureParts
	- xxxxx
!

!PgiTexturedMorph class methodsFor: 'instance creation' stamp: 'pre 2/12/2011 13:57:47.94'!
newPgiTexturedMorphFromGraphic: aPgiGraphic Mask: aPolygon
	"This method takes an Image wich will later be used as the texture and a Polygon Morph, which clips the Texture"
	| newMorph |
	aPolygon hide.
	newMorph := PgiTexturedMorph new.
	newMorph graphic: aPgiGraphic;
				baseImage: (newMorph graphic next);
				bounds: ((newMorph correctShift: aPolygon) bounds);
				position: 0@0;
				shape: (newMorph correctShift: aPolygon);
			      drawTexture;
                        clipTextures.
	^newMorph! !

!PgiTexturedMorph methodsFor: 'creation and cleanup' stamp: 'rs 2/18/2011 18:45:56.563'!
abandon

	self texturePart abandon.
	super abandon.! !

!PgiTexturedMorph methodsFor: 'accessing' stamp: 'kh 12/2/2010 20:47:58.734'!
baseImage
	^ baseImage! !

!PgiTexturedMorph methodsFor: 'accessing' stamp: 'RS 12/16/2010 22:12:41.77'!
baseImage: aColorForm

	baseImage := aColorForm.! !

!PgiTexturedMorph methodsFor: 'graphics' stamp: 'RS 2/10/2011 21:18:57.053'!
changeAppearance: anAppearanceSymbol

	self graphic changeAppearance: anAppearanceSymbol.
	self baseImage: (self graphic next).
	
	self drawTexture;
             clipTextures.! !

!PgiTexturedMorph methodsFor: 'graphics' stamp: 'pre 2/16/2011 19:05:05.882'!
clipTextures
	| clippingForm aBitBlt |

	clippingForm := (Form extent: (self shape extent + (5@5))) fillBlack.
	aBitBlt := BitBlt	destForm: clippingForm 
					sourceForm: (self shape filledForm) 
					fillColor: nil
					combinationRule: 6 
					destOrigin: 0@0
					sourceOrigin: 0@0 
					extent: (self shape extent) 
					clipRect: (clippingForm computeBoundingBox).
	aBitBlt copyBits.
	aBitBlt := BitBlt	destForm: (self texturing) 
					sourceForm: (clippingForm ) 
					fillColor: nil
					combinationRule: 17 
					destOrigin: 0@0
					sourceOrigin: 0@0 
					extent: (clippingForm  extent) 
					clipRect: (self texturing computeBoundingBox).
	aBitBlt copyBits.
	
	self texturePart image: (self texturing).
	self texturePart changed.! !

!PgiTexturedMorph methodsFor: 'graphics' stamp: 'pre 2/12/2011 13:47:57.24'!
correctShift: aPolygon

	| resultPolygon shift |
	
	shift := ((aPolygon vertices detectMin: [ :each | each x] ) x) @ ((aPolygon vertices detectMin: [:each | each y] ) y).
	resultPolygon := PolygonMorph new.
	resultPolygon setVertices: (aPolygon vertices do: [ :eachVertice | eachVertice - shift]).
	
	^resultPolygon! !

!PgiTexturedMorph methodsFor: 'graphics' stamp: 'rs 2/18/2011 18:27:54.623'!
drawTexture
	| textureForm aTexture aBitBlt |
	
	textureForm := Form extent: (self shape extent) depth: 32.
	aTexture := self baseImage deepCopy .
	textureForm := Form extent: (self shape extent) depth: 32.
	0 to: ((textureForm extent x) / (aTexture extent x)) do: [ :eachX | 
		0 to: ((textureForm extent y) / (aTexture extent y)) do: [ :eachY |
			aBitBlt := BitBlt	destForm: textureForm 
							sourceForm: aTexture 
							fillColor: nil
							combinationRule: 7 
							destOrigin: (eachX * (aTexture extent x))@(eachY *(aTexture extent y)) 
							sourceOrigin: 0@0 
							extent: (aTexture extent) 
							clipRect: (textureForm computeBoundingBox).
			aBitBlt copyBits.
		]].

	self texturing: textureForm.! !

!PgiTexturedMorph methodsFor: 'accessing' stamp: 'pre 1/9/2011 14:19:21.383'!
graphic
	"Answer the value of graphic"

	^ graphic! !

!PgiTexturedMorph methodsFor: 'accessing' stamp: 'pre 1/9/2011 14:19:21.397'!
graphic: anObject
	"Set the value of graphic"

	graphic := anObject! !

!PgiTexturedMorph methodsFor: 'initialization' stamp: 'rs 2/18/2011 18:47:10.557'!
initialize
	"This method is only doing minor things, because further details (e.g. Texture Image etc.) are handled by the Factory method of this Class"
	super initialize.
	self clipSubmorphs: true;
		 color: Color transparent;
		 texturePart: (ImageMorph new);
		 addMorph: (self texturePart).
	! !

!PgiTexturedMorph methodsFor: 'accessing' stamp: 'pre 1/9/2011 14:40:16.433'!
shape
	"Answer the value of shape"

	^ shape! !

!PgiTexturedMorph methodsFor: 'accessing' stamp: 'pre 1/9/2011 14:40:16.443'!
shape: anObject
	"Set the value of shape"

	shape := anObject! !

!PgiTexturedMorph methodsFor: 'accessing' stamp: 'pre 2/12/2011 14:05:42.982'!
texturePart
	"Answer the value of texturePart"

	^ texturePart! !

!PgiTexturedMorph methodsFor: 'accessing' stamp: 'pre 2/12/2011 14:05:42.994'!
texturePart: anObject
	"Set the value of texturePart"

	texturePart := anObject! !

!PgiTexturedMorph methodsFor: 'accessing' stamp: 'pre 2/12/2011 13:30:08.106'!
texturing
	"Answer the value of texturing"

	^ texturing! !

!PgiTexturedMorph methodsFor: 'accessing' stamp: 'pre 2/12/2011 13:30:08.118'!
texturing: anObject
	"Set the value of texturing"

	texturing := anObject! !

!PgiTexturedMorph methodsFor: 'graphics' stamp: 'pre 2/19/2011 02:01:29.674'!
updateShape: aVerticesCollection withShift: aShiftVector
	
	self shape: aVerticesCollection.
	self clipTextures.
	
	(aShiftVector x sign > 0 ) ifTrue: [ self position: ((self position) + ((aShiftVector x)@0)) ].
	self position: (self position) + (0@(aShiftVector y)).! !

ImageMorph subclass: #PgiWorldObjectMorph
	instanceVariableNames: 'graphic shape worldObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelMorphic'!

!PgiWorldObjectMorph commentStamp: 'RS 2/11/2011 11:55:52.825' prior: 0!
A PgiWorldObjectMorph is the visual representation of the WorldObject.

Instance Variables
	graphic:		<PgiGraphic>
	shape:		<PolygonMorph>
	worldObject:		<PgiWorldObject>

graphic
	- xxxxx

shape
	- Polygon which determines the WorldObjects shape.

worldObject
	- logical counterpart of the WorldObjectMorph
!

!PgiWorldObjectMorph class methodsFor: 'creation' stamp: 'RS 2/10/2011 21:18:30.733'!
newPgiWorldObjectMorphWithGraphics: aPgiGraphic
	
	| worldObjectMorph | 
	
	worldObjectMorph := self new.
	worldObjectMorph graphic: aPgiGraphic;
					      image: (worldObjectMorph graphic next).
								
	^worldObjectMorph! !

!PgiWorldObjectMorph methodsFor: 'creation and cleanup' stamp: 'rs 2/18/2011 18:46:14.938'!
abandon 

	super abandon.! !

!PgiWorldObjectMorph methodsFor: 'graphics' stamp: 'pre 1/7/2011 17:30:40.706'!
changeAppearance: anAppearanceSymbol
	(self graphic changeAppearance: anAppearanceSymbol) next.
	self nextFrame.! !

!PgiWorldObjectMorph methodsFor: 'accessing' stamp: 'pre 12/20/2010 15:50:39.053'!
graphic
	"Answer the value of graphic"

	^ graphic! !

!PgiWorldObjectMorph methodsFor: 'accessing' stamp: 'RS 12/22/2010 12:25:37.341'!
graphic: aPgiGraphic
	"Set the value of graphic"

	graphic := aPgiGraphic.! !

!PgiWorldObjectMorph methodsFor: 'accessing' stamp: 'kh 1/12/2011 02:07:02.593'!
handlesMouseDown: anEvent

	^true! !

!PgiWorldObjectMorph methodsFor: 'accessing' stamp: 'kh 1/13/2011 17:14:41.076'!
mouseDown: anEvent

	anEvent redButtonPressed ifTrue:
			[self worldObject handleClickEvent].! !

!PgiWorldObjectMorph methodsFor: 'graphics' stamp: 'pre 1/13/2011 19:38:39.778'!
moveBy: aVector
	self position: (self position + aVector ).! !

!PgiWorldObjectMorph methodsFor: 'private' stamp: 'pre 2/16/2011 16:43:54.604'!
nextFrame 
	self image: (self graphic next).
	self comeToFront.
	self beSticky.
	self image changed.
! !

!PgiWorldObjectMorph methodsFor: 'accessing' stamp: 'pre 12/20/2010 15:50:39.23'!
shape
	"Answer the value of shape"

	^ shape! !

!PgiWorldObjectMorph methodsFor: 'accessing' stamp: 'RS 12/22/2010 12:25:55.364'!
shape: aPolygonMorph
	"Set the value of shape"

	shape := aPolygonMorph.! !

!PgiWorldObjectMorph methodsFor: 'graphics' stamp: 'pre 1/13/2011 19:36:20.094'!
update
	self nextFrame! !

!PgiWorldObjectMorph methodsFor: 'accessing' stamp: 'kh 1/12/2011 02:08:46.432'!
worldObject
	"Answer the value of worldObject"

	^ worldObject! !

!PgiWorldObjectMorph methodsFor: 'accessing' stamp: 'kh 1/12/2011 02:08:46.497'!
worldObject: anObject
	"Set the value of worldObject"

	worldObject := anObject! !

IconicButton subclass: #PgiButton
	instanceVariableNames: 'purpose image menu isActivated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiButton commentStamp: 'RS 2/11/2011 00:17:13.105' prior: 0!
A PgiButton represents the Superclass of every clickable button in the Game itself.

Instance Variables
	activated:		<Bool>
	image:		<Object>
	menu:		<PgiMenu>
	purpose:		<Object>

activated
	- can activate or deactivate the button

image
	- the buttons icon

menu
	- the Menu to which the button belongs.

purpose
	- the action the button will perform, after he has been clicked.
!

!PgiButton methodsFor: 'accessing' stamp: 'pre 2/13/2011 17:54:57.415'!
activate

	^ self isActivated: true! !

!PgiButton methodsFor: 'accessing' stamp: 'pre 2/13/2011 17:55:01.526'!
deactivate

	^ self isActivated: false! !

!PgiButton methodsFor: 'event handling' stamp: 'kh 2/11/2011 11:26:29.098'!
handlesMouseDown: anEvent

	^true! !

!PgiButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:55:18.189'!
image

	^ image! !

!PgiButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:56:23.757'!
image: aForm

	image := aForm! !

!PgiButton methodsFor: 'initialization' stamp: 'kh 2/11/2011 17:36:50.134'!
initialize

	super initialize.
	self 
		color: Color transparent;
		borderWidth: 0;
		isActivated: true.! !

!PgiButton methodsFor: 'accessing' stamp: 'kh 2/11/2011 12:16:38.367'!
isActivated 

	^ isActivated! !

!PgiButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:55:29.108'!
isActivated: aBoolean

	isActivated := aBoolean! !

!PgiButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:55:33.984'!
menu

	^ menu! !

!PgiButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:55:42.299'!
menu: aPgiMenu

	menu := aPgiMenu! !

!PgiButton methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:22:53.92'!
mouseDown: anEvent

	anEvent redButtonPressed ifTrue:
		[self isActivated ifTrue:
			[self menu receive: self purpose]].! !

!PgiButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:55:51.59'!
purpose

	^ purpose! !

!PgiButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:55:55.608'!
purpose: anObject

	purpose := anObject! !

PgiButton subclass: #PgiLevelStartButton
	instanceVariableNames: 'levelNumber cover'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiLevelStartButton commentStamp: 'RS 2/11/2011 00:00:49.08' prior: 0!
A PgiLevelButton is a Button that will start a new Game.

Instance Variables
	coveredImage:		<Object>
	levelNumber:		<Object>

coveredImage
	- is the icon with whom the button shows the user which level he is going to start

levelNumber
	- is the levelnumber, that the button is going to use to instantiate the specified LevelState with the Level that it is supposed to start.
!

!PgiLevelStartButton methodsFor: 'accessing' stamp: 'kh 2/11/2011 12:09:25.646'!
activate

	self isActivated: true.
	self hideCover.! !

!PgiLevelStartButton methodsFor: 'activation' stamp: 'kh 2/16/2011 05:02:43.428'!
activateIfBelongsTo: aHighestReachedLevelNumber

		self setCover.
	      (self levelNumber <= aHighestReachedLevelNumber)
				 ifTrue: [self activate]
				 ifFalse: [self deactivate].


	! !

!PgiLevelStartButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:56:38.977'!
cover

	^ cover! !

!PgiLevelStartButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:57:03.435'!
cover: aMorph

	cover := aMorph! !

!PgiLevelStartButton methodsFor: 'nil' stamp: 'kh 2/11/2011 12:09:35.828'!
deactivate

	self isActivated: false.
	self showCover.! !

!PgiLevelStartButton methodsFor: 'visual properties' stamp: 'kh 2/16/2011 04:54:22.693'!
hideCover

	self cover visible: false.! !

!PgiLevelStartButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:52:25.281'!
levelNumber

	^ levelNumber! !

!PgiLevelStartButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:52:36.552'!
levelNumber: aNumber

	levelNumber := aNumber! !

!PgiLevelStartButton methodsFor: 'visual properties' stamp: 'kh 2/18/2011 11:23:43.661'!
setCover

	self cover position: self position;
				 openInWorld;
				 visible: false. 
				
	self addMorph: self cover.! !

!PgiLevelStartButton methodsFor: 'visual properties' stamp: 'kh 2/16/2011 04:54:42.533'!
showCover

	self cover visible: true.! !

PgiButton subclass: #PgiToolButton
	instanceVariableNames: 'textMorph leftToolUsages marker'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiToolButton commentStamp: 'RS 2/11/2011 00:05:02.202' prior: 0!
A PgiToolButton is a Button with whom the user can apply special crafts to the penguins such as building a bridge or parachuting.

Instance Variables
	textMorph:		<Object>
	toolUsagesLeft:		<Object>

textMorph
	- the text describing the tool

toolUsagesLeft
	- saves how many times the user is still allowed to apply the tool to a penguin. if the user drained all his tickets for this tool, this variable is zero, and he is no longer allowed to apply it a penguin.
!

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/11/2011 12:59:04.821'!
areToolUsagesLeft

	^  self leftToolUsages > 0! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 1/18/2011 21:28:48.517'!
deactivate 

	super deactivate.
	self resetHighlighting.! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/11/2011 13:15:43.872'!
decreaseLeftToolUsages

	self leftToolUsages: self leftToolUsages -1.
	self updateTextMorph.
	
	(self areToolUsagesLeft ) ifFalse: [self deactivate].
! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/11/2011 16:57:01.771'!
initialize 

	super initialize.
	self textMorph: TextMorph new.
! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:44:11.689'!
initializeMarker

	self marker position: self position;
				openInWorld;
				visible: false. 
				
	self addMorph: self marker.! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/11/2011 12:54:22.09'!
leftToolUsages

	^ leftToolUsages! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:24:41.219'!
leftToolUsages: aNumber

	leftToolUsages := aNumber! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:49:32.796'!
marker

	^ marker! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:25:09.664'!
marker: aMorph

	marker := aMorph! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:43:12.192'!
resetHighlighting

	self marker visible: false.! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:43:22.92'!
setHighlighting

	self marker visible: true.! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:51:34.146'!
setTextMorph
	
	| offset fontSize |
	offset :=  6@55.
	fontSize := 3.
	
	self textMorph position: (self position + offset);
					contents: (Text string: (self purpose, ': ', self leftToolUsages) attribute: TextEmphasis bold);
					color: Color white;
					beAllFont: (StrikeFont familyName: #BitstreamVeraSans size: fontSize);
					lock;
					openInWorld.

	self addMorph: self textMorph.! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:32:30.926'!
setToolUsagesLeft: aToolUsageDictionary

	self leftToolUsages: (aToolUsageDictionary at: self purpose);
		 setTextMorph.
	
	(self areToolUsagesLeft) ifTrue: [self activate]
							   ifFalse: [self deactivate].
	

! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:45:57.641'!
textMorph

	^ textMorph! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:25:51.314'!
textMorph: aTextMorph

	textMorph := aTextMorph! !

!PgiToolButton methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:48:53.636'!
updateTextMorph
	
	self textMorph contents: (self purpose, ': ', self leftToolUsages).! !

TestCase subclass: #PgiFactoryTest
	instanceVariableNames: 'spec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Test'!

!PgiFactoryTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/11/2011 16:49:06.79'!
generalTestOn: aObject ofClass: aTestClass
"Performs a general set of tests which apply to all objects created in PgiFactory. Specific tests should be implemented in the appropriate testBuildXYZ method."
	
	self assert: (aTestClass = aObject class);
		 assert: spec edges = aObject shape vertices;
		 assert: (PgiWorld = aObject world class);
		 assert: (spec position = aObject position).! !

!PgiFactoryTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/11/2011 16:50:01.405'!
setUp: testType appearance: testAppereance
"Creates a LevelObject according to testType and with testAppereance."

	|factory object |

	factory := self setUpFactory.

	self spec: PgiLevelObject new.
	self spec appearance: testAppereance;
			   type: testType;
			   position: 200@300;
			   edges: {0@0 . 50@0 . 50@50 . 0@50 }.
		
	object := factory build: self spec.
	^object! !

!PgiFactoryTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/11/2011 13:26:30.484'!
setUpFactory 
	
	| factory |
	
	factory := PgiFactory new.
	factory resourceManager: PgiRessourceManager new.
	factory world: PgiWorld new.
	^factory ! !

!PgiFactoryTest methodsFor: 'accessing' stamp: 'lsb 2/11/2011 16:24:42.04'!
spec
	"Answer the value of spec"

	^ spec! !

!PgiFactoryTest methodsFor: 'accessing' stamp: 'lsb 2/11/2011 16:24:42.122'!
spec: anObject
	"Set the value of spec"

	spec := anObject! !

!PgiFactoryTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/11/2011 16:47:25.805'!
testBuildGoal

	| goal |
	goal := self setUp: #Goal appearance: #standardGoal.
	
	self generalTestOn: goal ofClass: PgiPenguinGoal;
		 assert: (PgiWorldObjectMorph = goal morph class).! !

!PgiFactoryTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/11/2011 16:39:48.238'!
testBuildPenguin

	| penguin |

	penguin := self setUp: #Penguin appearance: #PenguinWalkRight.
	
	self generalTestOn: penguin ofClass: PgiPenguin;
		 assert: (PgiWorldObjectMorph = penguin morph class);
		 assert: (false = penguin rescued).! !

!PgiFactoryTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/11/2011 16:47:30.335'!
testBuildWell

	| well |
	
	well := self setUp: #Well appearance: #standardWell.
	self generalTestOn: well ofClass: PgiPenguinWell;
		 assert: (PgiWorldObjectMorph = well morph class).! !

!PgiFactoryTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/11/2011 16:45:16.263'!
testBuildWorldObject

	| worldObject |
	
	worldObject := self setUp: #Standard appearance: #ice.
	self generalTestOn: worldObject ofClass: PgiWorldObject;
		 assert: (PgiTexturedMorph = worldObject morph class).! !

TestCase subclass: #PgiGoalTest
	instanceVariableNames: 'spec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Test'!

!PgiGoalTest methodsFor: 'as yet unclassified' stamp: 'rs 2/19/2011 01:37:47.484'!
setUp: testType appearance: testAppereance
"Creates a LevelObject according to testType and with testAppereance."

	|factory object |

	factory := self setUpFactory.

	self spec: PgiLevelObject new.
	self spec appearance: testAppereance;
			   type: testType;
			   position: 200@300;
			   edges: {0@0 . 100@0 . 100@100 . 0@100 }.
		
	object := factory build: self spec.
	^object! !

!PgiGoalTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/18/2011 17:24:44.962'!
setUpFactory 
	
	| factory |
	
	factory := PgiFactory new.
	factory resourceManager: PgiRessourceManager new.
	factory world: PgiWorld new.
	^factory ! !

!PgiGoalTest methodsFor: 'accessing' stamp: 'lsb 2/18/2011 17:29:04.34'!
spec
	"Answer the value of spec"

	^ spec! !

!PgiGoalTest methodsFor: 'accessing' stamp: 'lsb 2/18/2011 17:29:04.428'!
spec: anObject
	"Set the value of spec"

	spec := anObject! !

!PgiGoalTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/18/2011 17:54:03.294'!
testArrivedPenguin

	| aGoal  aPenguin |
	
	aGoal := self setUp: #Goal appearance: #standardGoal.
	aGoal world gameState: (PgiGameStateLevel new).
	aGoal world gameState menu: (PgiLevelMenu new).
	aGoal world gameState menu scorePenguinsRescued: (TextMorph new).
	aGoal world gameState menu gameState: (aGoal world gameState).
	aGoal world gameState stage: aGoal world.
	
	aPenguin := self setUp: #Penguin appearance: #PenguinWalkRight.
	aGoal world registerPenguin: aPenguin.
	aGoal world gameState stepper subscribe: aPenguin  every: 1.
	aPenguin world: aGoal world.

	self assert: (aGoal world allPenguins size = 1).
	aGoal arrivedPenguin: (aPenguin).
	self assert: (aGoal world penguinsRescued = 1).
	self assert: (aGoal world allPenguins size = 0).! !

TestCase subclass: #PgiPenguinTest
	instanceVariableNames: 'spec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Test'!

!PgiPenguinTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/11/2011 18:03:12.106'!
setUp: testType appearance: testAppereance
"Creates a LevelObject according to testType and with testAppereance."

	|factory object |

	factory := self setUpFactory.

	self spec: PgiLevelObject new.
	self spec appearance: testAppereance;
			   type: testType;
			   position: 200@300;
			   edges: {0@0 . 50@0 . 50@50 . 0@50 }.
		
	object := factory build: self spec.
	^object! !

!PgiPenguinTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/11/2011 18:04:54.581'!
setUpFactory 
	
	| factory |
	
	factory := PgiFactory new.
	factory resourceManager: PgiRessourceManager new.
	factory world: PgiWorld new.
	^factory ! !

!PgiPenguinTest methodsFor: 'accessing' stamp: 'lsb 2/11/2011 18:06:21.094'!
spec
	"Answer the value of spec"

	^ spec! !

!PgiPenguinTest methodsFor: 'accessing' stamp: 'lsb 2/11/2011 18:06:21.17'!
spec: anObject
	"Set the value of spec"

	spec := anObject! !

!PgiPenguinTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/11/2011 19:06:21.328'!
testChangeStateTo
	
	| penguin startState startDirection|
	
	penguin := penguin := self setUp: #Penguin appearance: #PenguinWalkRight.
	startState := penguin state.
	startDirection := penguin state direction.
	
	penguin changeStateTo: (PgiParachuterWalking from: (penguin state)).
	
	self assert: ((startState = penguin state) not);
		 assert: (startDirection = penguin state direction).
	! !

!PgiPenguinTest methodsFor: 'as yet unclassified' stamp: 'rs 2/19/2011 01:39:38.562'!
testMoveBy
	
	| penguin startPos startPosMorph movementPoint|
	
	movementPoint := 10@10.
	penguin := penguin := self setUp: #Penguin appearance: #PenguinWalkRight.
	
	startPos := penguin position.
	startPosMorph := penguin morph position.
	
	penguin moveBy: movementPoint.
	
	self assert: ((startPos + movementPoint) = penguin position);
		 assert: ((startPosMorph + movementPoint) = penguin morph position).
		
	penguin moveBy: (-1 * movementPoint).
	
	self assert: (startPos  = penguin position);
		 assert: (startPosMorph = penguin morph position).
	! !

!PgiPenguinTest methodsFor: 'as yet unclassified' stamp: 'rs 2/19/2011 01:43:06.68'!
testReachGoal

	| penguin goal |
	
	penguin := self setUp: #Penguin appearance: #PenguinWalkRight.
	goal := self setUp: #Goal appearance: #standardGoal.
	penguin world goal: goal.
	
	penguin goal: goal.
	penguin position: penguin position + (0@50).
	self assert: (penguin reachedGoal).
	
	penguin moveBy: 30@30.
	self assert:(penguin reachedGoal not).! !

TestCase subclass: #PgiStepperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Test'!

!PgiStepperTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/4/2011 17:11:38.346'!
testStep
	| aStepper aObject|

	aStepper := PgiStepper new.
	aObject := PgiTestObject new.
	aStepper subscribe: (aObject) every: 1.
	aStepper step.			
	self assert: (aObject stepped).! !

!PgiStepperTest methodsFor: 'as yet unclassified' stamp: 'pre 1/24/2011 22:13:00.7'!
testSubscribe
	| aStepper |
	
	aStepper := PgiStepper new.
	self should: [aStepper subscribe: (Object new) every: (aStepper maxCycle + 1). ] raise: Error.
	aStepper pause.
	self assert: (aStepper == (aStepper subscribe: (Object new) every: 10)).! !

!PgiStepperTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/4/2011 16:24:17.001'!
testUnsubscribe
	| aStepper aObject |
	
	aStepper := PgiStepper new.
	aObject := Object new.
	aStepper subscribe: (aObject) every: 10.
	aStepper unsubscribe: (aObject).
	self assert: (aStepper toBeStepped size == 0).
	self should: [aStepper unsubscribe: aObject  ] raise: Error.! !

TestCase subclass: #PgiWellTest
	instanceVariableNames: 'spec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Test'!

!PgiWellTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/18/2011 18:06:34.627'!
setUp: testType appearance: testAppereance
"Creates a LevelObject according to testType and with testAppereance."

	|factory object |

	factory := self setUpFactory.

	self spec: PgiLevelObject new.
	self spec appearance: testAppereance;
			   type: testType;
			   position: 200@300;
			   edges: {0@0 . 50@0 . 50@50 . 0@50 }.
		
	object := factory build: self spec.
	^object! !

!PgiWellTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/18/2011 18:06:57.002'!
setUpFactory 
	
	| factory |
	
	factory := PgiFactory new.
	factory resourceManager: PgiRessourceManager new.
	factory world: PgiWorld new.
	^factory ! !

!PgiWellTest methodsFor: 'accessing' stamp: 'lsb 2/18/2011 18:11:26.673'!
spec
	"Answer the value of spec"

	^ spec! !

!PgiWellTest methodsFor: 'accessing' stamp: 'lsb 2/18/2011 18:11:26.759'!
spec: anObject
	"Set the value of spec"

	spec := anObject! !

!PgiWellTest methodsFor: 'as yet unclassified' stamp: 'lsb 2/18/2011 20:43:54.963'!
testPgiStep


	| aWell |
	
	aWell := self setUp: #Well appearance: #standardWell.
	aWell world gameState: (PgiGameStateLevel new).
	aWell world gameState menu: (PgiLevelMenu new).
	aWell world gameState menu scorePenguinsSpawned: (TextMorph new).
	aWell world gameState menu gameState: (aWell world gameState).
	aWell world gameState stepper pause.
	aWell world gameState stage: aWell world.
	aWell penguinBuilder: (self setUpFactory).
	aWell world penguinsToSpawn: 10.
	aWell world size: 800@600.

	self assert:( aWell world penguinsSpawned =0).
	aWell pgiStep.
	self assert:( aWell world penguinsSpawned=1).! !

Object subclass: #Penguins
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!Penguins commentStamp: 'RS 2/10/2011 23:58:16.277' prior: 0!
Penguins is the initiator Class for the Game itself.

Instance Variables
!

!Penguins class methodsFor: 'startPlaying' stamp: 'RS 12/9/2010 18:04:14.891'!
startGame

	Penguins new.! !

!Penguins methodsFor: 'initialize-release' stamp: 'kh 2/18/2011 11:19:04.084'!
initialize

	super initialize.
	PgiGame new.! !

Object subclass: #PgiCollision
	instanceVariableNames: 'type vector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic'!

!PgiCollision commentStamp: 'RS 2/11/2011 12:02:11.117' prior: 0!
A PgiCollision is a collison event in the form of an Object which is returned by the collision detection and can than be evaluated for further purposes.

Instance Variables
	type:		<Symbol>
	vector:		<Point>

type
	- is the kind of collsion that occured such as : bigObstacle, smallObstacle, mountainObstacle....

vector
	-  an advise in the form of a Point of the collision detection to move to.
!

!PgiCollision class methodsFor: 'instance creation' stamp: 'lsb 2/16/2011 02:06:19.183'!
newWithType: aType

	^ (self newWithVector:(0@0) ofType: aType)! !

!PgiCollision class methodsFor: 'instance creation' stamp: 'lsb 2/16/2011 02:06:26.097'!
newWithVector: aVector ofType: aType

	| collision |
	
	collision := (PgiCollision  new).
	collision  vector: aVector;
				type: aType.
				
	^collision! !

!PgiCollision methodsFor: 'initialize-release' stamp: 'pre 2/15/2011 10:04:30.36'!
initialize
	self vector: 0@0.
	super initialize.! !

!PgiCollision methodsFor: 'accessing' stamp: 'RS 2/3/2011 21:53:13.519'!
type
	"Answer the value of type"

	^ type! !

!PgiCollision methodsFor: 'accessing' stamp: 'RS 2/3/2011 21:53:13.525'!
type: anObject
	"Set the value of type"

	type := anObject! !

!PgiCollision methodsFor: 'accessing' stamp: 'RS 2/3/2011 21:53:13.529'!
vector
	"Answer the value of vector"

	^ vector! !

!PgiCollision methodsFor: 'accessing' stamp: 'RS 2/3/2011 21:53:13.533'!
vector: anObject
	"Set the value of vector"

	vector := anObject! !

Object subclass: #PgiCollisionDetectionMethod
	instanceVariableNames: 'computationDistance collisionEvent collidingMorphs collisionPointCalculations walkingCalculations deadlyMorphs penguin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic'!

!PgiCollisionDetectionMethod commentStamp: 'pre 2/19/2011 02:19:30.046' prior: 0!
A PgiCollisionDetectionMethod is a method object that encapsulates the collision detection calculation for penguins. 
The following fields have to be set to start it: penguin, computationDistance.
Then you can start it with compute.
It returns a PgiCollision.

Instance Variables
	collidingMorphs:		<Collection>
	collisionEvent:		<PgiCollision>
	collisionPointCalculations:		<Dictionary>
	computationDistance:		<signed Number>
	deadlyMorphs:		<Collection>
	penguin:		<PgiPenguin>
	walkingCalculations: <Dictionary>

collidingMorphs
	- the Collection of Morphs the penguin collides with.

collisionEvent
	- the collisionEvent which is assembeled in this method and returned.

collisionPointCalculations
	- a Dictionary of Blocks which determine the calculation of several collision points.

computationDistance
	- the Distance where collisions should be determined. The sign determines the direction.

deadlyMorphs
	- a Collection of Objects which would kill the penguin if they collide.

penguin
	- the penguin for which the collisions should be detected.
	
walkingCalculations
	- a Dictionary of Blocks which determine the calculation of the movement vector in several collision situations.
!

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:51'!
bottomLeft

	^((self topLeft) + (0@(self height)))! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:52'!
bottomMiddle

	^self bottomLeft + ((((0.5 * (self width)) rounded)@0))! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:52'!
bottomRight

	^(self topRight + (0@(self height)))! !

!PgiCollisionDetectionMethod methodsFor: 'collision-calculation' stamp: 'pre 2/19/2011 03:10:13.235'!
calculateClimbing
	
	| factor |
	
	factor := 0.
	factor := (1 to: (self mountainVector x))
				detect: [:each | (self collidingMorphs)
						anySatisfy: [:aMorph | (aMorph containsPoint: 
										(((self getEdgeFrom: self topMiddle to: (self computationDistance)) 
										+ ((self computationDistance)@0)) 
										+ (each * ((self computationDistance) sign)@0)))]].
	
	^ ((self computationDistance) sign)@0 * (factor)! !

!PgiCollisionDetectionMethod methodsFor: 'collision-calculation' stamp: 'pre 2/13/2011 23:45'!
calculateFalling
	^self gravityVector ! !

!PgiCollisionDetectionMethod methodsFor: 'collision-calculation' stamp: 'pre 2/15/2011 00:20:32.554'!
calculateHoveringCorrection

	| factor movementVector |
	
	movementVector := self collisionEvent vector.
	
	factor := 0.
	factor := (1 to: (self gravityVector y))
				detect: [:each | (self collidingMorphs)
						anySatisfy: [:aMorph | (aMorph containsPoint: ((self bottomLeft + movementVector ) + (each * self downVector )))
								or: (aMorph containsPoint: ((self bottomRight + movementVector ) + (each * self downVector )))]]
				ifNone: [0].
				
	(factor > 0) ifTrue: [factor := factor - 1].
	
	self collisionEvent vector: ((self collisionEvent vector) + (self downVector * (factor)))! !

!PgiCollisionDetectionMethod methodsFor: 'collision-calculation' stamp: 'pre 2/15/2011 00:14:16.667'!
calculateSmallWalking
	"This calculates the maximum vector to the obstacle. The first sumand in the central calculation is just the right edge (left or right). Then we add a test vector and the already calculated vector."
	
	| factor movementVector |
	
	movementVector := self collisionEvent vector.
	
	factor := 0.
	factor := (1 to: (self computationDistance))
				detect: [:each | (self collidingMorphs)
						anySatisfy: [:aMorph | (aMorph containsPoint: 
							((self getEdgeFrom: (self topMiddle) to: (self computationDistance)) 
								+ movementVector 
								+ ((each * ((self computationDistance) sign)) * (self collisionVector) ))) 
							or:
							(aMorph containsPoint: 
							((self getEdgeFrom: (self criticalPoint ) to: (self computationDistance)) 
								+ movementVector 
								+ ((each * ((self computationDistance) sign)) * (self collisionVector) ))) 
							or:
							(aMorph containsPoint: 
							((self getEdgeFrom: (self bottomMiddle ) to: (self computationDistance)) 
								+ movementVector 
								+ ((each * ((self computationDistance) sign)) * (self collisionVector) ))) ]]
				ifNone: [0].
				
	(factor > 0) ifTrue: [factor := factor - 1].
	
	self collisionEvent vector: (self collisionEvent vector + (self collisionVector  * (factor))).! !

!PgiCollisionDetectionMethod methodsFor: 'collision-calculation' stamp: 'pre 2/19/2011 02:29:02.142'!
calculateWalking
	| collisionSymbol |
	
	collisionSymbol := (self checkCollision).
	
	self collisionEvent type: collisionSymbol.
	
	(self walkingCalculations at: collisionSymbol) value.
	! !

!PgiCollisionDetectionMethod methodsFor: 'collision-calculation' stamp: 'pre 2/14/2011 23:53:32.576'!
checkBitingTheDust
	
	^ self collidingUndergroundsIn: (self deadlyMorphs)! !

!PgiCollisionDetectionMethod methodsFor: 'collision-calculation' stamp: 'pre 2/19/2011 03:09:56.116'!
checkCollision

	| resultSymbol |

	resultSymbol := #noObstacle.

	self collidingMorphs do:
	[ :aMorph | 
		((aMorph containsPoint: (self collisionPoint: #bottom inDirection: (self computationDistance) )))
			ifTrue: [resultSymbol := #smallObstacle]. 
	
		((aMorph containsPoint: (self collisionPoint: #bottom inDirection: (self computationDistance) )) 
		and: (aMorph containsPoint: (self collisionPoint: #mountain inDirection: (self computationDistance)))) 
			ifTrue: [resultSymbol := #mountainObstacle]. 
	
		(((aMorph containsPoint: (self collisionPoint: #top inDirection: (self computationDistance))) or:
		(aMorph containsPoint: (self collisionPoint: #middle inDirection: (self computationDistance)))) )
			ifTrue: [resultSymbol := #bigObstacle].].
	
	^ resultSymbol 
! !

!PgiCollisionDetectionMethod methodsFor: 'collision-calculation' stamp: 'pre 2/14/2011 23:42:42.174'!
checkFalling
	^ (self collidingUndergroundsIn: (self collidingMorphs)) not
! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/14/2011 23:44:09.396'!
collidingMorphs
	"Answer the value of collidingMorphs"

	^ collidingMorphs! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/14/2011 23:44:09.408'!
collidingMorphs: anObject
	"Set the value of collidingMorphs"

	collidingMorphs := anObject! !

!PgiCollisionDetectionMethod methodsFor: 'collision-calculation' stamp: 'pre 2/18/2011 20:28:18.176'!
collidingUndergroundsIn: aMorphCollection

	^ (aMorphCollection
		anySatisfy: [:aMorph | (aMorph
				containsPoint: (self gravityPoint: #left))
				or: (aMorph
						containsPoint: (self gravityPoint: #right))])! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/14/2011 23:58:16.613'!
collisionEvent
	"Answer the value of collisionEvent"

	^ collisionEvent! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/14/2011 23:58:16.624'!
collisionEvent: anObject
	"Set the value of collisionEvent"

	collisionEvent := anObject! !

!PgiCollisionDetectionMethod methodsFor: 'collision-accessing' stamp: 'pre 2/19/2011 01:48:24.308'!
collisionPoint: aSymbol inDirection: aNumber

	^ (self collisionPointCalculations at: aSymbol) value: aNumber
! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/18/2011 20:37:31.116'!
collisionPointCalculations
	"Answer the value of collisionPointCalculations"

	^ collisionPointCalculations! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/18/2011 20:37:31.128'!
collisionPointCalculations: anObject
	"Set the value of collisionPointCalculations"

	collisionPointCalculations := anObject! !

!PgiCollisionDetectionMethod methodsFor: 'collision-accessing' stamp: 'pre 2/13/2011 23:47'!
collisionPointsInDirection: aNumber
	^ OrderedCollection withAll: {
		(self collisionPoint: #top inDirection: aNumber) . 
		(self collisionPoint: #middle inDirection: aNumber) . 
		(self collisionPoint: #bottom inDirection: aNumber) . 
		(self gravityPoint: #left) . 
		(self gravityPoint: #right) . 
	      (self collisionPoint: #mountain inDirection: aNumber) . 
		(self downPoint: #left) . 
		(self downPoint: #right)}! !

!PgiCollisionDetectionMethod methodsFor: 'collision-accessing' stamp: 'pre 2/13/2011 23:47'!
collisionVector
	"Answer the value of collisionVector"
	
	^ 1@0! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:47'!
computationDistance
	"Answer the value of computationDistance"

	^ computationDistance! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:47'!
computationDistance: anObject
	"Set the value of computationDistance"

	computationDistance := anObject! !

!PgiCollisionDetectionMethod methodsFor: 'methodObject-call' stamp: 'pre 2/13/2011 23:56'!
compute
	
	^self compute: (self computationDistance)! !

!PgiCollisionDetectionMethod methodsFor: 'methodObject-call' stamp: 'pre 2/19/2011 02:22:42.452'!
compute: aNumber

	self collisionEvent: PgiCollision new.
	
	self collidingMorphs: (self world collisionObjects
							    select: [:aMorph | (self collisionPointsInDirection: aNumber) 
									     anySatisfy: [:aPoint | aMorph containsPoint: aPoint]]).
							
	self deadlyMorphs: (self world deadlyObjects
						    select: [:aMorph | self collidingMorphs includes: aMorph]).
	
	
	(self checkFalling)
		ifTrue:  [ self collisionEvent vector: (self calculateFalling).
				 self collisionEvent type:  #falling] 
		ifFalse: [ 
				self calculateWalking.
			
				(self checkBitingTheDust) ifTrue:[ self collisionEvent type: #bitingTheDust].

				 self calculateSmallWalking. 
				 self calculateHoveringCorrection.].
		
	^self collisionEvent ! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:47'!
criticalPoint
	"Answer the value of criticalPoint"
	
	^ (self topLeft + (((self width * (1/2)) rounded)@(self gravityVector y + 1)))! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/14/2011 23:32:14.499'!
deadlyMorphs
	"Answer the value of deadlyMorphs"

	^ deadlyMorphs! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/14/2011 23:32:14.515'!
deadlyMorphs: anObject
	"Set the value of deadlyMorphs"

	deadlyMorphs := anObject! !

!PgiCollisionDetectionMethod methodsFor: 'collision-accessing' stamp: 'pre 2/13/2011 23:48'!
downPoint: aSymbol

	| resultingPoint |
	
	(aSymbol = #left) ifTrue: [ resultingPoint := (self bottomLeft + (self downVector )) ].
	(aSymbol = #right) ifTrue: [ resultingPoint := (self bottomRight + (self downVector ))].
	
	^resultingPoint! !

!PgiCollisionDetectionMethod methodsFor: 'collision-accessing' stamp: 'pre 2/13/2011 23:48'!
downVector
	"Answer the value of downVector"

	^ 0@1! !

!PgiCollisionDetectionMethod methodsFor: 'collision-accessing' stamp: 'rs 2/18/2011 23:25:15.837'!
getEdgeFrom: aPoint to: aDirection

	^ ((aPoint x + (self width * 0.5 *(aDirection sign))) rounded)@((aPoint y) rounded)! !

!PgiCollisionDetectionMethod methodsFor: 'collision-accessing' stamp: 'pre 2/19/2011 03:26:47.618'!
gravityPoint: aSymbol

	^ self penguin gravityPoint: aSymbol
	! !

!PgiCollisionDetectionMethod methodsFor: 'collision-accessing' stamp: 'pre 2/19/2011 03:26:29.306'!
gravityVector
	^ self penguin gravityVector! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:51'!
height

	^ (self penguin height)! !

!PgiCollisionDetectionMethod methodsFor: 'initialize-release' stamp: 'pre 2/19/2011 03:12:08.659'!
initialize 

	self collisionPointCalculations: Dictionary new.
	self collisionPointCalculations 
		add: #top -> [:aNumber | (self getEdgeFrom: (self topMiddle) to: aNumber) 
									+ (self collisionVector * aNumber) ];
		add: #middle -> [:aNumber | ((self getEdgeFrom: (self criticalPoint) to: aNumber) 
										+ (self collisionVector * aNumber))];
		add: #bottom -> [:aNumber | ((self getEdgeFrom: (self bottomMiddle) to: aNumber) 
										+ (self collisionVector * aNumber))];
		add: #mountain -> [:aNumber | ((self getEdgeFrom: (self topMiddle) to: aNumber) 
										+ (self mountainVector * (aNumber sign)))].
		
	self initializeWalkingCalculationsDict.! !

!PgiCollisionDetectionMethod methodsFor: 'initialize-release' stamp: 'pre 2/19/2011 03:11:47.547'!
initializeWalkingCalculationsDict
	
	self walkingCalculations: Dictionary new.
	(self walkingCalculations)
		add: #bigObstacle -> [self calculateSmallWalking];
		add: #smallObstacle -> [self collisionEvent vector: ((((self collisionVector x) * (self computationDistance))
															@((self collisionVector y) -  (self gravityVector y))))];
		add: #noObstacle -> [ self collisionEvent vector: ((((self collisionVector x) * (self computationDistance))
															@(self collisionVector y)))];
		add: #mountainObstacle-> [ |topEdge bottomEdge |
									topEdge := (self getEdgeFrom: self topMiddle to: (self computationDistance)).
									bottomEdge := (self getEdgeFrom: (self bottomMiddle) to: (self computationDistance)).
									self calculateClimbing. 
									self collisionEvent vector:  topEdge 
																+ (self collisionEvent vector)
																- bottomEdge ]. 
! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:52'!
midPoint
	"Returns the midPoint of PgiWorldObjects grafikal representation."
	| width height |
	
	width := ((self width) / 2) rounded.
	height :=((self height) / 2) rounded.
	
	^ (self penguin position + (width@height))
	! !

!PgiCollisionDetectionMethod methodsFor: 'collision-accessing' stamp: 'rs 2/18/2011 15:48:23.858'!
mountainVector
	"Answer the value of mountainVector"

	^13@0! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/14/2011 23:59:57.958'!
movementVector
	(self collisionEvent) ifNil: [ 
		self collisionEvent: (PgiCollision new vector: 0@0). ].
	
	^ self collisionEvent vector! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/15/2011 00:01:03.879'!
movementVector: aPoint
	(self collisionEvent) ifNil: [ 
		self collisionEvent: (PgiCollision new). ].
	
	self collisionEvent vector: aPoint.! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:47'!
penguin
	"Answer the value of penguin"

	^ penguin! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:47'!
penguin: anObject
	"Set the value of penguin"

	penguin := anObject! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:53'!
topLeft 

	^(self penguin position )! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:53'!
topMiddle

	^ (self topLeft + (((0.5 * (self width)) rounded)@0))! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:53'!
topRight

	^((self topLeft ) + ((self width)@0))! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/19/2011 02:12:52.847'!
walkingCalculations
	"Answer the value of walkingCalculations"

	^ walkingCalculations! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/19/2011 02:12:52.858'!
walkingCalculations: anObject
	"Set the value of walkingCalculations"

	walkingCalculations := anObject! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:51'!
width

	^ (self penguin width)! !

!PgiCollisionDetectionMethod methodsFor: 'accessing' stamp: 'pre 2/13/2011 23:53'!
world
	"Answer the value of world"

	^ self penguin world! !

Object subclass: #PgiFactory
	instanceVariableNames: 'resourceManager world objectLookupDict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiFactory commentStamp: 'RS 2/11/2011 00:10:24.911' prior: 0!
A PgiFactory is a Product trader for all kinds of Objects that are dynamicly constructed at runtime or at the start of a level.

Instance Variables
	objectLookupDict:		<Object>
	resourceManager:		<Object>
	world:		<Object>

objectLookupDict
	- this dict looks up the appropriate codeblock to construct the desired Object, which is specified by a given Specification (PgiLevelObject)

resourceManager
	- reference to the ressource manager from whom the trader can recieve all the needed images.

world
	- the stage (usually a world in case of a running game), on which he is supposed to add and register the requested objects.
!

!PgiFactory class methodsFor: 'creation' stamp: 'kh 2/18/2011 11:19:28.987'!
newOn: aWorld withRecourceManager: aRecMan

	| resultFactory |
	
	resultFactory := self new.
	resultFactory world: aWorld;
				   resourceManager: aRecMan.
	
	^resultFactory ! !

!PgiFactory methodsFor: 'public' stamp: 'RS 2/11/2011 20:49:15.049'!
build: aLevelObject
	
	^((objectLookupDict at: aLevelObject type) value: aLevelObject)! !

!PgiFactory methodsFor: 'building WorldObjects' stamp: 'RS 2/11/2011 20:50:33.086'!
buildDeadlyObject: aDeadlyObject

	| aWorldObject |
	
	aWorldObject := self createDeadlyObject: aDeadlyObject.
	
	^(self buildMorphFor: aWorldObject)! !

!PgiFactory methodsFor: 'building Morphs' stamp: 'kh 2/18/2011 11:27:03.362'!
buildDecorationObject: aDecorationObject

	| resultMorph |
	
	resultMorph := ImageMorph new image: ((self resourceManager requestAppearance: (aDecorationObject appearance)) first).
	resultMorph position: ((self world visualPart position) + (aDecorationObject position));
				 beSticky.
				
	self world visualPart addMorph: resultMorph.
				
	^resultMorph
	! !

!PgiFactory methodsFor: 'building Wells and Goals' stamp: 'kh 2/18/2011 23:54:05.219'!
buildGoal: aLevelObject

	| aGoal aTexturedMorph|
	
	aGoal := self createGoal: aLevelObject.
	aTexturedMorph := self createWorldObjectMorphFrom: aGoal.
	
	aGoal morph: aTexturedMorph.
	aTexturedMorph worldObject: aGoal.
	
	self world goal: aGoal.
	self world visualPart addMorph: aGoal morph.
	self world registerWorldObject: aGoal.
	
	^aGoal	
	! !

!PgiFactory methodsFor: 'building WorldObjects' stamp: 'RS 2/11/2011 20:50:28.95'!
buildMorphFor: anObject

	|aWorldObjectMorph |

	aWorldObjectMorph := self createTexturedMorphFrom: anObject.
	anObject morph: aWorldObjectMorph.
	
	^anObject
	! !

!PgiFactory methodsFor: 'building penguins' stamp: 'rs 2/18/2011 20:55:25.936'!
buildPenguin: aLevelObject

	| penguinObject penguinObjectMorph |
	
	penguinObject := self createPenguinObject: aLevelObject.
	penguinObjectMorph := self createWorldObjectMorphFrom: penguinObject.
	penguinObject morph: penguinObjectMorph.
	penguinObjectMorph worldObject: penguinObject.
	
	^penguinObject! !

!PgiFactory methodsFor: 'building Wells and Goals' stamp: 'kh 2/18/2011 23:53:52.206'!
buildWell: aLevelObject

	| aWell aTexturedMorph |	
		
	aWell  := self createWell: aLevelObject.
	aTexturedMorph := self createWorldObjectMorphFrom: aWell.
	
	aWell morph: aTexturedMorph.
	aTexturedMorph worldObject: aWell.
	
	self world allObjects add: aWell.
	self world visualPart addMorph: aWell morph.
	self world registerCollisionObject: aWell.
	
	^aWell! !

!PgiFactory methodsFor: 'building WorldObjects' stamp: 'RS 2/11/2011 20:50:40.797'!
buildWorldObject: aLevelObject

	| aWorldObject |
	
	aWorldObject := self createWorldObject: aLevelObject.
	
	^(self buildMorphFor: aWorldObject)
	
	! !

!PgiFactory methodsFor: 'building WorldObjects' stamp: 'pre 2/13/2011 17:30:38.346'!
constructWorldObjectFrom: aLevelObject with: aClass

	| aWorldObject |

	aWorldObject := aClass within: self world.
	aWorldObject position: (aLevelObject position);
				    shape: (PolygonMorph new setVertices: (aLevelObject edges));
				    appearance: (aLevelObject appearance).
				
	^aWorldObject! !

!PgiFactory methodsFor: 'building WorldObjects' stamp: 'RS 2/11/2011 20:50:49.815'!
createDeadlyObject: aLevelObject

	| aDeadlyObject |
	
	aDeadlyObject := self createWorldObject: aLevelObject.
	self world registerDeadlyObject: aDeadlyObject.

	^aDeadlyObject! !

!PgiFactory methodsFor: 'building Wells and Goals' stamp: 'RS 2/11/2011 15:28:11.896'!
createGoal: aGoal

	| aWorldObject |
	
	aWorldObject := self constructWorldObjectFrom: aGoal with: PgiPenguinGoal.
				
	^aWorldObject! !

!PgiFactory methodsFor: 'building penguins' stamp: 'RS 2/11/2011 15:28:51.063'!
createPenguinObject: aLevelObject

	| aPenguin | 
	
	aPenguin := self constructWorldObjectFrom: aLevelObject with: PgiPenguin.
	
	^aPenguin ! !

!PgiFactory methodsFor: 'building WorldObjects' stamp: 'rs 2/18/2011 20:47:04.103'!
createTexturedMorphFrom: aWorldObject
	
	"This method creates a TexturedMorph counterpart to a given World Object"
	
	| resultMorph tempGraphic|
					
	tempGraphic := PgiGraphic new.
	tempGraphic ressourceManager: (self resourceManager).
	tempGraphic changeAppearance: aWorldObject appearance.
	
	resultMorph := PgiTexturedMorph newPgiTexturedMorphFromGraphic: tempGraphic Mask: aWorldObject shape.
	resultMorph 
		position: ((aWorldObject world) visualPart topLeft) + aWorldObject position;
		beSticky.
		
	self world visualPart addMorph: resultMorph.
	
	^resultMorph! !

!PgiFactory methodsFor: 'building Wells and Goals' stamp: 'RS 2/11/2011 15:28:31.443'!
createWell: aWell

	| aWorldObject |
	
	aWorldObject := self constructWorldObjectFrom: aWell with: PgiPenguinWell.
				
	^aWorldObject! !

!PgiFactory methodsFor: 'building WorldObjects' stamp: 'rs 2/18/2011 20:48:39.625'!
createWorldObject: aLevelObject

	"This method creates the logical part of a World Object"
	
	| aWorldObject |
	
	aWorldObject := self constructWorldObjectFrom: aLevelObject with: PgiWorldObject.		
	self world registerWorldObject: aWorldObject.
	self world registerCollisionObject: aWorldObject.
				
	^aWorldObject
	
	! !

!PgiFactory methodsFor: 'building WorldObjects' stamp: 'rs 2/18/2011 20:49:02.617'!
createWorldObjectMorphFrom: aWorldObject
	
	"This method creates a WorldObjectMorph counterpart to a given World Object"
	
	| resultMorph tempGraphic|
					
	tempGraphic := PgiGraphic new.
	tempGraphic ressourceManager: (self resourceManager).
	tempGraphic changeAppearance: aWorldObject  appearance.
	resultMorph := PgiWorldObjectMorph newPgiWorldObjectMorphWithGraphics: tempGraphic..
	resultMorph 
		position: ((aWorldObject world) visualPart topLeft) + aWorldObject position;
		beSticky.
	self world visualPart addMorph: resultMorph.
	
	^resultMorph! !

!PgiFactory methodsFor: 'initializing' stamp: 'RS 2/11/2011 20:49:06.787'!
initialize 

	super initialize.
	
	"initialize the dict that looks up the methods to call for each type of Object"
	
	objectLookupDict := Dictionary newFrom: {  #Penguin -> [:aLevelObject | self buildPenguin: aLevelObject] .
												  #Goal -> [:aLevelObject | self buildGoal: aLevelObject] . 
												  #Well -> [:aLevelObject | self buildWell: aLevelObject] .
												  #Standard -> [:aLevelObject | self buildWorldObject: aLevelObject] .
												  #Deadly ->[:aLevelObject | self buildDeadlyObject: aLevelObject].
												  #Decorator -> [:aLevelObject | self buildDecorationObject: aLevelObject]} ! !

!PgiFactory methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:28:16.472'!
resourceManager

	^ resourceManager! !

!PgiFactory methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:28:27.066'!
resourceManager: aPgiResourceManager

	resourceManager := aPgiResourceManager! !

!PgiFactory methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:28:34.199'!
world

	^ world! !

!PgiFactory methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:28:41.099'!
world: aPgiWorld

	world := aPgiWorld! !

Object subclass: #PgiGameState
	instanceVariableNames: 'game stateSymbol menu stage stepper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiGameState commentStamp: 'pre 12/23/2010 11:45:39.407' prior: 0!
A PgiGameState is a simple message Object to transfer state information in one atomary unit.

Instance Variables
	menu:		PgiMenu
	stage:		PgiStage
	stateSymbol:		aSymbol

menu
	- xxxxx

stage
	- xxxxx

stateSymbol
	- xxxxx
!

!PgiGameState methodsFor: 'creation and cleanup' stamp: 'kh 1/19/2011 01:29:14.458'!
abandon

	self stepper abandon.
	self stage abandon.
	self menu abandon.
	! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 18:45:27.873'!
bottomLeft

	^self game bottomLeft! !

!PgiGameState methodsFor: 'transition' stamp: 'rs 2/16/2011 16:55:19.218'!
changeState: aStateClass

	self game changeState: aStateClass.! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 19:24:34.206'!
closeGame 

	self game closeGame.! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 15:58:13.402'!
extent

	^self game extent! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:30:21.962'!
game

	^ game! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:31:35.828'!
game: aPgiGame

	game := aPgiGame! !

!PgiGameState methodsFor: 'initialize-release' stamp: 'kh 1/19/2011 01:41:03.867'!
initialize 

	super initialize.	
	self stepper: PgiStepper new.! !

!PgiGameState methodsFor: 'creation and cleanup' stamp: 'lsb 1/15/2011 00:22:22.448'!
launch

	self stage launch.
	self menu launch.! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 19:05:14.186'!
levelNumber

	^self stage levelNumber! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:41:14.529'!
menu

	^ menu! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:41:23.063'!
menu: aPgiMenu

	menu := aPgiMenu! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 15:58:30.238'!
position

	^self game position! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 16:07:39.005'!
requestAppearance: anAppearance

	^self game ressourceManager requestAppearance: anAppearance! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 16:13:14.126'!
requestHomeButtonsFor: aMenu

	^self ressourceManager requestHomeButtonsFor: aMenu! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 16:14:20.194'!
requestLevelButtonsFor: aMenu

	^self ressourceManager requestLevelButtonsFor: aMenu! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 19:23:22.208'!
requestLevelStartButtonsFor: aMenu

	^self ressourceManager requestLevelStartButtonsFor: aMenu! !

!PgiGameState methodsFor: 'delegating' stamp: 'kh 2/16/2011 02:56:36.558'!
requestToolButtonsFor: aMenu

	^self ressourceManager requestToolButtonsFor: aMenu! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 19:18:36.217'!
requestToolUsagesFor: aLevelNumber

	^self ressourceManager requestToolUsagesFor: aLevelNumber ! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 18:48:06.317'!
requestWorldButtonsFor: aMenu

	^self ressourceManager requestLevelButtonsFor: aMenu! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:41:28.251'!
ressourceManager 

	 ^ self game ressourceManager ! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:41:32.205'!
stage

	^ stage! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:41:39.941'!
stage: aPgiStage

	stage := aPgiStage! !

!PgiGameState methodsFor: 'delegating' stamp: 'RS 2/11/2011 19:20:26.515'!
startLevel: aLevelNumber

	self game startLevel: aLevelNumber.! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:42:36.935'!
stateSymbol

	^ stateSymbol! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:42:51.332'!
stateSymbol: anObject

	stateSymbol := anObject! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:43:15.193'!
stepper

	^ stepper! !

!PgiGameState methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:43:26.377'!
stepper: aPgiStepper

	stepper := aPgiStepper! !

PgiGameState subclass: #PgiGameStateFinalize
	instanceVariableNames: 'successful finalizedLevel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiGameStateFinalize commentStamp: 'rs 2/18/2011 21:48:09.563' prior: 0!
A PgiGameStateFinalize is the tate for the Screen that shows up after the user has quit a level.

Instance Variables
	finalizedLevel:		<aNumber>
	successful:		<aBool>

finalizedLevel
	- the Level that the user did stopped playing.

successful
	- determines wether the user has played through the level sucessfully i.e. he saved all the penguins.
!

!PgiGameStateFinalize class methodsFor: 'creation' stamp: 'kh 2/18/2011 10:59:22.246'!
newFinalizeStateFor: aGame with: aLevelNumber successful: aBoolean

	| resultLoadingState |
	
	resultLoadingState  := self new.
	resultLoadingState  game: aGame.
	
	resultLoadingState successful: aBoolean;
						finalizedLevel: aLevelNumber.
	
	resultLoadingState menu initializeMenu.
	resultLoadingState stage initializeStage.
	
      ^resultLoadingState! !

!PgiGameStateFinalize methodsFor: 'accessing' stamp: 'kh 2/18/2011 10:58:56.104'!
finalizedLevel

	^ finalizedLevel! !

!PgiGameStateFinalize methodsFor: 'accessing' stamp: 'kh 2/18/2011 10:59:02.251'!
finalizedLevel: aNumber

	finalizedLevel := aNumber! !

!PgiGameStateFinalize methodsFor: 'initialize-release' stamp: 'rs 2/16/2011 15:41:03.326'!
initialize 

	super initialize.
	
	self stateSymbol: #Finalize.
	self menu: ((PgiFinalizeMenu new) gameState: self).
	self stage: ((PgiFinalizeStage new) gameState: self).! !

!PgiGameStateFinalize methodsFor: 'creation and cleanup' stamp: 'rs 2/16/2011 15:39:59.997'!
launch

	super launch.! !

!PgiGameStateFinalize methodsFor: 'delegation' stamp: 'kh 2/18/2011 11:05:50.805'!
openHomeScreen 

	self game openHomeScreen.! !

!PgiGameStateFinalize methodsFor: 'initialize-release' stamp: 'kh 2/18/2011 11:03:19.966'!
restartLevel 

	self startLevel: self finalizedLevel.! !

!PgiGameStateFinalize methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:04:37.498'!
startNextLevel

	(self finalizedLevel < self game highestPossibleLevel) ifTrue: [ self startLevel: self finalizedLevel +1]
															ifFalse: [self startLevel: self finalizedLevel].! !

!PgiGameStateFinalize methodsFor: 'accessing' stamp: 'kh 2/18/2011 10:55:10.177'!
successful

	^ successful! !

!PgiGameStateFinalize methodsFor: 'accessing' stamp: 'kh 2/18/2011 10:55:19.441'!
successful: aBoolean

	successful := aBoolean! !

PgiGameState subclass: #PgiGameStateHome
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiGameStateHome commentStamp: 'RS 2/11/2011 00:19:23.276' prior: 0!
A PgiGameStateHome is the Game state in which the user can select between all possible levels and start the desired one.

Instance Variables
!

!PgiGameStateHome class methodsFor: 'creation' stamp: 'rs 2/16/2011 15:55:39.251'!
newHomeStateFor: aGame with: aLevelNumber

	| resultHomeState |
	
	resultHomeState  := self new.
	resultHomeState  game: aGame.
	
	resultHomeState menu initializeMenu: aLevelNumber.
	resultHomeState stage initializeStage.
	
      ^resultHomeState! !

!PgiGameStateHome methodsFor: 'initialize-release' stamp: 'RS 2/11/2011 16:17:05.292'!
initialize 

	super initialize.
	
	self stateSymbol: #Home.
	self menu: ((PgiHomeMenu new) gameState: self).
	self stage: ((PgiHomeStage new) gameState: self).
	! !

PgiGameState subclass: #PgiGameStateLevel
	instanceVariableNames: 'scroller'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiGameStateLevel commentStamp: 'RS 2/11/2011 00:21:39.832' prior: 0!
A PgiGameStateLevel is the game state in which a level is currently running and the user interacts with the Penguins.

Instance Variables
	scroller:		<PgiScroller>
	selectedTool:		<Object>

scroller
	- the Object that is able to scroll the whole visual representation of the running game, so the user can reach every part of the map.

selectedTool
	- is the temporary selected tool that the user cann apply when he clicks on a Penguin. It can be changed by clicking a different toolButton.
!

!PgiGameStateLevel class methodsFor: 'creation' stamp: 'rs 2/16/2011 15:55:32.978'!
newLevelStateFor: aGame with: aLevelNumber

	| resultLevelState |
	
	resultLevelState  := self new.
	resultLevelState game: aGame;
					  loadLevel: aLevelNumber.
					
	resultLevelState menu initializeMenu: aLevelNumber.
	
      ^resultLevelState! !

!PgiGameStateLevel methodsFor: 'accessing stage' stamp: 'kh 2/17/2011 17:58:20.046'!
abandon

	super abandon.
	self scroller abandon.! !

!PgiGameStateLevel methodsFor: 'delegation' stamp: 'lsb 2/16/2011 16:57:45.821'!
finalizeLevel 

	| levelSuccessful |
	
	levelSuccessful := self penguinsRescued >= self penguinsToRescue.
	
	self game endLevel: self levelNumber successful: levelSuccessful.! !

!PgiGameStateLevel methodsFor: 'initialize-release' stamp: 'RS 2/11/2011 16:20:27.229'!
initialize 

	super initialize.
	
	self stateSymbol: #Level;
		 menu: (PgiLevelMenu new gameState: self).

	! !

!PgiGameStateLevel methodsFor: 'creation and cleanup' stamp: 'rs 2/18/2011 23:51:34.046'!
launch 

	| scrollingFrequency |
	
	scrollingFrequency := 2.
	
	super launch.
	
	self scroller: (PgiScroller newPgiScrollerOn: (self game) withExtent: (self game) extent at: (self game topLeft)). 
	(self stepper) subscribe: self scroller every: scrollingFrequency;
				    startStepping.
! !

!PgiGameStateLevel methodsFor: 'creation and cleanup' stamp: 'kh 2/18/2011 00:46:31.185'!
loadLevel: aLevelNumber 

	self stage: (((PgiWorldBuilder new resourceManager: self ressourceManager)
				gameState: self)
				buildWorld: aLevelNumber).

! !

!PgiGameStateLevel methodsFor: 'accessing' stamp: 'kh 1/18/2011 21:53:35.706'!
pauseGame

	self stepper pause.! !

!PgiGameStateLevel methodsFor: 'accessing stage' stamp: 'kh 2/16/2011 05:16:26.149'!
penguinsRescued
	
	^self stage penguinsRescued! !

!PgiGameStateLevel methodsFor: 'accessing stage' stamp: 'kh 1/6/2011 16:57:56.051'!
penguinsSpawned

	^self stage penguinsSpawned! !

!PgiGameStateLevel methodsFor: 'accessing stage' stamp: 'kh 2/16/2011 05:16:38.321'!
penguinsToRescue

	^self stage penguinsToRescue! !

!PgiGameStateLevel methodsFor: 'accessing stage' stamp: 'kh 1/6/2011 16:58:08.234'!
penguinsToSpawn

	^self stage penguinsToSpawn! !

!PgiGameStateLevel methodsFor: 'initialize-release' stamp: 'kh 1/18/2011 22:19:19.05'!
playGame 

	self stepper resume.! !

!PgiGameStateLevel methodsFor: 'initialize-release' stamp: 'kh 1/18/2011 22:19:37.297'!
runFaster 

	self stepper speedUp.! !

!PgiGameStateLevel methodsFor: 'accessing' stamp: 'kh 2/18/2011 00:22:19.091'!
scrollablePart 

	^ self stage visualPart! !

!PgiGameStateLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:47:29.459'!
scroller

	^ scroller! !

!PgiGameStateLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:47:38.64'!
scroller: aPgiScroller

	scroller := aPgiScroller! !

!PgiGameStateLevel methodsFor: 'accessing' stamp: 'kh 2/10/2011 17:42:46.232'!
selectedTool

	^ self menu selectedTool! !

!PgiGameStateLevel methodsFor: 'accessing' stamp: 'kh 2/10/2011 17:24:05.216'!
toolHasBeenApplied: aTool

	self menu toolHasBeenApplied: aTool.! !

!PgiGameStateLevel methodsFor: 'accessing' stamp: 'pre 2/13/2011 18:00:16.151'!
toolHasBeenSelected: aTool

	self menu toolHasBeenSelected: aTool.
! !

!PgiGameStateLevel methodsFor: 'delegation' stamp: 'RS 2/11/2011 15:47:28.643'!
unsubscribe: aPenguin

	self stepper unsubscribe: aPenguin.! !

!PgiGameStateLevel methodsFor: 'delegation' stamp: 'kh 2/16/2011 05:15:30.838'!
updateNumberOfRescuedPenguins

	self menu updateNumberOfRescuedPenguins.! !

!PgiGameStateLevel methodsFor: 'delegation' stamp: 'RS 2/11/2011 15:45:39.877'!
updateNumberOfSpawnedPenguins

	self menu updateNumberOfSpawnedPenguins.! !

PgiGameState subclass: #PgiGameStateLoading
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiGameStateLoading commentStamp: 'RS 2/11/2011 00:22:14.606' prior: 0!
A PgiGameStateLoading is the Loading Screen that is being displayed while the Level is set up.

Instance Variables
!

!PgiGameStateLoading class methodsFor: 'creation' stamp: 'rs 2/16/2011 15:55:25.369'!
newLoadingStateFor: aGame

	| resultLoadingState |
	
	resultLoadingState  := self new.
	resultLoadingState  game: aGame.
	
	resultLoadingState menu initializeMenu.
	resultLoadingState stage initializeStage.
	
      ^resultLoadingState! !

!PgiGameStateLoading methodsFor: 'initialize-release' stamp: 'RS 2/11/2011 16:20:13.278'!
initialize 

	super initialize.
	
	self stateSymbol: #Loading.
	self menu: ((PgiLoadingMenu new) gameState: self).
	self stage: ((PgiLoadingStage new) gameState: self).! !

Object subclass: #PgiGraphic
	instanceVariableNames: 'ressourceManager baseImages count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelMorphic'!

!PgiGraphic commentStamp: 'RS 2/11/2011 11:38:48.771' prior: 0!
A PgiGraphic is the Object assigned to a LevelObject that supplies it with correct Forms to build up its texture.

Instance Variables
	baseImages:		<OrderedCollection>
	count:		<Number>
	ressourceManager:		<PgiRessourceManager>

baseImages
	- The Collection of forms, that are needed by the LevelObject. In case of a penguin it contains mutliple entries on which the Penguin can iterate by the next: method.

count
	- xxxxx

ressourceManager
	- the recourceManger that supplies thet graphic with the requested form Objects.
!

!PgiGraphic methodsFor: 'accessing' stamp: 'pre 12/20/2010 15:56:42.859'!
baseImages
	"Answer the value of baseImage"

	^ baseImages! !

!PgiGraphic methodsFor: 'accessing' stamp: 'RS 12/22/2010 16:52:38.036'!
baseImages: anArray
	"Set the value of baseImage"

	baseImages := anArray.! !

!PgiGraphic methodsFor: 'graphic' stamp: 'pre 1/7/2011 15:45:51.17'!
changeAppearance: anAppearance
	
	self baseImages: ((self ressourceManager ) requestAppearance: anAppearance).
	self count: 1.! !

!PgiGraphic methodsFor: 'accessing' stamp: 'pre 12/20/2010 15:44:41.208'!
count
	"Answer the value of count"

	^ count! !

!PgiGraphic methodsFor: 'accessing' stamp: 'pre 12/20/2010 15:44:41.445'!
count: anObject
	"Set the value of count"

	count := anObject! !

!PgiGraphic methodsFor: 'initialize-release' stamp: 'tm 12/30/2010 03:38:21.83'!
initialize 
	super initialize.
	
	self 
		baseImages: (OrderedCollection new);
		count: 1.! !

!PgiGraphic methodsFor: 'graphic' stamp: 'pre 1/9/2011 20:22:46.384'!
next
	"This will return the next frame. In case of a texturedMorph the next frame is the texture itself."
	
	| returnForm |

	returnForm := (self baseImages ) at: count.
	count := ((count = (self baseImages size)) ifTrue: [1] ifFalse: [count + 1]).
	
	^returnForm ! !

!PgiGraphic methodsFor: 'accessing' stamp: 'pre 12/20/2010 15:44:40.392'!
ressourceManager
	"Answer the value of ressourceManager"

	^ ressourceManager! !

!PgiGraphic methodsFor: 'accessing' stamp: 'RS 12/22/2010 16:51:53.928'!
ressourceManager: aRecourceManager
	"Set the value of ressourceManager"

	ressourceManager := aRecourceManager.! !

Object subclass: #PgiHalo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic'!

!PgiHalo commentStamp: 'pre 2/19/2011 01:27:48.152' prior: 0!
A PgiHalo is a superclass for digging halos. It provides standard methods that stay the same with every orientation of the halo (vertically, horizontally).

Instance Variables
!

!PgiHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/19/2011 03:30:16.049'!
calculateShift: aPointCollection
	
	^((aPointCollection detectMin: [:each | each x] ) x) 
		@ ((aPointCollection detectMin: [:each | each y] ) y)! !

!PgiHalo methodsFor: 'diggingHelpers' stamp: 'lsb 2/16/2011 17:06:29.523'!
underground 

	self subclassResponsibility.! !

!PgiHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/19/2011 01:30:17.132'!
undergroundVerticesFromIndex: startIndex toIndex: endIndex

	| resultVertices |
	
	resultVertices := OrderedCollection new.

	(endIndex > startIndex ) ifTrue: [
		startIndex to: endIndex do: [:each | 
			resultVertices add: (self underground vertices at: each)]]
	ifFalse: [
		startIndex to: (self underground vertices size)
		do: [ :each |
			resultVertices add: (self underground vertices at: each)].
		(startIndex > 1) ifTrue: [
			1 to: endIndex do: [:each | 
				resultVertices add: (self underground vertices at: each)]]].
	
	^resultVertices! !

!PgiHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/19/2011 00:15:11.412'!
vectorToIntersectionInto: aPolygon from: aPoint in: aDirection until: aMaximumFactor
	
	| testFactor |
	
	testFactor := ( 1 to: aMaximumFactor ) detect: [ :each | 	
			(aPolygon containsPoint: (aPoint + (aDirection * each)))].
	^(aDirection * testFactor )! !

!PgiHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/19/2011 03:12:42.611'!
vectorToIntersectionOutOf: aPolygon from: aPoint in: aDirection until: aMaximumFactor
	
	| testFactor |

	testFactor := ( 1 to: aMaximumFactor ) detect: [ :each | 
		(aPolygon containsPoint: (aPoint + (aDirection * each)))  not ].
	^(aDirection * testFactor )! !

PgiHalo subclass: #PgiMineHalo
	instanceVariableNames: 'mine top bottom world underground leftVertices rightVertices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic'!

!PgiMineHalo commentStamp: 'pre 2/19/2011 01:26:26.353' prior: 0!
A PgiMineHalo is an abstraction of a mine. It can prepare the underground for digging through splitting it. It also deals with the mine itself (extending, finishing).
This class is very similar to PgiTunnelHalo. This is because a mine is a tunnel flipped by 90 degrees. Therefore the splitting algorithm changes fundamentally.

Instance Variables
	bottom:		<Dictionary>
	leftVertices:		<OrderedCollection>
	mine:		<OrderedCollection>
	rightVertices:		<OrderedCollection>
	top:		<Dictionary>
	underground:		<PgiWorldObject>
	world:		<PgiWorld>

bottom
	a Dictionary that stores the bottom vertices of the tunnel. Keys: #left, #right.

leftVertices
	a Collection that stores all the vertices for the left side of the splitted Underground.

mine
	a Collection of vertices that make the mine.

rightVertices
	a Collection that stores all the vertices for the right side of the splitted Underground.

top
	a Dictionary that stores the top vertices of the tunnel. Keys: #left, #right.

underground
	the ground which the mine is going through.

world
	the world in which the mine ist digged.
!

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:23.674'!
bottom
	"Answer the value of bottom"

	^ bottom! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:23.683'!
bottom: anObject
	"Set the value of bottom"

	bottom := anObject! !

!PgiMineHalo methodsFor: 'digging' stamp: 'pre 2/19/2011 01:59:17.249'!
buildMineOn: aPolygon fromLeft: aLeftPoint toRight: aRightPoint

	self underground: aPolygon.

	"Here we catch whether the top was buildable or not."
	(self calculateMineTopFromLeft: aLeftPoint toRight: aRightPoint) ifFalse: [ ^false ].
	self calculateMineEnd.
	
	self calculateUndergroundHalfs.
	
	self splitUnderground.

	^true! !

!PgiMineHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/19/2011 03:31:05.713'!
calculateMineEnd
	| firstGuessMineEnd testFactorEndDetailVector vectorToIntersection variance stepLength |
	
	variance := 50.
	stepLength := 10.
	
	{ #left . #right} do: [ :eachSide |
		vectorToIntersection := self vectorToIntersectionOutOf: (self underground) 
									from: (self top at: eachSide)
									in: (0@stepLength)
									until: (((self underground height + variance) / stepLength) rounded). 
									
		firstGuessMineEnd := (self top at: eachSide) + vectorToIntersection.
		
		testFactorEndDetailVector := self vectorToIntersectionInto: (self underground) 
										from: firstGuessMineEnd
										in: (0@(-1))
										until: stepLength.
		
		self bottom at: eachSide put: (firstGuessMineEnd + testFactorEndDetailVector). ].! !

!PgiMineHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/19/2011 03:23:02.318'!
calculateMineTopFromLeft: aLeftPoint toRight: aRightPoint
	| vectorToIntersection mineHorizontalExtension |

	mineHorizontalExtension := 5.

	"If checks whether its possible to dig here"
	((self underground containsPoint: ((aRightPoint) + (0@10))) and:
	(self underground containsPoint: ((aLeftPoint) + (0@10)))) 
	ifTrue: [ vectorToIntersection := self vectorToIntersectionInto: (self underground) from: aLeftPoint in: (0@1) until: 20. 
		self top at: #left put: (aLeftPoint + vectorToIntersection - (mineHorizontalExtension@0)).
	
		vectorToIntersection := self vectorToIntersectionInto: (self underground) from: aRightPoint in: (0@1) until: 20. 
		self top at: #right put: (aRightPoint + vectorToIntersection + (mineHorizontalExtension@0)).
		^true ] 
	ifFalse: [ ^false ].! !

!PgiMineHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/12/2011 17:06:23.984'!
calculateUndergroundHalfs
	| endVertexIndex startVertexIndex |
	
	startVertexIndex := (self closeSegmentFromPoint: (self top at: #right ) To:  1@0 ).
	endVertexIndex := (self closeSegmentFromPoint: (self bottom at: #right) To: 1@0 ).
	self rightVertices add: (self top at: #right ).
	self rightVertices addAll: (self undergroundVerticesFromIndex: startVertexIndex toIndex: endVertexIndex ).
	self rightVertices add: (self bottom at: #right ).
		
	startVertexIndex := (self closeSegmentFromPoint: (self bottom at: #left) To: (-1)@0 ).
	endVertexIndex := (self closeSegmentFromPoint: (self top at: #left ) To:  (-1)@0 ).
	self leftVertices add: (self bottom at: #left ).
	self leftVertices addAll: (self undergroundVerticesFromIndex: startVertexIndex toIndex: endVertexIndex ).
	self leftVertices add: (self top at: #left ).! !

!PgiMineHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/12/2011 17:06:24.037'!
closeSegmentFromPoint: aPoint To: aDirection
	^ self closeSegmentFromPoint: aPoint To: aDirection onPolygon: (self underground)! !

!PgiMineHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/18/2011 21:05:10.065'!
closeSegmentFromPoint: aPoint To: aDirection onPolygon: aPolygon
	| vertexIndex nextVertexIndex previousVertexIndex resultVertexIndex variance |
	
	vertexIndex := aPolygon closestSegmentTo: aPoint.
	nextVertexIndex := (((vertexIndex)  = (aPolygon vertices size)) ifTrue: [1] ifFalse: [vertexIndex + 1]).
	previousVertexIndex := (((vertexIndex)  = 1) ifTrue: [aPolygon vertices size] ifFalse: [vertexIndex -1]).
	
	variance := 5.
	
	resultVertexIndex := {vertexIndex . nextVertexIndex . previousVertexIndex} detect: [:eachVertexIndex |
			((((aPolygon vertices at: eachVertexIndex) x - (aPoint x)) sign = (aDirection x sign)))
		] ifNone: [
			resultVertexIndex := {vertexIndex . nextVertexIndex . previousVertexIndex} detect: [ :eachVertexIndex |
				((((aPolygon vertices at: eachVertexIndex) y - (aPoint y - variance)) sign = (aDirection y sign))). ] ].

	^resultVertexIndex! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:24.139'!
direction
	^0@9! !

!PgiMineHalo methodsFor: 'digging' stamp: 'pre 2/19/2011 03:12:58.995'!
doesMineEndInDepth: aDifference

	^(((self top at: #left) y) > ((self bottom at: #left) y)) 
		or: (((self top at: #right) y) > ((self bottom at: #right) y))! !

!PgiMineHalo methodsFor: 'digging' stamp: 'pre 2/19/2011 02:00:17.442'!
extendBy: aDifference

	| tempVertices tempTopDict |

	(self doesMineEndInDepth: aDifference )
		ifTrue: [ self finishMine. ] 
		ifFalse: [
	
			tempTopDict := Dictionary new.
			self top keysAndValuesDo: [ :aKey :aValue | tempTopDict at: aKey put: (aValue + (0@aDifference))].
			self top: tempTopDict.
			
			tempVertices := OrderedCollection new.
			tempVertices  add: (self top at: #left);
							add: (self top at: #right);
							add: (self bottom at: #right);
							add: (self bottom at: #left).
			self mine updateShape: tempVertices withShift: 0@aDifference. 
			]
	
	
	 ! !

!PgiMineHalo methodsFor: 'digging' stamp: 'pre 2/16/2011 19:12:17.284'!
finishMine

	self world unregisterCollisionObject: (self mine).
	self mine abandon.! !

!PgiMineHalo methodsFor: 'initialize-release' stamp: 'pre 2/18/2011 17:32:48.716'!
initialize
	super. 
	self top: Dictionary new;
		bottom: Dictionary new;
		leftVertices: OrderedCollection new;
		rightVertices: OrderedCollection new.
		 ! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:23.691'!
leftVertices
	"Answer the value of leftVertices"

	^ leftVertices! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:23.697'!
leftVertices: anObject
	"Set the value of leftVertices"

	leftVertices := anObject! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/16/2011 19:00:36.882'!
mine
	"Answer the value of mine"

	^ mine! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/16/2011 19:00:36.89'!
mine: anObject
	"Set the value of mine"

	mine := anObject! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:23.716'!
rightVertices
	"Answer the value of rightVertices"

	^ rightVertices! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:23.721'!
rightVertices: anObject
	"Set the value of rightVertices"

	rightVertices := anObject! !

!PgiMineHalo methodsFor: 'digging' stamp: 'pre 2/19/2011 01:04:36.598'!
splitUnderground

	| newMountainLevelObject shiftedVertices shift mineVertices |
	
	shiftedVertices := OrderedCollection new.
	shift := self calculateShift: (self leftVertices ).
	self leftVertices do: [:each | shiftedVertices add: (each - shift)].
	newMountainLevelObject := PgiLevelObject new.
	newMountainLevelObject appearance: (self underground appearance);
					    		   edges: ( shiftedVertices );
					               position: shift;
					    		   type: #Standard.
	((self world) factory build: newMountainLevelObject) .
	
	
	shiftedVertices := OrderedCollection new.
	shift := self calculateShift: (self rightVertices ).
	self rightVertices do: [:each | shiftedVertices add: (each - shift)].
	newMountainLevelObject := PgiLevelObject new.
	newMountainLevelObject appearance: (self underground appearance);
					    		   edges: ( shiftedVertices);
					               position: shift;
					    		   type: #Standard.
	((self world) factory build: newMountainLevelObject).


	mineVertices := OrderedCollection new.
	mineVertices  add: (self top at: #left);
					add: (self top at: #right);
					add: (self bottom at: #right);
					add: (self bottom at: #left).
	shift := self calculateShift: ( mineVertices ).
	shiftedVertices := OrderedCollection new.
	mineVertices do: [:each | shiftedVertices add: (each - shift)].
	newMountainLevelObject := PgiLevelObject new.
	newMountainLevelObject appearance: (self underground appearance);
					    		   edges: ( shiftedVertices);
					               position: shift;
					    		   type: #Standard.
	self mine: ((self world) factory build: newMountainLevelObject).
	
	self world unregisterCollisionObject: (self underground).
	self underground abandon.
	
! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:23.726'!
top
	"Answer the value of top"

	^ top! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:23.732'!
top: anObject
	"Set the value of top"

	top := anObject! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:23.738'!
underground
	"Answer the value of underground"

	^ underground! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:06:23.765'!
underground: anObject
	"Set the value of underground"

	underground := anObject! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:20:21.594'!
world
	"Answer the value of world"

	^ world! !

!PgiMineHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 17:20:21.603'!
world: anObject
	"Set the value of world"

	world := anObject! !

PgiHalo subclass: #PgiTunnelHalo
	instanceVariableNames: 'tunnel front end direction underground upperVertices lowerVertices world'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic'!

!PgiTunnelHalo commentStamp: 'pre 2/19/2011 01:19:12.409' prior: 0!
A PgiTunnelHalo is an abstraction of a tunnel. It can prepare the underground for digging through splitting it. It also deals with the tunnel itself (extending, finishing).

Instance Variables
	direction:		<Point>
	end:		<Directory>
	front:		<Directory>
	lowerVertices:		<OrderedCollection>
	tunnel:		<PgiTunnelHalo>
	underground:		<PgiWorldObject>
	upperVertices:		<OrderedCollection>
	world:		<PgiWorld>

direction
	stores the direction in which the tunnel extends.

end
	is a Dictionary of the two endpoints. Keys: #top, #bottom.

front
	is a Dictionary of the two frontpoint. Keys: #top, #bottom.

lowerVertices
	stores all Vertices for the lower part of the underground after splitting. 

tunnel
	stores the vertices of the tunnel itself.

underground
	the ground which the tunnel is going through.

upperVertices
	stores all Vertices for the upper part of the underground after splitting. 

world
	the world in which the tunnel ist digged.
!

!PgiTunnelHalo methodsFor: 'digging' stamp: 'pre 2/19/2011 01:58:43.288'!
buildTunnelOn: aPolygon withTop: aTopPoint withBottom: aBottomPoint

	self underground: aPolygon.

	"Here we catch the case when the front is not buildable."
	(self calculateTunnelFrontWithTop: aTopPoint andBottom: (aBottomPoint-(0@2))) ifFalse: [ ^false ].
	self calculateTunnelEnd.
	
	self calculateUndergroundHalfs.

	self splitUnderground.
	
	^true

! !

!PgiTunnelHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/19/2011 03:32:09.78'!
calculateTunnelEnd 

	| firstGuessTunnelEnd testFactorEndDetailVector vectorToIntersection stepLength variance |
	
	variance := 50.
	stepLength := 10.
	
	{ #top . #bottom } do: [ :eachSide |
	vectorToIntersection := self vectorToIntersectionOutOf: (self underground)
								from: ((self front at: eachSide))
								in: (((self direction x sign)*stepLength)@0)
								until: (((self underground width + variance) / stepLength) rounded).							
	firstGuessTunnelEnd := (self front at: eachSide) + vectorToIntersection.
	
	testFactorEndDetailVector := self vectorToIntersectionInto: (self underground)
									from: firstGuessTunnelEnd
									in: ((self direction x sign negated)@0)
									until: (2*stepLength).
	self end at: eachSide put: (firstGuessTunnelEnd + testFactorEndDetailVector).]
	! !

!PgiTunnelHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/19/2011 03:33:24.592'!
calculateTunnelFrontWithTop: aTopPoint andBottom: aBottomPoint
	| vectorToIntersection tunnelVerticalExtension distanceToUnderground |

	tunnelVerticalExtension := 5.
	distanceToUnderground := 20.

	"This if clause is needed because otherwise vectorToIntersection would break if theres nothing to intersect with."
	((self underground containsPoint: ( aBottomPoint + ((distanceToUnderground@0)*(self direction x sign)) ) ) 
	and: (self underground containsPoint: ( aTopPoint + ((distanceToUnderground@0)*(self direction x sign)) ))) 
		ifTrue: [vectorToIntersection := self vectorToIntersectionInto: (self underground) 
									from: aTopPoint
									in: ((self direction x sign)@0) 
									until: distanceToUnderground. 
			self front at: #top put: (aTopPoint + vectorToIntersection - (0@(tunnelVerticalExtension))).
	
			vectorToIntersection := self vectorToIntersectionInto: (self underground)
									from: (aBottomPoint)
									in: ((self direction x sign)@0) 
									until: distanceToUnderground.
			self front at: #bottom put: (aBottomPoint + vectorToIntersection). ] ifFalse: [ ^false ].	
	^true

	! !

!PgiTunnelHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/12/2011 14:46:47.854'!
calculateUndergroundHalfs

	| endVertexIndex startVertexIndex |
	
	(direction x sign > 0) ifTrue: [
	startVertexIndex := (self closeSegmentFromPoint: (self front at: #top ) To:  0@(-1) ).
	endVertexIndex := (self closeSegmentFromPoint: (self end at: #top) To: 0@(-1) ).
	self upperVertices add: (self front at: #top ).
	self upperVertices addAll: (self undergroundVerticesFromIndex: startVertexIndex toIndex: endVertexIndex ).
	self upperVertices add: (self end at: #top ).
		
	startVertexIndex := (self closeSegmentFromPoint: (self end at: #bottom) To: 0@(1) ).
	endVertexIndex := (self closeSegmentFromPoint: (self front at: #bottom ) To:  0@(1) ).
	self lowerVertices add: (self end at: #bottom ).
	self lowerVertices addAll: (self undergroundVerticesFromIndex: startVertexIndex toIndex: endVertexIndex ).
	self lowerVertices add: (self front at: #bottom ). ].

	(direction x sign < 0) ifTrue: [
	endVertexIndex := (self closeSegmentFromPoint: (self front at: #top ) To:  0@(-1) ).
	startVertexIndex := (self closeSegmentFromPoint: (self end at: #top) To: 0@(-1) ).
	self upperVertices add: (self end at: #top ).
	self upperVertices addAll: (self undergroundVerticesFromIndex: startVertexIndex toIndex: endVertexIndex ).
	self upperVertices add: (self front at: #top ).
		
	endVertexIndex := (self closeSegmentFromPoint: (self end at: #bottom) To: 0@(1) ).
	startVertexIndex := (self closeSegmentFromPoint: (self front at: #bottom ) To:  0@(1) ).
	self lowerVertices add: (self front at: #bottom ).
	self lowerVertices addAll: (self undergroundVerticesFromIndex: startVertexIndex toIndex: endVertexIndex ).
	self lowerVertices add: (self end at:#bottom ). ]! !

!PgiTunnelHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/6/2011 13:26:03.086'!
closeSegmentFromPoint: aPoint To: aDirection
	
	^self closeSegmentFromPoint: aPoint To: aDirection onPolygon: (self underground)
	
	! !

!PgiTunnelHalo methodsFor: 'diggingHelpers' stamp: 'pre 2/18/2011 21:04:38.572'!
closeSegmentFromPoint: aPoint To: aDirection onPolygon: aPolygon
	| vertexIndex nextVertexIndex previousVertexIndex resultVertexIndex variance |
	
	vertexIndex := aPolygon closestSegmentTo: aPoint.
	nextVertexIndex := (((vertexIndex)  = (aPolygon vertices size)) ifTrue: [1] ifFalse: [vertexIndex + 1]).
	previousVertexIndex := (((vertexIndex)  = 1) ifTrue: [aPolygon vertices size] ifFalse: [vertexIndex -1]).
	
	variance := 5.
	
	resultVertexIndex := {vertexIndex . nextVertexIndex . previousVertexIndex} detect: [:eachVertexIndex |
			((((aPolygon vertices at: eachVertexIndex) y - (aPoint y)) sign = (aDirection y sign)))
		] ifNone: [
	resultVertexIndex := {vertexIndex . nextVertexIndex . previousVertexIndex} detect: [ :eachVertexIndex |
				((((aPolygon vertices at: eachVertexIndex) y - (aPoint y - variance)) sign = (aDirection y sign))). ] ].

	^resultVertexIndex
! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:26:02.94'!
direction
	"Answer the value of direction"

	^ direction! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:26:02.946'!
direction: anObject
	"Set the value of direction"

	direction := anObject! !

!PgiTunnelHalo methodsFor: 'digging' stamp: 'pre 2/18/2011 20:40:20.991'!
doesTunnelEnd: aDifference

	^(((self front at: #top) x * (aDifference sign)) 
		> ((self end at: #top) x  * (aDifference sign))) 
	  or: (((self front at: #bottom) x * (aDifference sign)) 
		> ((self end at: #bottom) x * (aDifference sign)))
! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 14:43:21.66'!
end
	"Answer the value of end"

	^ end! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/12/2011 14:43:21.67'!
end: anObject
	"Set the value of end"

	end := anObject! !

!PgiTunnelHalo methodsFor: 'digging' stamp: 'pre 2/19/2011 02:03:18.309'!
extendBy: aDifference

	| tempVertices tempFrontDict |

	(self doesTunnelEnd: aDifference )
		ifTrue: [ self finishTunnel. ] 
		ifFalse: [
			tempFrontDict := Dictionary new.
			self front keysAndValuesDo: [ :aKey :aValue | tempFrontDict at: aKey put: (aValue + (aDifference@0))].
			self front: tempFrontDict.
		
			tempVertices := OrderedCollection new.
			tempVertices add: (self front at: #top);
						    add: (self end at: #top);
						    add: (self end at: #bottom);
						    add: (self front at: #bottom).		 
			self tunnel updateShape: tempVertices asArray withShift: (aDifference@0).
			]
	
	
	 ! !

!PgiTunnelHalo methodsFor: 'digging' stamp: 'pre 2/16/2011 17:06:38.946'!
finishTunnel 

	self world unregisterCollisionObject: (self tunnel).
	self tunnel abandon.
	
	
! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:26:02.978'!
front
	"Answer the value of front"

	^ front! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:26:02.986'!
front: anObject
	"Set the value of front"

	front := anObject! !

!PgiTunnelHalo methodsFor: 'initialize-release' stamp: 'pre 2/18/2011 17:34:34.625'!
initialize 
	super initialize.
	self direction: 0@0;
		 front: Dictionary new;
		 end: Dictionary new;
		 upperVertices: OrderedCollection new;
		 lowerVertices: OrderedCollection new.
		 ! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:26:03.02'!
lowerVertices
	"Answer the value of lowerVertices"

	^ lowerVertices! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:26:03.028'!
lowerVertices: anObject
	"Set the value of lowerVertices"

	lowerVertices := anObject! !

!PgiTunnelHalo methodsFor: 'digging' stamp: 'pre 2/18/2011 20:40:38.458'!
splitUnderground  

	| newMountainLevelObject shiftedVertices shift tunnelVertices |
	
	shiftedVertices := OrderedCollection new.
	shift := self calculateShift: (self lowerVertices ).
	self lowerVertices do: [:each | shiftedVertices add: (each - shift)].
	newMountainLevelObject := PgiLevelObject new.
	newMountainLevelObject appearance: (self underground appearance);
					    		   edges: ( shiftedVertices );
					               position: shift;
					    		   type: #Standard.
	((self world) factory build: newMountainLevelObject) .
	
	
	shiftedVertices := OrderedCollection new.
	shift := self calculateShift: (self upperVertices ).
	self upperVertices do: [:each | shiftedVertices add: (each - shift)].
	newMountainLevelObject := PgiLevelObject new.
	newMountainLevelObject appearance: (self underground appearance);
					    		   edges: ( shiftedVertices);
					               position: shift;
					    		   type: #Standard.
	((self world) factory build: newMountainLevelObject).

	tunnelVertices := OrderedCollection new.
	tunnelVertices  add: (self front at: #top);
					add: (self end at: #top);
					add: (self end at: #bottom);
					add: (self front at: #bottom).
	shiftedVertices := OrderedCollection new.
	shift := self calculateShift: ( tunnelVertices ).
	tunnelVertices do: [:each | shiftedVertices add: (each - shift)].
	newMountainLevelObject := PgiLevelObject new.
	newMountainLevelObject appearance: (self underground appearance);
					    		   edges: (shiftedVertices);
					               position: shift;
					    		   type: #Standard.
	self tunnel: ((self world) factory build: newMountainLevelObject).

	self world unregisterCollisionObject: (self underground).
	self underground abandon.
	
! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/16/2011 16:14:44.266'!
tunnel
	"Answer the value of tunnel"

	^ tunnel! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/16/2011 16:14:44.274'!
tunnel: anObject
	"Set the value of tunnel"

	tunnel := anObject! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:26:03.034'!
underground
	"Answer the value of underground"

	^ underground! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:26:03.042'!
underground: anObject
	"Set the value of underground"

	underground := anObject! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:26:03.048'!
upperVertices
	"Answer the value of upperVertices"

	^ upperVertices! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:26:03.056'!
upperVertices: anObject
	"Set the value of upperVertices"

	upperVertices := anObject! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 14:42:57.816'!
world
	"Answer the value of world"

	^ world! !

!PgiTunnelHalo methodsFor: 'accessing' stamp: 'pre 2/6/2011 14:42:57.826'!
world: anObject
	"Set the value of world"

	world := anObject! !

Object subclass: #PgiLevel
	instanceVariableNames: 'levelNumber penguinsToRescue penguinsToSpawn worldSize penguinWell penguinGoal decorationObjects levelObjects toolUsages startPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Resources'!

!PgiLevel commentStamp: 'tm 2/19/2011 01:14:38.274' prior: 0!
A PgiLevel is the logical Structure of a Level and provides all the information that is needed by the WorldBuilder to instantiate it.

Instance Variables
	decorationObjects:		<OrderedCollection>
	levelNumber:			<Integer>
	levelObjects:		<OrderedCollection>
	penguinGoal:		<PgiPenguinGoal>
	penguinWell:		<PgiPenguinWell>
	penguinsToSave:		<Number>
	penguinsToSpawn:		<Number>
	toolValues:		<Object>
	worldSize:		<Point>

decorationObjects
	- Object Configurations of type PgiDecorationObject describing a visual part that does not cause collision

levelNumber
	- The Level Number between 1 and 7

levelObjects
	- Object Configurations of type PgiLevelObject which cause collision, have a texture and polygon bounds

penguinGoal
	- A PgiLevelObject acting as the Goal when the World is built

penguinWell
	- A PgiLevelObject acting as the Well when the World is built

penguinsToSave
	- The number of penguins which have to be rescued until level is considered successful
	
penguinsToSpawn
	- The maximum number of penguins which are spawned by the well

toolValues
	- A dictionary mapping from tool-Symbols to their numbers how often they can be applied within the level

worldSize
	- The dimensions of the full world
!

!PgiLevel class methodsFor: 'instance creation' stamp: 'lsb 2/16/2011 02:07:45.389'!
fromFileNamed: aFileName

	^(PgiLevelReader fromFileNamed: aFileName) readLevel! !

!PgiLevel methodsFor: 'accessing' stamp: 'tm 2/18/2011 22:28:40.532'!
addLevelObject: aLevelObject
	"Add a new object to my level objects"
	
	self levelObjects add: aLevelObject! !

!PgiLevel methodsFor: 'double dispatch' stamp: 'tm 2/18/2011 15:59:37.405'!
attach: anObjectConfiguration
	"attach the configuration for an object within this level"
	
	anObjectConfiguration attachToLevel: self.
	
	! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:52:35.116'!
decorationObjects

	^ decorationObjects! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:54:50.883'!
decorationObjects: anObject

	decorationObjects := anObject! !

!PgiLevel methodsFor: 'initialize-release' stamp: 'tm 2/18/2011 16:01:07.619'!
initialize 

	super initialize.
	
	self 
		decorationObjects: (OrderedCollection new);
		levelObjects: (OrderedCollection new);
		toolUsages: (Dictionary new);
		startPosition: (0@0).! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:55:06.723'!
levelNumber

	^ levelNumber! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:55:23.14'!
levelNumber: aNumber

	levelNumber := aNumber! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:55:28.003'!
levelObjects

	^ levelObjects! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:55:32.571'!
levelObjects: anObject

	levelObjects := anObject! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:55:36.408'!
penguinGoal

	^ penguinGoal! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:56:43.049'!
penguinGoal: aPgiPenguinGoal

	penguinGoal := aPgiPenguinGoal! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:55:55.053'!
penguinWell

	^ penguinWell! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:56:54.712'!
penguinWell: aPgiPenguinWell

	penguinWell := aPgiPenguinWell! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 2/18/2011 00:55:25.153'!
penguinsToRescue

	^ penguinsToRescue! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 2/18/2011 00:55:21.571'!
penguinsToRescue: aNumber

	penguinsToRescue := aNumber! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:57:23.462'!
penguinsToSpawn

	^ penguinsToSpawn! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 01:57:30.047'!
penguinsToSpawn: aNumber

	penguinsToSpawn := aNumber! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 2/18/2011 00:50:10.876'!
startPosition

	^ startPosition! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 2/18/2011 00:29:08.763'!
startPosition: aPoint

	startPosition := aPoint! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 2/11/2011 16:05:58.934'!
toolUsages

	^ toolUsages! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 2/11/2011 16:06:15.132'!
toolUsages: aDictionary

	toolUsages := aDictionary! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:00:39.937'!
worldSize

	^ worldSize! !

!PgiLevel methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:00:45.161'!
worldSize: aPoint

	worldSize := aPoint! !

Object subclass: #PgiLevelReader
	instanceVariableNames: 'dom colorDependentConfiguration tagDispatchMap configMapper fileName level'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Resources'!

!PgiLevelReader commentStamp: 'tm 2/19/2011 01:35:29.259' prior: 0!
A PgiLevelReader is a Method Object which reads the full configurations of level objects from an SVG file so that Levels can be designed using any vector editor. Metadata is read from a corresponding TXT file.

Instance Variables
	colorDependentConfiguration:		<Dictionary>
	configMapper:		<Dictionary>
	dom:		<Object>
	fileName:		<String>
	level:		<PgiLevel>
	tagDispatchMap:		<Dictionary>

colorDependentConfiguration
	- A Dictionary mapping from a color code (such as '#FF00FF') to a closure which defines how to configure a PgiObjectConfiguration having that color in the SVG file

configMapper
	- A Dictionary mapping from a key defined in the meta-TXT-file to a closure handling this configuration entry

dom
	- The Document Object Model of the SVG file

fileName
	- The path of the SVG file

level
	- The level being built during read process

tagDispatchMap
	- A Dictionary mapping from a DOM node type to a method handling this node
!

!PgiLevelReader class methodsFor: 'instance creation' stamp: 'tm 2/15/2011 23:43:21.047'!
fromDOM: aDOMDocument
	"initialize Level Reader with an SVG DOM"

	| instance |
	instance := self basicNew.
	^instance dom: 
		aDOMDocument;
		initialize;
		yourself! !

!PgiLevelReader class methodsFor: 'instance creation' stamp: 'tm 2/19/2011 01:00:18.429'!
fromFileNamed: aString
	"initialize Level Reader with an SVG DOM from a file"

	| instance |
	instance := self fromDOM: (XMLDOMParser parseDocumentFrom: 
		(FileStream fileNamed: aString)).
	^ instance
		fileName: aString;
		yourself! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/16/2011 17:11:57.942'!
colorDependentConfiguration
	"Answer the value of colorDependentConfiguration"

	^ colorDependentConfiguration! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/19/2011 01:28:39.879'!
colorDependentConfiguration: aDictionary
	"Set the value of colorDependentConfiguration"

	colorDependentConfiguration := aDictionary! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/16/2011 17:11:58.204'!
configMapper
	"Answer the value of configMapper"

	^ configMapper! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/19/2011 01:28:48.268'!
configMapper: aDictionary
	"Set the value of configMapper"

	configMapper := aDictionary! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/10/2011 19:08:00.274'!
dom
	"Answer the value of dom"

	^ dom! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/19/2011 01:28:58.246'!
dom: anObject
	"Set the value of dom"

	dom := anObject! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/15/2011 21:55:56.35'!
extractWorldSize: aNode

	^ ((aNode attributes at: 'width') asInteger) @ 
	   ((aNode attributes at: 'height') asInteger)! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/16/2011 17:11:58.291'!
fileName
	"Answer the value of fileName"

	^ fileName! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/19/2011 01:29:06.586'!
fileName: aString
	"Set the value of fileName"

	fileName := aString! !

!PgiLevelReader methodsFor: 'initialize-release' stamp: 'tm 2/16/2011 16:18:29.105'!
initialize

	super initialize.
	self
		initializeColorDependentConfiguration;
		initializeConfigMapper;
		initializeTagDispatchMap
	! !

!PgiLevelReader methodsFor: 'initialize-release' stamp: 'tm 2/18/2011 22:02:19.218'!
initializeColorDependentConfiguration
	"The color of objects we load from the vector graphics file
	determines their behavior. This makes level design much easier."

	self colorDependentConfiguration: Dictionary new.
	self colorDependentConfiguration 
		"yellow = standard object"
		add: '#FFFF00' -> [:levelObj | 
			levelObj 
				type: #Standard;
				yourself];					
		
		"red = penguin well"
		add: '#FF0000' -> [:levelObj | 
			levelObj 
				type: #Well; 
				appearance: #standardWell; 
				yourself]; 			
		
		"green = penguin goal"
		add: '#00FF00' -> [:levelObj | 
			levelObj 
				type: #Goal;
				appearance: #standardGoal;
				yourself];	
		
		"magenta = deadly thing"
		add: '#FF00FF' -> [:levelObj | 
			levelObj 
				type: #Deadly;
				yourself]
! !

!PgiLevelReader methodsFor: 'initialize-release' stamp: 'tm 2/18/2011 22:02:35.235'!
initializeConfigMapper

	self configMapper: Dictionary new.
	self configMapper
		add: 'levelNumber' -> [:values | self levelNumber: values last asInteger];
		add: 'penguinsToSpawn' -> [:values | self penguinsToSpawn: values last asInteger];
		add: 'penguinsToRescue' -> [:values | self penguinsToRescue: values last asInteger];
		add: 'startPositionX' -> [:values | self startPositionX: values last asInteger];
		add: 'startPositionY' -> [:values | self startPositionY: values last asInteger];
		add: 'tools' -> [:values | self tool: (values nextToLast asSymbol)
									 timesAllowed: values last asInteger]! !

!PgiLevelReader methodsFor: 'initialize-release' stamp: 'tm 2/18/2011 22:02:57.181'!
initializeTagDispatchMap
	"This determines which sort of node in the vector graphics
	definition is processed by which function"

	self tagDispatchMap: Dictionary new.
	self tagDispatchMap
		add: #g -> [:node | self processGroup: node];
		add: #polygon -> [:node | self processPolygon: node];
		add: #rect -> [:node | self processRect: node];
		add: #image -> [:node | self processImage: node]! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'tm 2/16/2011 19:11:15.47'!
interpretLine: aString
	"Parses and interprets a line in the metadata config file"
	
	"Syntax is
		 attribute : value [ : additionalValues]* "

	| args |
	args := (aString subStrings: ':') collect: [:ea | ea withBlanksTrimmed].
	(self configMapper at: args first ifAbsent: [^self]) cull: args! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'tm 2/16/2011 19:08:56.882'!
interpretMetaData: aString
	"Interprets a set of configuration lines"

	(aString subStrings: Character cr asString) do: [: ea |
		self interpretLine: ea
	]! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/16/2011 17:26:50.749'!
level
	"Answer the value of level"

	^ level! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/19/2011 01:29:19.24'!
level: aPgiLevel
	"Set the value of level"

	level := aPgiLevel! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'tm 2/16/2011 17:35:38.032'!
levelNumber: anInteger
	
	self level levelNumber: anInteger! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'tm 2/16/2011 19:12:54.48'!
loadMetaFile
	"Loads and interprets the text file containing the meta data
	corresponding to this level."
	
	| file |
	file := FileStream fileNamed: self metaFileName.
	self interpretMetaData: file upToEnd.
	file close! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'tm 2/16/2011 19:09:16.497'!
metaFileName
	"generate the name of the text file containing meta data"
	
	^ (FileDirectory baseNameFor: self fileName), '.txt'! !

!PgiLevelReader methodsFor: 'attribute parsing' stamp: 'lsb 2/16/2011 02:08:33.191'!
parseAppearance: aPath
	"converts an include path into an appearance symbol
	(naked filename without extension)"

	"only / as path separator will be used here!!"
	^((aPath subStrings: '/') last subStrings: '.') first asSymbol! !

!PgiLevelReader methodsFor: 'attribute parsing' stamp: 'tm 2/18/2011 21:52:22.122'!
parseFloat: aString
	"Parses decimal floating point numbers like 3.141"
	"<!!> Rewrite if anyone found the squeak way of doing this..."
	
	| split prePoint postPoint |
	split := aString subStrings: '.' .
	prePoint := split first asInteger.
	postPoint := split last asInteger / (10.0 raisedTo: split last size).
	^ prePoint + ((prePoint >= 0) 
		ifTrue: [postPoint] 
		ifFalse: [postPoint negated])! !

!PgiLevelReader methodsFor: 'attribute parsing' stamp: 'tm 2/18/2011 17:16:10.735'!
parseMatrix: aString
	"extracts x- and y-stretching and translation from a matrix transform"
	
	| entries begin end |
	
	"syntax is: 'matrix(a, b, c, d, deltaX, deltaY). As we only translate and stretch,
	we need a and d as factors and the deltas as offset.
	
	here we need to adjust begin and end by +/-1 
	as we dont' want to have brackets included"
	begin := (aString indexOf: $( ifAbsent: [0]) + 1.
	end := (aString indexOf: $) ifAbsent: [1 + aString size]) - 1.
	entries := (aString copyFrom: begin to: end) subStrings: ' '.

	^{  
		self parseFloat: entries first.
		self parseFloat: entries fourth.
		(entries nextToLast) asInteger @ (entries last asInteger).
	  }
	
! !

!PgiLevelReader methodsFor: 'attribute parsing' stamp: 'lsb 2/16/2011 02:08:39.519'!
parsePoint: aString

	| coords |
	coords := aString subStrings: ',' .
	
	"this will intentionally ignore digits after a point"
	^ (coords first asInteger) @ (coords last asInteger)! !

!PgiLevelReader methodsFor: 'attribute parsing' stamp: 'tm 2/18/2011 17:17:19.806'!
parsePoints: aString
	"converts a space separated point array into a collection of points"
	
	"Syntax is: points are separated by spaces, coordinates by colon"
	^((aString subStrings: ' ') select: [:ea | ea includes: $, ])
		 collect: [:ea | self parsePoint: ea]! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'tm 2/16/2011 17:33:44.894'!
penguinsToRescue: anInteger
	
	self level penguinsToRescue: anInteger! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'tm 2/16/2011 17:33:49.685'!
penguinsToSpawn: anInteger
	
	self level penguinsToSpawn: anInteger
	! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/15/2011 22:47:22.635'!
polygonFromRectNode: aNode
	"Converts a rectangle entry in the vector graphics file
	to a polygon as this game operates on polygones only."
	
	| x y width height |
	"Sorry, but we have to do some math here..."
	
	x := (aNode attributes at: 'x') asInteger.
	y := (aNode attributes at: 'y') asInteger.
	width := (aNode attributes at: 'width') asInteger.
	height := (aNode attributes at: 'height') asInteger.
	
	^{ 
		x @ y.
		(x + width) @ y.
		(x + width) @ (y + height).
		x @ (y + height).
	}
	! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/18/2011 22:35:54.184'!
processGroup: aNode
	
	| levelObject texture |
	
	levelObject := nil.
	texture := nil.

	"Don't use the dispatch-dictionary here, it processes images as separate
	LevelObjects instead of symbols only:"
	aNode elementsAndContents do: [ : ea |
		(ea name = #polygon) ifTrue: [levelObject :=  self processPolygon: ea].
		(ea name = #rect) ifTrue: [levelObject := self processRect: ea].
		(ea name = #image) ifTrue: [texture := self processSymbolicImage: ea]
	].
	
	levelObject ifNotNil: [
		texture ifNotNil: [levelObject appearance: texture].
		^levelObject
	] ifNil: [ ^nil	]
! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/16/2011 18:53:24.038'!
processImage: aNode
	"Interprets a single image entry as decoration object"

	| appearance matrix |
	
	appearance := self processSymbolicImage: aNode.
	matrix := self parseMatrix: (aNode attributes at: 'transform').
	
	^ (PgiDecorationObject new) 
		appearance: appearance;
		position: matrix last;
		yourself
	
	! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/18/2011 22:03:40.814'!
processNode: aNode
	"Dispatch node processing by node name 
	or return nil if the node is unsupported."

	^(self tagDispatchMap at: aNode name ifAbsent: [^nil]) 
		value: aNode! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/18/2011 17:28:49.227'!
processPolygon: aNode
	"Converts a polygon entry of the vector graphics file to a point array"

	| levelObject |
	
	levelObject := self processShape: aNode.
	^levelObject 
		edges: (self parsePoints: (aNode attributes at: 'points'));
		normalizeShape;
		yourself! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/18/2011 17:28:53.488'!
processRect: aNode
	"Converts a rectangle entry of the vector graphics file to a point array"

	| levelObject |
	
	levelObject := self processShape: aNode.
	^levelObject
		edges: (self polygonFromRectNode: aNode);
		normalizeShape;
		yourself
! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/16/2011 22:11:29.291'!
processRoot: aNode
	"Walks the root node of the vector graphics file processing all entries
	and attaching them to the level."
	
	aNode 
		elementsAndContents do: [ :ea |  | processedObject |
			processedObject := self processNode: ea.
			processedObject ifNotNil: [processedObject attachToLevel: self level.]
		]! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/18/2011 22:03:33.48'!
processShape: aNode
	"return a generic pre-configured object from a shape-like
	vector graphic entry."
	
	| color levelObject |
	
	levelObject := PgiLevelObject new.
	color := (aNode attributes at: 'fill') asUppercase.
	(self colorDependentConfiguration includesKey: color) ifTrue: [
		levelObject := (self colorDependentConfiguration at: color) 
						value: levelObject 
	].

	^levelObject! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/11/2011 00:52:03.464'!
processSymbolicImage: aNode
	"As no image is ever loaded by this object, we only extract a symbol that
	encodes which image was meant and let it load somewhere else."
	
	^self parseAppearance: (aNode attributes at: 'xlink:href')! !

!PgiLevelReader methodsFor: 'node processing' stamp: 'tm 2/17/2011 00:58:26.794'!
readLevel
	"Set up the full level reading from the underlying vector graphics file"
	| root |
	
	root := self dom elementsAndContents first.
	self
		level: PgiLevel new;
		processRoot: root;
		loadMetaFile.
	^self level 
		worldSize: (self extractWorldSize: root);
		yourself! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'kh 2/18/2011 00:50:49.204'!
startPosition: aPoint
	
	self level startPosition: aPoint! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'kh 2/18/2011 01:18:40.681'!
startPositionX: aNumber
	
	self level startPosition:  self level startPosition + (aNumber@0)! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'kh 2/18/2011 01:19:09.133'!
startPositionY: aNumber
	
	self level startPosition:  self level startPosition + (0@aNumber)
	

! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/16/2011 17:11:58.112'!
tagDispatchMap
	"Answer the value of tagDispatchMap"

	^ tagDispatchMap! !

!PgiLevelReader methodsFor: 'accessing' stamp: 'tm 2/19/2011 01:29:32.009'!
tagDispatchMap: aDictionary
	"Set the value of tagDispatchMap"

	tagDispatchMap := aDictionary! !

!PgiLevelReader methodsFor: 'metadata' stamp: 'tm 2/16/2011 19:12:43.012'!
tool: aSymbol timesAllowed: anInteger

	self level toolUsages at: aSymbol put: anInteger! !

Object subclass: #PgiMenu
	instanceVariableNames: 'gameState buttonPurposes visualPart'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiMenu commentStamp: 'RS 2/11/2011 00:29:05.722' prior: 0!
A PgiMenu is a Structure containing all the needed Buttons in a specific state, such as LevelButtons or toolButtons.

Instance Variables
	buttonPurposes:		<Object>
	buttons:		<Object>
	state:		<Object>
	visualPart:		<Object>

buttonPurposes
	- xxxxx

buttons
	- xxxxx

state
	- xxxxx

visualPart
	- xxxxx
!

PgiMenu subclass: #PgiFinalizeMenu
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiFinalizeMenu commentStamp: 'rs 2/18/2011 21:44:37.737' prior: 0!
A PgiFinalizeMenu is the menu, that appears when a level is terminated by the user.

Instance Variables
!

!PgiFinalizeMenu methodsFor: 'initialize' stamp: 'kh 2/18/2011 11:05:22.81'!
initialize

	super initialize.
	
	self buttonPurposes: (Dictionary newFrom: {
			(#TryAgain -> [self gameState restartLevel] ).	
			(#Continue -> [self gameState startNextLevel] ).
			(#HomeMenu -> [self gameState openHomeScreen])}).! !

!PgiFinalizeMenu methodsFor: 'initialize' stamp: 'rs 2/16/2011 18:45:42.466'!
initializeControlButtons
 
	| controlButtons  |
	
	(self gameState successful) ifTrue: [controlButtons := self gameState ressourceManager requestSuccessButtonsFor: self]
					   ifFalse: [controlButtons := self gameState ressourceManager  requestLooseButtonsFor: self].
	
	controlButtons do: [:each | self add: each to: self visualPart].! !

!PgiFinalizeMenu methodsFor: 'initialize' stamp: 'rs 2/16/2011 18:49:21.311'!
initializeMenu

	self initializeControlButtons;
		initializeVisualPart.! !

!PgiFinalizeMenu methodsFor: 'initialize' stamp: 'rs 2/16/2011 18:48:27.678'!
initializeVisualPart
	
	(self visualPart) extent: self gameState extent;
					color: Color transparent;
					position: self gameState position.! !

PgiMenu subclass: #PgiHomeMenu
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiHomeMenu commentStamp: 'RS 2/11/2011 00:41:01.996' prior: 0!
A PgiHomeMenu contains all Buttons for starting the levels or leaving the game.

Instance Variables
!

!PgiHomeMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:58:48.414'!
initialize

	super initialize.
	
	self buttonPurposes: (Dictionary newFrom: {
			(#ExitGame -> [self gameState closeGame] ).	
			(#LoadLevel1 -> [self gameState startLevel: 1]).
			(#LoadLevel2 -> [self gameState startLevel: 2]).
			(#LoadLevel3 -> [self gameState startLevel: 3]).
			(#LoadLevel4 -> [self gameState startLevel: 4]).
			(#LoadLevel5 -> [self gameState startLevel: 5]).
			(#LoadLevel6 -> [self gameState startLevel: 6]).
			(#LoadLevel7 -> [self gameState startLevel: 7]).}).! !

!PgiHomeMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 02:51:21.859'!
initializeControlButtons
 
	| controlButtons  |
	
	controlButtons := (self gameState requestHomeButtonsFor: self).
	controlButtons do: [:each | self add: each to: self visualPart].

! !

!PgiHomeMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:04:35.438'!
initializeSpecialButtonsFor: aHighestReachedLevelNumber
 
	| levelButtons |
	
	levelButtons :=  (self gameState requestLevelStartButtonsFor: self).
	levelButtons do: [:button | self add: button to: self visualPart.
							    button activateIfBelongsTo: aHighestReachedLevelNumber.]
										
																	
! !

!PgiHomeMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 02:50:43.333'!
initializeVisualPart
	
	(self visualPart) extent: self gameState extent;
					color: Color transparent;
					position: self gameState position.


! !

PgiMenu subclass: #PgiLevelMenu
	instanceVariableNames: 'score scorePenguinsRescued scorePenguinsSpawned toolButtons selectedTool'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiLevelMenu commentStamp: 'RS 2/11/2011 00:41:32.849' prior: 0!
A PgiLevelMenu contains all buttons to apply tools and to pause and quit the running level.

Instance Variables
	penguinsSaved:		<Object>
	penguinsSpawned:		<Object>
	score:		<Object>
	toolButtons:		<Object>

penguinsSaved
	- xxxxx

penguinsSpawned
	- xxxxx

score
	- xxxxx

toolButtons
	- xxxxx
!

!PgiLevelMenu methodsFor: 'accessing' stamp: 'RS 1/2/2011 23:44:04.99'!
abandon 

	super abandon.
	self score abandon.! !

!PgiLevelMenu methodsFor: 'update level indicators' stamp: 'kh 2/16/2011 05:10:12.102'!
areToolUsagesLeftFor: aToolSymbol

	^ (self toolButtons at: aToolSymbol) areToolUsagesLeft

	! !

!PgiLevelMenu methodsFor: 'text managing' stamp: 'kh 2/16/2011 04:16:00.552'!
createTextMorph: aString at: aPosition withSize: aSize

	| resultTextMorph |
	
	resultTextMorph := TextMorph new.
	resultTextMorph position: self score position + aPosition;
					  color: Color white;
					  contents: (Text string: aString attribute: TextEmphasis bold);
					  beAllFont: (StrikeFont familyName: #BitstreamVeraSans size: aSize);
					  lock;
					  openInWorld.
					
	self score addMorph: resultTextMorph.

	^ resultTextMorph 
						! !

!PgiLevelMenu methodsFor: 'text managing' stamp: 'kh 2/16/2011 03:49:03.398'!
fontSize

	^30! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:06:53.932'!
highlightButton:  aSymbol

	(self toolButtons at: aSymbol) setHighlighting. 

! !

!PgiLevelMenu methodsFor: 'initialize-release' stamp: 'rs 2/18/2011 17:24:10.415'!
initialize

	super initialize.
	
	self selectedTool: #None.
			
	self buttonPurposes: (Dictionary newFrom: {		
			(#Walker -> [self gameState toolHasBeenSelected: #Walker] ).
			(#Digger -> [self gameState toolHasBeenSelected: #Digger] ).
			(#Blocker -> [self gameState toolHasBeenSelected: #Blocker] ).
			(#Glider -> [self gameState toolHasBeenSelected: #Glider]) .
			(#Builder -> [self gameState toolHasBeenSelected: #Builder]).
			(#Jetpack -> [self gameState toolHasBeenSelected: #Jetpack]).
			(#Miner -> [self gameState toolHasBeenSelected: #Miner]).
			
			(#PauseGame -> [self gameState pauseGame]).
			(#PlayGame -> [self gameState playGame]).
			(#RunFaster -> [self gameState runFaster]).
			(#FinalizeLevel -> [self gameState finalizeLevel])}).
		! !

!PgiLevelMenu methodsFor: 'initialize-release' stamp: 'kh 2/16/2011 02:54:08.584'!
initializeControlButtons

	| controlButtons |
	
	controlButtons := self gameState requestWorldButtonsFor: self.	
	controlButtons do: [:button | self add: button to: self visualPart].
! !

!PgiLevelMenu methodsFor: 'initialize-release' stamp: 'kh 2/16/2011 05:31:23.803'!
initializeDynamicScore

	| offsetPenguinsSpawned offsetPenguinsRescued |
	
	offsetPenguinsSpawned := self xSpawnedText +45.
	offsetPenguinsRescued := self xRescuedText +67.
	
	self scorePenguinsSpawned:	(self createTextMorph: (self gameState penguinsSpawned) asString
												 at: (offsetPenguinsSpawned @ self yScoreText)
												 withSize: self fontSize).
	
	self scorePenguinsRescued: (self createTextMorph: (self gameState penguinsRescued) asString
										    	   at: (offsetPenguinsRescued @ self yScoreText)
											   withSize: self fontSize).
! !

!PgiLevelMenu methodsFor: 'initialize-release' stamp: 'kh 2/16/2011 03:03:39.851'!
initializeMenu: aLevelNumber
	
	super initializeMenu: aLevelNumber.
	self initializeScore.


		! !

!PgiLevelMenu methodsFor: 'initialize-release' stamp: 'kh 2/16/2011 03:59:24.476'!
initializeScore

	| scoreImage |
	
	scoreImage := (((self gameState requestAppearance: #scoreBox) first) asMorph) beSticky.
	
	self score: PasteUpMorph new beSticky.
	(self score) color: Color transparent;
				 borderWidth: 0;
				 extent: scoreImage extent;
				 position: self topMiddle.
	
	scoreImage position: self score position.	 
	self score addMorph: scoreImage.

	self setStaticText;
		 initializeDynamicScore.
! !

!PgiLevelMenu methodsFor: 'initialize-release' stamp: 'kh 2/16/2011 04:44:00.425'!
initializeSpecialButtonsFor: aLevelNumber

	| toolUsages | 
	
	self toolButtons: (self gameState requestToolButtonsFor: self).
	toolUsages := self gameState requestToolUsagesFor: aLevelNumber.
	
	self toolButtons do: [:button | self add: button to: self visualPart.
								   button initializeMarker;
										  setToolUsagesLeft: toolUsages].
! !

!PgiLevelMenu methodsFor: 'initialize-release' stamp: 'kh 2/16/2011 05:05:55.259'!
initializeVisualPart
	
	| menuHeight menuPositionY |
	
	menuHeight := 80.
	menuPositionY := (self gameState bottomLeft)-(0@40).
	
	(self visualPart) extent: (self gameState extent x)@menuHeight;
					bottomLeft: menuPositionY;
					color: Color transparent;
					borderWidth: 0.! !

!PgiLevelMenu methodsFor: 'creation and cleanup' stamp: 'kh 2/16/2011 05:06:22.468'!
launch 

	"Open in world for both pasteup morphs"
	super launch.
	self gameState game addMorph: self score.
! !

!PgiLevelMenu methodsFor: 'initialize-release' stamp: 'kh 2/16/2011 05:06:43.526'!
resetButton: aSymbol

	(self toolButtons at: aSymbol) resetHighlighting. 

! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:07:18.411'!
score

	^ score! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:07:42.017'!
score: aPasteUpMorph

	score := aPasteUpMorph! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:31:39.224'!
scorePenguinsRescued

	^ scorePenguinsRescued! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:32:33.602'!
scorePenguinsRescued: aTextMorph

	scorePenguinsRescued := aTextMorph! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:31:50.915'!
scorePenguinsSpawned

	^ scorePenguinsSpawned! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/18/2011 11:32:37.543'!
scorePenguinsSpawned: aTextMorph

	scorePenguinsSpawned := aTextMorph! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/11/2011 13:10:06.13'!
selectedTool

	^ selectedTool! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:07:52.109'!
selectedTool: aSymbol

	selectedTool := aSymbol! !

!PgiLevelMenu methodsFor: 'initialize-release' stamp: 'kh 2/16/2011 05:18:33.228'!
setStaticText

	| level out home xHeadline yHeadline fontSizeHeadline|
	
	xHeadline := 134.
	yHeadline := 20.	
	fontSizeHeadline := 45.
	
	level := 'Level: ',self gameState levelNumber.
	out :=  'Out:      of: ', self gameState penguinsToSpawn.
	home := 'Home:      of: ',self gameState penguinsToRescue.
	
	self createTextMorph: level at:  xHeadline@yHeadline withSize: fontSizeHeadline;
		 createTextMorph: out at: (self xSpawnedText @ self yScoreText) withSize: self fontSize;
		 createTextMorph: home at: (self xRescuedText @ self yScoreText)  withSize: self fontSize.	
! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:07:26.716'!
toolButtons

	^ toolButtons! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:08:05.619'!
toolButtons: aDictionary

	toolButtons := aDictionary! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:09:58.377'!
toolHasBeenApplied: aToolSymbol

	(self toolButtons at: aToolSymbol) decreaseLeftToolUsages.	
		
	(self areToolUsagesLeftFor: aToolSymbol) ifFalse: [self selectedTool: #None].  	
! !

!PgiLevelMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:10:43.356'!
toolHasBeenSelected: aToolSymbol

	(self selectedTool ~= #None) ifTrue: [self resetButton: self selectedTool].		
	
	self highlightButton: aToolSymbol.
	self selectedTool: aToolSymbol.
! !

!PgiLevelMenu methodsFor: 'initialize-release' stamp: 'lsb 2/16/2011 15:52:29.934'!
topMiddle 

	^ self gameState position + ( (((self gameState game width)-(self score width)) / 2) @0)! !

!PgiLevelMenu methodsFor: 'update level indicators' stamp: 'kh 2/16/2011 05:31:56.299'!
updateNumberOfRescuedPenguins

	self scorePenguinsRescued contents: (self gameState penguinsRescued) asString.
	! !

!PgiLevelMenu methodsFor: 'update level indicators' stamp: 'kh 2/16/2011 05:32:27.567'!
updateNumberOfSpawnedPenguins

	self scorePenguinsSpawned contents: (self gameState penguinsSpawned) asString.
	! !

!PgiLevelMenu methodsFor: 'text managing' stamp: 'kh 2/16/2011 03:51:14.063'!
xRescuedText

	^ 210! !

!PgiLevelMenu methodsFor: 'text managing' stamp: 'kh 2/16/2011 03:51:03.231'!
xSpawnedText

	^ 30! !

!PgiLevelMenu methodsFor: 'text managing' stamp: 'kh 2/16/2011 04:18:27.256'!
yScoreText

	^66! !

PgiMenu subclass: #PgiLoadingMenu
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiLoadingMenu commentStamp: 'RS 2/11/2011 11:09:18.609' prior: 0!
A PgiLoadingMenu is a dummy Menu that merely does nothing at all, since the loading screen has no options yet.

Instance Variables
!

!PgiLoadingMenu methodsFor: 'initialize' stamp: 'pre 2/13/2011 18:04:30.396'!
initializeMenu

	^self! !

!PgiMenu methodsFor: 'accessing' stamp: 'pre 1/2/2011 20:38:30.666'!
abandon

	self visualPart abandon.! !

!PgiMenu methodsFor: 'functionality' stamp: 'kh 2/10/2011 22:11:54.842'!
add: aButton to: aMorph

	aButton position: ((aMorph position) + (aButton position));
			 labelGraphic: aButton image;
			 beSticky;
			 openInWorld.
			
	aMorph addMorph: aButton.! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:02:04.995'!
buttonPurposes

	^ buttonPurposes! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:03:58.358'!
buttonPurposes: aDictionary

	buttonPurposes := aDictionary! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 02:18:25.098'!
gameState

	^ gameState! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 02:18:35.822'!
gameState: aPgiGameState

	gameState := aPgiGameState! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 2/10/2011 20:19:08.181'!
initialize

	super initialize.
	self visualPart: PasteUpMorph new.
	self visualPart beSticky.
	! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 02:50:01.033'!
initializeControlButtons

	self subclassResponsibility! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 02:53:58.271'!
initializeMenu: aLevelNumber

	self initializeVisualPart;
		 initializeControlButtons;
		 initializeSpecialButtonsFor: aLevelNumber.
! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 02:50:12.8'!
initializeSpecialButtonsFor: aLevelNumber

	self subclassResponsibility! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 2/18/2011 14:30:57.677'!
initializeVisualPart

	self subclassResponsibility.! !

!PgiMenu methodsFor: 'creation and cleanup' stamp: 'RS 2/11/2011 16:19:34.697'!
launch 

	self gameState game addMorph: visualPart.
! !

!PgiMenu methodsFor: 'evaluation' stamp: 'kh 2/16/2011 04:58:17.561'!
receive: aPurpose

	"this method gets called by clicked buttons who provide their purpose symbol, so that the menu knows how to react."
	(self buttonPurposes at: aPurpose ) value.! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:02:39.526'!
visualPart

	^ visualPart! !

!PgiMenu methodsFor: 'accessing' stamp: 'kh 2/16/2011 04:58:39.172'!
visualPart: aPasteUpMorph

	visualPart := aPasteUpMorph! !

Object subclass: #PgiObjectConfiguration
	instanceVariableNames: 'appearance position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Resources'!

!PgiObjectConfiguration commentStamp: 'tm 2/19/2011 01:21:47.12' prior: 0!
A PgiObjectConfiguration is a Superclass of the configuration of any Object appearing in the World when built.
It is neither a GUI nor a logical component but some kind of instruction as it is interpreted by some factory to built active level objects.

Instance Variables
	appearance:		<Symbol>
	position:		<Point>

appearance
	- A symbol mapping to an external image/animation representing the appearance of the object. Mapping is done by the PgiResource class.

position
	- The top left corner where the Object will be placed on the World
!

PgiObjectConfiguration subclass: #PgiDecorationObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Resources'!

!PgiDecorationObject commentStamp: 'tm 2/19/2011 01:24:09.778' prior: 0!
A PgiDecorationObject describes an object not causing collision or interaction. It is interpreted by some factory which then builds a visible object.
!

!PgiDecorationObject methodsFor: 'double dispatch' stamp: 'tm 2/11/2011 03:24:43.825'!
attachToLevel: aLevel

	aLevel decorationObjects add: self.! !

!PgiDecorationObject methodsFor: 'accessing' stamp: 'RS 2/11/2011 20:49:22.548'!
type

	^#Decorator! !

PgiObjectConfiguration subclass: #PgiLevelObject
	instanceVariableNames: 'edges type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Resources'!

!PgiLevelObject commentStamp: 'tm 2/19/2011 01:25:39.93' prior: 0!
A PgiLevelObject is the plain configuration of a Object in a Level and provides all the information that is needed by some factory to create the interactive and visible object.

Instance Variables
	appearance:		<Symbol>
	deadly:		<Boolean>
	edges:		<OrderedCollection>
	position:		<Point>
	type:		<Symbol>

appearance
	- defines the desired texture / picture of the Object

deadly
	- sets wether the object will kill Penguins that collide with it.

edges
	- the vertices of the object, i.e. its shape.

position
	- position inside the Level.

type
	- defines the kind of Object and is needed by the Product trader, to look up the appropriate Production rule.
!

!PgiLevelObject methodsFor: 'double dispatch' stamp: 'tm 2/18/2011 22:28:49.8'!
attachToLevel: aLevel
	
	(self type = #Well) ifTrue: [aLevel penguinWell: self] ifFalse: [
		(self type = #Goal) ifTrue: [aLevel penguinGoal: self] ifFalse: [
			aLevel addLevelObject: self.
		]
	]

	"Consider subclassing when adding a third special type!! --tm"! !

!PgiLevelObject methodsFor: 'geometry' stamp: 'tm 2/18/2011 16:08:07.056'!
detectOffset
	"detects the minimum offset all vertices of my shape share"
	
	| top left |
	left := (self edges detectMin: [:each | each x]) x.
	top := (self edges detectMin: [:each | each y]) y.
	
	^ left @ top! !

!PgiLevelObject methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:01:22.908'!
edges

	^ edges! !

!PgiLevelObject methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:01:26.407'!
edges: anObject

	edges := anObject! !

!PgiLevelObject methodsFor: 'initialize-release' stamp: 'tm 2/17/2011 02:20:20.492'!
initialize
	
	super initialize.
	self type: #Standard.! !

!PgiLevelObject methodsFor: 'geometry' stamp: 'tm 2/19/2011 01:02:10.151'!
makeVerticesClockwise
	"Adjusts my vertices to be in clockwise order"

	| clockScore size i j k z |
	
	"This algorithm determines the directional area of
	my polygon. Its sign indicates the order of vertices.
	Source:
	http://local.wasp.uwa.edu.au/~pbourke/geometry/clockwise/index.html
	
	Note that we changed the algorithm...
	* ... that it fits the 1-based array indices (which bloats code)
	* ... that it corrects the mirrored coordinate system (y grows in bottom direction!!)
	"
	clockScore := 0.
	size := self edges size.
	
	0 to: (self edges size - 1) do: [ :eachIndex |
 
 		i := eachIndex + 1.
 		j := ((eachIndex + 1) \\ (size)) + 1.
 		k := ((eachIndex + 2) \\ (size)) + 1.
 
 		z := ( ( ((self edges at: j) x ) - ((self edges at: i) x) ) 
			* ( ( (self edges at: k) y ) - ((self edges at: j) y) ) ).
 		z := z - ((((self edges at: j ) y ) - ((self edges at: i) y)) 
			* (((self edges at: k) x ) - ((self edges at: j) x))).
 
  		clockScore := clockScore + z sign.
	].

	(clockScore < 0) ifTrue: 
		[ self edges: self edges reversed.].
	
	^self! !

!PgiLevelObject methodsFor: 'geometry' stamp: 'tm 2/18/2011 17:36:53.366'!
normalizeShape
	"Brings my shape into the normal form. That is:
	- one vertex has x = 0, another y = 0 so that
	  the only shift is denoted by the position vector
	- vertices are clockwise
	Presuming these invariants later avoids evil edge cases!!"
	
	| offset |
	offset := self detectOffset.
	self 
		retrogressBy: offset;
		position: (self position + offset);
		makeVerticesClockwise.
		
	
	! !

!PgiLevelObject methodsFor: 'geometry' stamp: 'tm 2/18/2011 16:46:46.031'!
retrogressBy: aVector
	"eliminates a constant offset from all vertices"
	
	self edges: (self edges collect: [:ea | ea - aVector])
	! !

!PgiLevelObject methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:01:40.486'!
type

	^ type! !

!PgiLevelObject methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:01:46.034'!
type: anObject

	type := anObject! !

!PgiObjectConfiguration methodsFor: 'accessing' stamp: 'tm 2/11/2011 03:22:45.363'!
appearance
	"Answer the value of appearance"

	^ appearance! !

!PgiObjectConfiguration methodsFor: 'accessing' stamp: 'tm 2/19/2011 01:15:10.085'!
appearance: aSymbol
	"Set the value of appearance"

	appearance := aSymbol! !

!PgiObjectConfiguration methodsFor: 'double dispatch' stamp: 'tm 2/11/2011 03:21:28.05'!
attachToLevel: aLevel

	^self subclassResponsibility! !

!PgiObjectConfiguration methodsFor: 'initialize-release' stamp: 'tm 2/18/2011 17:31:58.802'!
initialize
	
	super initialize.
	self 
		appearance: #Untextured;
		position: 0@0! !

!PgiObjectConfiguration methodsFor: 'accessing' stamp: 'tm 2/11/2011 03:22:45.537'!
position
	"Answer the value of position"

	^ position! !

!PgiObjectConfiguration methodsFor: 'accessing' stamp: 'tm 2/19/2011 01:15:50.415'!
position: aPoint
	"Set the value of position"

	position := aPoint! !

!PgiObjectConfiguration methodsFor: 'accessing' stamp: 'tm 2/18/2011 22:35:02.397'!
type
	
	self subclassResponsibility! !

Object subclass: #PgiPenguinState
	instanceVariableNames: 'penguin direction collisionObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic'!

!PgiPenguinState commentStamp: 'RS 2/11/2011 12:03:02.374' prior: 0!
A PgiPenguinState is an Abstract class determining all functions that are needed to be impleemented for all Penguin states that are possible.

Instance Variables
	direction:		Point
	movementVector:		Point
	penguin:		PgiPenguin

direction
	- The direction the Penguin is moving to. This determines direction and speed.

movementVector
	- This is the actual direction the Penguin will move to during this step.

penguin
	- The Penguin the state is currently connected to.
!

PgiPenguinState subclass: #PgiBitingTheDust
	instanceVariableNames: 'counter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiBitingTheDust commentStamp: 'pre 2/19/2011 01:37:24.921' prior: 0!
A PgiBitingTheDust is the State the Penguin enters when is about to pass away.

Instance Variables
	counter:		<Number>

counter
	- How many cycles has the Penguin left to live.
!

!PgiBitingTheDust methodsFor: 'transitions' stamp: 'pre 2/13/2011 22:55'!
allowedTransitionTo: aStateClass
	"It is not possible to change the state, once a Penguin is biting the dust."

	^false! !

!PgiBitingTheDust methodsFor: 'accessing' stamp: 'kh 1/12/2011 03:53:22.019'!
appearance
	"answer my dying appearance"
	
	^#PenguinDying
	! !

!PgiBitingTheDust methodsFor: 'accessing' stamp: 'tm 1/6/2011 18:09:04.108'!
counter
	"Answer the value of counter"

	^ counter! !

!PgiBitingTheDust methodsFor: 'accessing' stamp: 'tm 1/6/2011 18:09:45.466'!
counter: anInteger
	"Set the value of counter"

	counter := anInteger! !

!PgiBitingTheDust methodsFor: 'transitions' stamp: 'rs 2/18/2011 23:30:12.77'!
enter
	"launch biting the dust"
	| framesToLive |
	
	super enter.
	
	framesToLive := 14.

	self counter: framesToLive.! !

!PgiBitingTheDust methodsFor: 'progressing' stamp: 'rs 2/18/2011 18:41:07.435'!
proceed
	"prevent state from leaving our world for a few cycles"
	
	self counter: (self counter - 1).
	(self counter <= 0)	ifTrue: [self penguin abandon].! !

PgiPenguinState subclass: #PgiBlocking
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiBlocking commentStamp: 'RS 2/11/2011 00:45:28.873' prior: 0!
A PgiBlocking is the state of a Penguin preventing others from going past him.

Instance Variables
!

!PgiBlocking methodsFor: 'accessing' stamp: 'kh 1/13/2011 20:03:09.95'!
appearance 

	^#PenguinBlocking
	! !

!PgiBlocking methodsFor: 'progressing' stamp: 'kh 1/12/2011 11:47:59.427'!
carryOn 

	^self! !

!PgiBlocking methodsFor: 'transitions' stamp: 'lsb 2/16/2011 19:06:08.572'!
decideTransition

	^self! !

!PgiBlocking methodsFor: 'transitions' stamp: 'rs 2/16/2011 15:20:27.649'!
enter
	super enter.
	self penguin world registerCollisionObject: self penguin.! !

!PgiBlocking methodsFor: 'transitions' stamp: 'rs 2/16/2011 15:20:48.113'!
leave

	self penguin world unregisterCollisionObject: self penguin.! !

!PgiBlocking methodsFor: 'progressing' stamp: 'kh 1/12/2011 11:48:56.389'!
proceed

	^self! !

PgiPenguinState subclass: #PgiBuildingBridge
	instanceVariableNames: 'counter piecesLeft world'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiBuildingBridge commentStamp: 'pre 2/19/2011 01:33:17.481' prior: 0!
A PgiBuildingBridge is state the Penguin is in when he places a bridge piece.

Instance Variables
	counter:		<Number>
	piecesLeft:		<Number>

counter
	current cycle number of drop cycle.

piecesLeft
	determines how many pieces the penguin can still place.
!

!PgiBuildingBridge class methodsFor: 'creation' stamp: 'rs 2/18/2011 17:54:27.51'!
from: aState

	| newState |
	
	newState := super from: aState.
	
	(aState class == PgiWalkingBridge) ifFalse: [newState piecesLeft: 10]
									       ifTrue: [newState piecesLeft: aState piecesLeft].
	
	^newState! !

!PgiBuildingBridge methodsFor: 'accessing' stamp: 'kh 1/18/2011 19:53:00.025'!
appearance

	(self direction x > 0) 
		ifTrue: [^#PenguinPlaceBridgePieceRight]
		ifFalse: [^#PenguinPlaceBridgePieceLeft]
	! !

!PgiBuildingBridge methodsFor: 'progressing' stamp: 'RS 1/18/2011 20:08:18.674'!
carryOn

	self counter: ((self counter) + 1).! !

!PgiBuildingBridge methodsFor: 'accessing' stamp: 'RS 1/6/2011 18:49:43.389'!
counter
	"Answer the value of counter"

	^ counter! !

!PgiBuildingBridge methodsFor: 'accessing' stamp: 'RS 1/6/2011 18:49:43.395'!
counter: anObject
	"Set the value of counter"

	counter := anObject! !

!PgiBuildingBridge methodsFor: 'transitions' stamp: 'rs 2/18/2011 23:31:23.384'!
decideTransition 

	| cyclesToFetchPiece |
	
	cyclesToFetchPiece := 7.
	
	(self counter == cyclesToFetchPiece) 
		ifTrue: [
			self placePieceAt: self determinePieceLocation.
			self	 changeTo: PgiWalkingBridge] 
		ifFalse: [self carryOn].! !

!PgiBuildingBridge methodsFor: 'progressing' stamp: 'rs 2/18/2011 17:53:41.864'!
determinePieceLocation

	| location piecePositionCorrection oppositeLocation |
	
	piecePositionCorrection := (((self direction x sign)*5)@ -9).
	
	location := self penguin getEdgeFrom: (self penguin bottomMiddle) to: (self direction x).
	location := location + piecePositionCorrection.
	
	oppositeLocation := location - (30@0).
	
	(self direction x sign == -1) ifTrue: [^oppositeLocation]. 
	
	^location! !

!PgiBuildingBridge methodsFor: 'transitions' stamp: 'rs 2/18/2011 18:05:52.205'!
enter
	super enter.
	
	world := self penguin world.
	counter := 0.! !

!PgiBuildingBridge methodsFor: 'accessing' stamp: 'RS 1/18/2011 20:26:41.491'!
piecesLeft
	"Answer the value of piecesLeft"

	^ piecesLeft! !

!PgiBuildingBridge methodsFor: 'accessing' stamp: 'RS 1/18/2011 20:26:41.498'!
piecesLeft: anObject
	"Set the value of piecesLeft"

	piecesLeft := anObject! !

!PgiBuildingBridge methodsFor: 'progressing' stamp: 'rs 2/18/2011 18:06:05.167'!
placePieceAt: aPoint
"creates and places a TexturedMoprh serving as a bridge-piece at aPoint"
	
	| pieceLevelObject |
	
	pieceLevelObject := PgiWorldObject levelObject.
	
	pieceLevelObject appearance: #BridgePiece;
					    edges: {0@0 . 30@0 . 30@10 . 0@10};
					    position: aPoint.
			
	self piecesLeft: ((self piecesLeft) - 1).
	
	world factory build: pieceLevelObject.! !

PgiPenguinState subclass: #PgiFalling
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiFalling commentStamp: 'RS 2/11/2011 00:46:47.166' prior: 0!
A PgiFalling is the State the Penguin enter when he realizes that no Ground is below him.

Instance Variables
!

!PgiFalling methodsFor: 'transitions' stamp: 'rs 2/18/2011 22:02:44.192'!
allowedTransitionTo: aStateClass
	
	| allowedStates |
	
	allowedStates := Set new.
	allowedStates 	add: PgiParachuterFalling;
					add: PgiWalking; 
					add: PgiParachuterWalking;
					add: PgiBitingTheDust;
					add: PgiJetPackUp.
	
	^(allowedStates includes: aStateClass)! !

!PgiFalling methodsFor: 'accessing' stamp: 'tm 12/27/2010 01:59:02.431'!
appearance
	"answer my free fall appearance"
	
	^#PenguinFall! !

!PgiFalling methodsFor: 'progressing' stamp: 'RS 2/3/2011 22:15:36.401'!
carryOn
	"carry on falling"

	self penguin moveBy: (self collisionObject vector).! !

!PgiFalling methodsFor: 'transitions' stamp: 'lsb 2/10/2011 20:18:18.523'!
decideTransition 
"continue falling or impact somewhere"
	
	(self hasGround)
		ifTrue: [self changeTo: PgiWalking]
		ifFalse: [self carryOn].! !

PgiFalling subclass: #PgiFallingFreely
	instanceVariableNames: 'startHeight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiFallingFreely commentStamp: 'pre 2/19/2011 01:34:11.897' prior: 0!
A PgiFallingFreely is the State the Penguin enters when he is (unluckily) Falling a without a parachute.

Instance Variables
	startHeight:		<Number>

startHeight
	- the height a whom he started falling. Needed to determine the critical height a which the Penguin is goning to die, when he hits the ground.
!

!PgiFallingFreely methodsFor: 'accessing' stamp: 'kh 1/13/2011 20:01:54.589'!
appearance

	^#PenguinFalling
! !

!PgiFallingFreely methodsFor: 'transitions' stamp: 'lsb 2/10/2011 20:18:42.96'!
decideTransition 

	(self hasGround)
		ifTrue: [(self fatalImpact)
			ifTrue: [(self changeTo: PgiBitingTheDust)]
			ifFalse: [(self changeTo: PgiWalking)]]
		ifFalse: [self carryOn]! !

!PgiFallingFreely methodsFor: 'transitions' stamp: 'pre 2/13/2011 22:50'!
enter
	"setup free fall state"

	super enter.
	self startHeight: self penguin position y.! !

!PgiFallingFreely methodsFor: 'perception' stamp: 'rs 2/18/2011 23:33:03.851'!
fatalImpact
	"determine whether the falling height was fatal"
	| fatalHeight |
	
	fatalHeight := 200.

	^((self startHeight - ((self penguin position) + (self collisionObject vector)) y) abs) > fatalHeight! !

!PgiFallingFreely methodsFor: 'accessing' stamp: 'pre 1/9/2011 16:58:14.155'!
startHeight
	"Answer the value of startHeight"

	^ startHeight! !

!PgiFallingFreely methodsFor: 'accessing' stamp: 'pre 1/9/2011 16:58:14.163'!
startHeight: anObject
	"Set the value of startHeight"

	startHeight := anObject! !

PgiFalling subclass: #PgiParachuterFalling
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiParachuterFalling commentStamp: 'RS 2/11/2011 00:49:25.906' prior: 0!
A PgiParachuterFalling is the State in which the Penguin wisely usses a parachute to glide down infinite heights and distances.

Instance Variables
!

!PgiParachuterFalling methodsFor: 'accessing' stamp: 'pre 1/12/2011 14:35:51.328'!
appearance 
	^#PenguinParachuterFalling! !

!PgiParachuterFalling methodsFor: 'progressing' stamp: 'rs 2/18/2011 18:54:51.223'!
carryOn 
	
	| latencyFactor |

	latencyFactor := 0.5.
 	
	self collisionObject vector: (self collisionObject vector x)@(latencyFactor * self collisionObject vector y).
	super carryOn.! !

PgiPenguinState subclass: #PgiJetPackSide
	instanceVariableNames: 'vector counter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiJetPackSide commentStamp: 'rs 2/19/2011 00:11:59.675' prior: 0!
A PgiJetPackSide is a Penguin flying sideways with the jetpack.

Instance Variables
	counter:		<aNumber>
	vector:		<aPoint>

counter
	- counts the number of cycles, and allows to determine when the fuel of the jetpack is empty.

vector
	- direction and distance the penguin flys sideways per step.
!

!PgiJetPackSide methodsFor: 'accessing' stamp: 'rs 2/18/2011 17:16:11.458'!
appearance
	(self direction x > 0) 
		ifTrue: [^#jetPackSideRight]
		ifFalse: [^#jetPackSideLeft]! !

!PgiJetPackSide methodsFor: 'progressing' stamp: 'rs 2/18/2011 17:21:40.38'!
carryOn 
	
	(vector) ifNil: [vector := (self direction sign * (5@0))].
	counter := counter + 1.
	
	self penguin moveBy: vector.! !

!PgiJetPackSide methodsFor: 'transitions' stamp: 'rs 2/18/2011 22:00:45.07'!
decideTransition 

	(counter = 20) ifTrue: [self changeTo: PgiParachuterFalling].
	self carryOn.
	 
	(self hasBarrier) 
			ifTrue: [	self 
						turnAround;
						changeTo: PgiWalking] 
			ifFalse: [ self carryOn. ]! !

!PgiJetPackSide methodsFor: 'transitions' stamp: 'rs 2/18/2011 17:17:14.965'!
enter

	super enter.
	counter := 0.! !

PgiPenguinState subclass: #PgiJetPackUp
	instanceVariableNames: 'vector counter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiJetPackUp commentStamp: 'rs 2/19/2011 00:14:06.558' prior: 0!
A PgiJetPackUp is a Penguin flying upwards with a Jetpack.

Instance Variables
	counter:		<aNumber>
	vector:		<aPoint>

counter
	- counts the number of cycles the Penguin has gone through. After this has reached a sertain size he starts flying sideways, to find a platform he can savely land on.

vector
	- direction and distance he moves upwards.
!

!PgiJetPackUp methodsFor: 'accessing' stamp: 'rs 2/18/2011 17:10:45.475'!
appearance
	(self direction x > 0) 
		ifTrue: [^#jetPackUpRight]
		ifFalse: [^#jetPackUpLeft]! !

!PgiJetPackUp methodsFor: 'progressing' stamp: 'rs 2/18/2011 17:14:21.793'!
carryOn 
	
	(vector) ifNil: [vector := 0@ -10].
	counter := counter + 1.
	
	self penguin moveBy: vector.! !

!PgiJetPackUp methodsFor: 'transitions' stamp: 'rs 2/18/2011 17:14:14.406'!
decideTransition 

	(counter = 20) ifTrue: [self changeTo: PgiJetPackSide].
	self carryOn.
	 
	(self hasBarrier) 
			ifTrue: [	self 
						turnAround;
						changeTo: PgiWalking] 
			ifFalse: [ self carryOn. ]! !

!PgiJetPackUp methodsFor: 'transitions' stamp: 'rs 2/18/2011 17:14:36.597'!
enter

	super enter.
	counter := 0.! !

!PgiPenguinState class methodsFor: 'creation' stamp: 'pre 2/15/2011 10:02:18.662'!
from: aPreviousState
	"Create a new state copying common attributes from the previous state."

	| newState |
	newState := self new.
	newState 
		direction: aPreviousState direction.
	
	^newState 	
		"Insert additional knowledge transfer here or override"! !

!PgiPenguinState methodsFor: 'transitions' stamp: 'pre 2/13/2011 22:56'!
allowedTransitionTo: aStateClass
	"Override this in subclass to define a list of states allowed to transform to.
	Use a Bag with a the allowed states and check whether it includes aStateClass. If it doesn't, return 	false."

	^true! !

!PgiPenguinState methodsFor: 'accessing' stamp: 'lsb 2/16/2011 18:43:52.541'!
appearance

	self subclassResponsibility.! !

!PgiPenguinState methodsFor: 'progressing' stamp: 'tm 12/27/2010 01:05:17.414'!
carryOn
	"perform the main activity in this state regardless of any circumstances"

	self subclassResponsibility.! !

!PgiPenguinState methodsFor: 'transitions' stamp: 'lsb 2/15/2011 02:27:09.954'!
changeTo: aStateClass
	"Template Method that triggers a state transition. Override allowedTransitionTo: aStateClass in subclasses to define a list of allowed states."
	(self allowedTransitionTo: aStateClass) ifTrue: [ 
		self leave.
		(self penguin) changeStateTo: (aStateClass from: self).
		^true].
	^false! !

!PgiPenguinState methodsFor: 'accessing' stamp: 'RS 2/3/2011 22:14:18.958'!
collisionObject
	"Answer the value of collisionObject"

	^ collisionObject! !

!PgiPenguinState methodsFor: 'accessing' stamp: 'RS 2/3/2011 22:14:18.964'!
collisionObject: anObject
	"Set the value of collisionObject"

	collisionObject := anObject! !

!PgiPenguinState methodsFor: 'transitions' stamp: 'lsb 2/10/2011 20:08:35.854'!
decideTransition

	self subclassResponsibility.! !

!PgiPenguinState methodsFor: 'accessing' stamp: 'TM 12/26/2010 21:08:34.283'!
direction
	"Answer the value of direction"

	^ direction! !

!PgiPenguinState methodsFor: 'accessing' stamp: 'tm 12/28/2010 02:52:01.606'!
direction: aNumber
	"Set the value of direction. Recommendet values 1 for right, -1 for left orientation"

	direction := aNumber! !

!PgiPenguinState methodsFor: 'transitions' stamp: 'pre 1/9/2011 15:42:47.957'!
enter
	"perform post-initialization steps after the state is attached to its penguin"
	self penguin changeAppearance: (self appearance ).! !

!PgiPenguinState methodsFor: 'perception' stamp: 'RS 2/3/2011 22:36:17.22'!
hasBarrier
	
	^(self collisionObject type = #mountainObstacle or: self collisionObject type = #bigObstacle) 
! !

!PgiPenguinState methodsFor: 'perception' stamp: 'lsb 2/8/2011 10:01:58.746'!
hasDeadlyObject
	
	^(self collisionObject type = #bitingTheDust) 
! !

!PgiPenguinState methodsFor: 'perception' stamp: 'RS 2/3/2011 22:18:09.324'!
hasGround
	^(self collisionObject vector x = 0 and: (self collisionObject vector y = self penguin gravityVector y)) not
! !

!PgiPenguinState methodsFor: 'transitions' stamp: 'pre 1/12/2011 14:12:52.668'!
leave
	"gets called before state is left"
	^self! !

!PgiPenguinState methodsFor: 'accessing' stamp: 'TM 12/25/2010 19:52:28.747'!
penguin
	"Answer the value of penguin"

	^ penguin! !

!PgiPenguinState methodsFor: 'accessing' stamp: 'TM 12/25/2010 19:52:28.842'!
penguin: anObject
	"Set the value of penguin"

	penguin := anObject! !

!PgiPenguinState methodsFor: 'progressing' stamp: 'lsb 2/16/2011 19:03:43.327'!
proceed
	"Since most states will change a penguin state to biting the dust when it walks over a deadly morph, proceed is implemented as template method. If you want to build a state immune to deadly morphs override proceed there accordingly."
	
	self collisionObject: (self penguin calculateMovementInDistance: (self direction x)).
	(self hasDeadlyObject)	
				ifTrue:[self changeTo: PgiBitingTheDust]
				ifFalse:[self decideTransition.] 
	

! !

!PgiPenguinState methodsFor: 'transitions' stamp: 'pre 1/9/2011 16:01:46.387'!
turnAround
	"change orientation towards the other side ('substate transition').
	Order matters here, because self appearance is direction sensitive."
	self direction: (self direction x negated)@(self direction y).
	self penguin changeAppearance: (self appearance).! !

PgiPenguinState subclass: #PgiWalking
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiWalking commentStamp: 'RS 2/11/2011 12:14:00.713' prior: 0!
A PgiWalking is the standart walking class the penguin usually resides in. 

Instance Variables
!

PgiWalking subclass: #PgiDigging
	instanceVariableNames: 'tunnel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiDigging commentStamp: 'pre 2/19/2011 01:33:22.665' prior: 0!
A PgiDigging is a digging Penguin.

Instance Variables
	tunnel:		<PgiTunnelHalo>

tunnel
	- reference to the tunnel halo that the Penguin is currently building.
!

!PgiDigging methodsFor: 'accessing' stamp: 'pre 2/12/2011 14:57:59.818'!
appearance
	(self direction x > 0) 
		ifTrue: [^#PenguinDigRight]
		ifFalse: [^#PenguinDigLeft]
	! !

!PgiDigging methodsFor: 'progressing' stamp: 'pre 2/12/2011 17:47:05.374'!
carryOn
	self tunnel extendBy: (self direction x).
	self collisionObject: (self penguin calculateMovementInDistance: (self direction x)).
	self penguin moveBy: (self collisionObject vector).! !

!PgiDigging methodsFor: 'transitions' stamp: 'pre 2/11/2011 11:18:18.35'!
decideTransition 
	
	(self tunnel doesTunnelEnd: (self direction x)) ifTrue: [ 
		self tunnel finishTunnel.
		self changeTo: PgiWalking. ]
	ifFalse: [self carryOn. ]! !

!PgiDigging methodsFor: 'transitions' stamp: 'pre 2/19/2011 17:10:24.592'!
enter
	| aPolygon |
	
	super enter.
	aPolygon := ((self penguin world collisionObjects) detect: [ :each |
			each containsPoint: 
				((self penguin collisionPoint: #middle 
								inDirection: (self direction x)) + ((self direction x sign)*(10@0))).
		] ifNone: [ self direction: (self direction negated). self changeTo: PgiWalkingDigger. ^self ] ). 
	
	self tunnel: PgiTunnelHalo new.
	(self tunnel) 
		underground: aPolygon;
		direction: ((self direction x)@0);
		world: (self penguin world).
	
	"The message is wrapped in an if statement to catch if digging is impossible."
	(self tunnel buildTunnelOn: (self tunnel underground )
				withTop: (self penguin getEdgeFrom: (self penguin topMiddle) to: (self direction x))
				withBottom: (self penguin getEdgeFrom: (self penguin bottomMiddle) to: (self direction x)))
	ifFalse: [ self direction: (self direction negated). self changeTo: PgiWalkingDigger ] ! !

!PgiDigging methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:20:34.902'!
tunnel
	"Answer the value of tunnel"

	^ tunnel! !

!PgiDigging methodsFor: 'accessing' stamp: 'pre 2/6/2011 13:20:34.918'!
tunnel: anObject
	"Set the value of tunnel"

	tunnel := anObject! !

PgiWalking subclass: #PgiMining
	instanceVariableNames: 'mine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiMining commentStamp: 'pre 2/19/2011 01:34:24.721' prior: 0!
A PgiDigging is a digging Penguin.

Instance Variables
	tunnel:		<PgiMineHalo>

tunnel
	- reference to the tunnel object that the Penguin is currently building.
!

!PgiMining methodsFor: 'accessing' stamp: 'pre 2/12/2011 19:59:09.643'!
appearance
	^#PenguinMining
	! !

!PgiMining methodsFor: 'progressing' stamp: 'pre 2/18/2011 22:36:51.821'!
carryOn
	self mine extendBy: 3.
	self collisionObject: (self penguin calculateMovementInDistance: 0).
	self penguin moveBy: (self collisionObject vector).! !

!PgiMining methodsFor: 'transitions' stamp: 'pre 2/18/2011 22:39:51.847'!
decideTransition 
	
	(self mine doesMineEndInDepth: 3) ifTrue: [ 
		self mine finishMine.
		self changeTo: PgiWalking. ]
	ifFalse: [self carryOn. ]! !

!PgiMining methodsFor: 'transitions' stamp: 'pre 2/19/2011 03:13:03.243'!
enter
	| aPolygon |
	
	super enter.
	aPolygon := ((self penguin world collisionObjects) detect: [ :each |
			(each containsPoint: (self penguin gravityPoint: #left)) 
				or: (each containsPoint: (self penguin gravityPoint: #right)).
	]).

	self mine: PgiMineHalo new.
	
	(self mine) underground: aPolygon;
				world: (self penguin world).
	
	(self mine buildMineOn: (self mine underground)
				fromLeft: (self penguin bottomLeft)
				toRight: (self penguin bottomRight))
	ifFalse: [ self changeTo: PgiWalking ].! !

!PgiMining methodsFor: 'accessing' stamp: 'pre 2/18/2011 22:38:41.885'!
mine
	"Answer the value of mine"

	^ mine! !

!PgiMining methodsFor: 'accessing' stamp: 'pre 2/18/2011 22:38:41.898'!
mine: anObject
	"Set the value of mine"

	mine := anObject! !

PgiWalking subclass: #PgiParachuterWalking
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiParachuterWalking commentStamp: 'RS 2/11/2011 12:16:16.239' prior: 0!
A PgiParachuterWalking is a parachute penguin currently not falling down.

Instance Variables
!

!PgiParachuterWalking methodsFor: 'accessing' stamp: 'pre 1/12/2011 14:18:36.762'!
appearance 
	"return direction dependent walking appearance"
	(self direction x > 0) 
		ifTrue: [^#PenguinParachuterWalkRight]
		ifFalse: [^#PenguinParachuterWalkLeft]! !

!PgiParachuterWalking methodsFor: 'transitions' stamp: 'lsb 2/10/2011 21:19:22.659'!
decideTransition 

	(self hasGround) 
		ifFalse: [self changeTo: PgiParachuterFalling] 
		ifTrue: [	(self hasBarrier) 
			ifTrue: [	self turnAround] 
			ifFalse: [ self carryOn. ]]
! !

!PgiWalking methodsFor: 'accessing' stamp: 'kh 1/12/2011 04:15:28.224'!
appearance
	"return direction dependent walking appearance"
	
	"Using dictionary or polymorphism is a little overkill here."
	(self direction x > 0) 
		ifTrue: [^#PenguinWalkRight]
		ifFalse: [^#PenguinWalkLeft]
	! !

!PgiWalking methodsFor: 'progressing' stamp: 'pre 2/15/2011 00:21:25.942'!
carryOn
	"move penguin forward"

	self penguin moveBy: self collisionObject vector.! !

!PgiWalking methodsFor: 'transitions' stamp: 'lsb 2/10/2011 20:06:17.069'!
decideTransition

	(self hasGround) 
		ifFalse: [self changeTo: PgiFallingFreely] 
		ifTrue: [	
			(self hasBarrier) 
				ifTrue: [	self turnAround ] 
				ifFalse: [ self carryOn. ]]! !

PgiWalking subclass: #PgiWalkingBridge
	instanceVariableNames: 'offset piecesLeft'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiWalkingBridge commentStamp: 'pre 2/19/2011 01:34:47.892' prior: 0!
A PgiWalkingBridge is a BridgeBuilder currentyl not placing a bridge pieces and walking some steps further to place a next piece.

Instance Variables
	offset:		<Number>
	piecesLeft:		<Number>

offset
	- determines how far he has yet moved. From this the state determines when he is going to switch back to PgiBuildingBridge.

piecesLeft
	- the number of pieces that can still be placed by the Builder.
!

!PgiWalkingBridge class methodsFor: 'creation' stamp: 'RS 1/18/2011 21:00:26.141'!
from: aState

	| result |
	
	result := super from: aState.
	
	result piecesLeft: aState piecesLeft.
	
	^result! !

!PgiWalkingBridge methodsFor: 'accessing' stamp: 'kh 1/18/2011 19:51:43.17'!
appearance

	(self direction x > 0) 
		ifTrue: [^#PenguinWalkingBridgeRight]
		ifFalse: [^#PenguinWalkingBridgeLeft]
! !

!PgiWalkingBridge methodsFor: 'transitions' stamp: 'rs 2/18/2011 23:34:50.329'!
decideTransition 

	| distanceToWalk |
	
	distanceToWalk := 20.

	super decideTransition.
	((self direction sign x)*(self penguin position x - self offset) >= distanceToWalk) 
				ifFalse: [self carryOn]
				ifTrue: [(self piecesLeft == 0) ifTrue: [self changeTo: PgiWalking]
							                            ifFalse: [self changeTo: PgiBuildingBridge]]! !

!PgiWalkingBridge methodsFor: 'transitions' stamp: 'RS 1/16/2011 01:05:43.472'!
enter

	super enter.
	self offset: self penguin position x.! !

!PgiWalkingBridge methodsFor: 'accessing' stamp: 'RS 1/18/2011 20:20:46.274'!
offset
	"Answer the value of offset"

	^ offset! !

!PgiWalkingBridge methodsFor: 'accessing' stamp: 'RS 1/18/2011 20:20:46.282'!
offset: anObject
	"Set the value of offset"

	offset := anObject! !

!PgiWalkingBridge methodsFor: 'accessing' stamp: 'RS 1/18/2011 20:49:31.919'!
piecesLeft
	"Answer the value of piecesLeft"

	^ piecesLeft! !

!PgiWalkingBridge methodsFor: 'accessing' stamp: 'RS 1/18/2011 20:49:31.923'!
piecesLeft: anObject
	"Set the value of piecesLeft"

	piecesLeft := anObject! !

PgiWalking subclass: #PgiWalkingDigger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic-States'!

!PgiWalkingDigger commentStamp: 'RS 2/11/2011 12:19:44.189' prior: 0!
A PgiWalkingDigger is a digging Penguin that is currently not digging because he hasnt found an appropriate wall yet.

Instance Variables
!

!PgiWalkingDigger methodsFor: 'transitions' stamp: 'lsb 2/10/2011 21:19:47.504'!
decideTransition 

	(self hasGround) 
		ifFalse: [self changeTo: PgiFallingFreely] 
		ifTrue: [	(self hasBarrier) 
			ifTrue: [	self changeTo: PgiDigging] 
			ifFalse: [ self carryOn. ]]! !

Object subclass: #PgiResource
	instanceVariableNames: 'cachedObject origin'
	classVariableNames: 'All Directory Extensions'
	poolDictionaries: ''
	category: 'Pgi-Resources'!

!PgiResource commentStamp: 'tm 2/19/2011 01:40:03.274' prior: 0!
A PgiResource is the superclass providing unified, cached and transparent access to all external resources. Instances of this class are uniquely named and available on class side.

Instance Variables
	cachedObject:		<Object>
	origin:		<Object>

cachedObject
	- The cached squeak representation of a loaded resource.

origin
	- A representation where the resource is stored. Type is defined by subclass.
!

!PgiResource class methodsFor: 'instance variables' stamp: 'pre 1/22/2013 20:08'!
All

	^ All! !

!PgiResource class methodsFor: 'instance variables' stamp: 'pre 1/22/2013 20:04'!
All: all

	All := all! !

!PgiResource class methodsFor: 'class initialization' stamp: 'pre 1/22/2013 20:04'!
initialize
	"self initialize"
	
	"do not call super here!! 
	seems like this messes things up on class side"
	
	self 
		initializeExtensions;
		initializeDirectory
		"resetAll"! !

!PgiResource class methodsFor: 'class initialization' stamp: 'tm 2/15/2011 01:05:21.187'!
initializeDirectory
	"self initializeDirectory"
	
	Directory := FileDirectory default / 'Penguins'! !

!PgiResource class methodsFor: 'class initialization' stamp: 'tm 2/16/2011 22:38:54.03'!
initializeExtensions
	"Map file extensions to their respective class representations"

	"PgiResource initializeExtensions"

	Extensions := Dictionary new.
	Extensions
		add: 'jpg' -> Form;
		add: 'png' -> Form;
		add: 'svg' -> PgiLevel.! !

!PgiResource class methodsFor: 'instance creation' stamp: 'tm 2/17/2011 00:33:02.176'!
locateFileFor: aSymbol
	
	Extensions keysDo: [:ea | | filename |
		filename := aSymbol asString, '.', ea.
		(Directory fileExists: filename)
			ifTrue: [ ^ Directory fullNameFor: filename ]
	].

	^ nil! !

!PgiResource class methodsFor: 'instance creation' stamp: 'tm 2/16/2011 23:26:28.042'!
locateFolderFor: aSymbol

	| folder |
	folder := Directory / (aSymbol asString).
	folder exists 
		ifTrue: [^ folder]
		ifFalse: [^ nil]! !

!PgiResource class methodsFor: 'instance variables' stamp: 'tm 2/17/2011 01:11:36.194'!
named: aSymbol
	"Retrieves or creates the named instance of the desired resource wrapper"
	
	^ All at: aSymbol ifAbsentPut: (self newNamed: aSymbol)! !

!PgiResource class methodsFor: 'instance creation' stamp: 'tm 2/17/2011 01:06:57.728'!
newNamed: aSymbol
	"Here we decide which subclass will represent the resource"
	| origin |
	
	(origin := (self locateFolderFor: aSymbol))
		ifNotNil: [^PgiMultiResource originatingFrom: origin] 
		ifNil: [
			(origin := self locateFileFor: aSymbol)
				ifNotNil: [^PgiSingleResource originatingFrom: origin]
				ifNil: [(KeyNotFound key: aSymbol) signal ]
		]
				! !

!PgiResource class methodsFor: 'instance creation' stamp: 'tm 2/16/2011 23:54:52.852'!
originatingFrom: anObject
	"Creates an instance of the concrete (sub)class and sets the origin"
	| concreteInstance |
	
	concreteInstance := self basicNew.
	^concreteInstance
		origin: anObject;
		initialize;
		yourself! !

!PgiResource class methodsFor: 'class initialization' stamp: 'pre 2/17/2011 18:25:46.457'!
resetAll
	"Resets the cache of named instances"
	
	All := Dictionary new.
	self initializeDirectory.! !

!PgiResource class methodsFor: 'instance variables' stamp: 'tm 2/17/2011 01:12:53.373'!
retrieve: aSymbol
	"Retrieve the concrete named resource"

	^(self named: aSymbol)	retrieve
! !

!PgiResource methodsFor: 'accessing' stamp: 'tm 2/12/2011 21:02:13.838'!
cachedObject
	"Answer the value of cachedObject"

	^ cachedObject! !

!PgiResource methodsFor: 'accessing' stamp: 'tm 2/12/2011 21:02:13.903'!
cachedObject: anObject
	"Set the value of cachedObject"

	cachedObject := anObject! !

!PgiResource methodsFor: 'caching' stamp: 'tm 2/17/2011 00:22:37.125'!
load
	"Part of a template method."
	
	self subclassResponsibility! !

!PgiResource methodsFor: 'accessing' stamp: 'tm 2/12/2011 21:05:05.586'!
origin
	"Answer the value of origin"

	^ origin! !

!PgiResource methodsFor: 'accessing' stamp: 'tm 2/16/2011 23:23:33.28'!
origin: anObject
	"Set the value of origin"

	origin := anObject! !

!PgiResource methodsFor: 'caching' stamp: 'tm 2/17/2011 00:21:51.383'!
retrieve
	"Retrieves the content of this resource. Loads if necessary and caches."
	
	^ self cachedObject ifNil: [self load]! !

PgiResource subclass: #PgiSingleResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Resources'!

!PgiSingleResource commentStamp: 'tm 2/19/2011 01:38:08.245' prior: 0!
A PgiSingleResource is a wrapper for an external resource consisting of a file.

Instance Variables
!

PgiSingleResource subclass: #PgiMultiResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Resources'!

!PgiMultiResource commentStamp: 'tm 2/19/2011 01:39:16.832' prior: 0!
A PgiMultiResource is a wrapper for a resource consisting of multiple files placed within a folder.
It is based on the file loading mechanisms of PgiSingleResource and therefore inherits them.

Instance Variables
!

!PgiMultiResource methodsFor: 'caching' stamp: 'tm 2/18/2011 22:37:38.999'!
load
	"Load the content of my directory and return as array"

	^self 
		cachedObject: ((
			(self origin entries sorted: [:a :b | a name <= b name])
				collect:	[:ea | self loadSingleFile: ea fullName]) 
				reject: [:ea | ea isNil]);
		cachedObject! !

!PgiSingleResource methodsFor: 'caching' stamp: 'tm 2/17/2011 00:24:25.116'!
load
	"Loads the content of the original file into the cache and returns it"
	
	"Let it be a single-element-array for uniform handling of single- and
	multi-resources!!"
	^self 
		cachedObject: {self loadSingleFile: self origin};
		cachedObject! !

!PgiSingleResource methodsFor: 'caching' stamp: 'tm 2/17/2011 02:24:43.444'!
loadSingleFile: aFileName
	
	| extension |
	extension := FileDirectory extensionFor: aFileName.
	^ (Extensions at: extension ifAbsent: [^nil]) fromFileNamed: aFileName! !

Object subclass: #PgiRessourceManager
	instanceVariableNames: 'cachedToolUsages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Resources'!

!PgiRessourceManager commentStamp: 'tm 2/19/2011 01:44:10.586' prior: 0!
A PgiRessourceManager is the interface to all resources. It provides commonly used objects.

Instance Variables
	cachedToolUsages:		<Object>

cachedToolUsages
	- Preserves the number of available tool usages when a level is requestet for later use. This avoids requesting the full PgiLevel again when the world has been built and is now initializing the buttons.
	
!

!PgiRessourceManager methodsFor: 'accessing' stamp: 'kh 2/11/2011 16:35:54.531'!
cachedToolUsages

	^ cachedToolUsages! !

!PgiRessourceManager methodsFor: 'accessing' stamp: 'kh 2/18/2011 23:34:06.221'!
cachedToolUsages: aDictionary

	cachedToolUsages := aDictionary! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'tm 2/17/2011 01:34:59.471'!
loadLevel: aLevelNumber

	^(PgiResource retrieve: ('level', aLevelNumber asString) asSymbol) first! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'tm 2/17/2011 02:07:38.4'!
requestAppearance: aSymbol

	^ (PgiResource retrieve: aSymbol)! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'kh 2/18/2011 23:36:20.053'!
requestHomeButtonsFor: aMenu
	"Setup routine for building home Buttons"

	| homeButtons |
		
	"add additional buttons here"	
	homeButtons := (OrderedCollection newFrom: 
		{({#ImageExitButton . 660@520 . #ExitGame})}).
							
	^homeButtons collect: [:each |
		(PgiButton new)
			image: ((self requestAppearance: (each first)) first);
			position: (each at: 2);
			purpose: (each at: 3);
			menu: aMenu;
			yourself ]
	! !

!PgiRessourceManager methodsFor: 'accessing' stamp: 'tm 2/19/2011 00:38:53.18'!
requestHomeScreenObjects
	"Comprehensive initialization routine for home screen objects"

	| resultHomeScreenObjects homeScreenObjects 
	starSky starMorph starDecoration|
	
	resultHomeScreenObjects := Array new: 4.
	homeScreenObjects := (OrderedCollection newFrom: 
		{  ({#ImageTitle . 190@5 . 4}).
			({#ImageEarth . 190@120 . 3}).
			({#ImageBackgroundHome . 0@0 . 1}).}).
															
	homeScreenObjects do: [:each |  |aDecoration|
		aDecoration := PgiDecorationObject new.
		aDecoration 
			appearance: ((self requestAppearance: (each first)) first) asMorph;
			position: (each second).
		resultHomeScreenObjects at: (each third) put: aDecoration].
			
	starSky  := ((self requestAppearance: #ImageStarSky) first) asMorph.	
	starSky beSticky.
	starMorph :=((TransformationMorph new) addMorph: starSky).
	starMorph beSticky.

	starDecoration := PgiDecorationObject new.
	starDecoration 
		appearance:  starMorph;
		position: 0@0.
	resultHomeScreenObjects at: 2 put: starDecoration.					
	^ resultHomeScreenObjects! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'kh 2/11/2011 16:39:57.133'!
requestLevel: aLevelNumber
	
	| currentLevel |
	
	currentLevel := self loadLevel: aLevelNumber.
	self cachedToolUsages: currentLevel toolUsages.
	
	^currentLevel ! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'kh 2/18/2011 23:26:56.838'!
requestLevelButtonsFor: aMenu 

	| buttons resultButtons |

	resultButtons := Dictionary new.
			
	buttons :=  (OrderedCollection newFrom: {  ({#ImagePauseGame . 554@0 . #PauseGame}).
												 ({#ImagePlayGame . 610@0 . #PlayGame}).
												 ({#ImageRunFaster . 666@0 . #RunFaster}). 
												 ({#ImageFinalizeLevel . 722@0 . #FinalizeLevel}).}).
												
	buttons do: [:each| |button| 
					button:= PgiButton  new.
					button
						image: ((self requestAppearance: (each first)) first);
						position: (each at: 2);
						purpose: (each at: 3);
						menu: aMenu.
					resultButtons at: button purpose put: button. ].
					
	^ resultButtons! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'kh 2/18/2011 23:33:17.028'!
requestLevelStartButtonsFor: aMenu
	"Comprehensive initialization routine for level buttons"
	| buttons resultButtons|
	
	resultButtons := OrderedCollection new.
	
	buttons := (OrderedCollection newFrom: {   ({#ImageButtonLevel1 . 440@150 . #LoadLevel1 . 1}).
								  				 ({#ImageButtonLevel2 . 530@260 . #LoadLevel2 . 2}).
												 ({#ImageButtonLevel3 . 450@290 . #LoadLevel3 . 3}).
												 ({#ImageButtonLevel4 . 440@400 . #LoadLevel4 . 4}).
			 					    			   	 ({#ImageButtonLevel5 . 300@400 . #LoadLevel5 . 5}).
												 ({#ImageButtonLevel6 . 250@300 . #LoadLevel6 . 6}).
												 ({#ImageButtonLevel7 . 300@160 . #LoadLevel7 . 7})}).
	
	buttons do: [:each| |button| 
						 button:= PgiLevelStartButton new.
						 button
								image: ((self requestAppearance: (each first)) first);
								position: (each at: 2);
								purpose: (each at: 3);
								levelNumber: (each at: 4);
								menu: aMenu;
								cover: ((self requestAppearance: #overlayLevelStartButton) at:1) asMorph.
 		 			      resultButtons add: button.].		
	^ resultButtons
	! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'kh 2/18/2011 23:28:37.902'!
requestLoadScreenObjects
	
	| resultLoadScreenObjects loadScreenObjects |
	
	resultLoadScreenObjects := Array new: 4.
	
	loadScreenObjects := (OrderedCollection newFrom: {  ({#ImageLoadingScreen . 0@0 . 4}).
															({#ImageEarth . 190@120 . 3}).
															({#ImageBackgroundHome . 0@0 . 1}).
															({#ImageStarSky . 0@0 . 2})}).

	loadScreenObjects do: [:each |  |aDecoration|
									    aDecoration := PgiDecorationObject new.
									    aDecoration appearance:  ((self requestAppearance: (each first)) first) asMorph;
													position: (each at: 2).
									     resultLoadScreenObjects at: (each at: 3) put: aDecoration.].
											
	^ resultLoadScreenObjects! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'kh 2/18/2011 23:27:51.269'!
requestLooseButtonsFor: aMenu

	| resultHomeButtons homeButtons |
	
	resultHomeButtons := OrderedCollection new.
		
	homeButtons := (OrderedCollection newFrom: { ({#ImageRetryButton . 304@300 . #TryAgain}).
									  				  ({#ImageHomeButton . 304@400 . #HomeMenu})}).
							
	homeButtons do: [:each| |button| 
							  button:= PgiButton  new.
								button
									image: ((self requestAppearance: (each first)) first);
									position: (each at: 2);
									purpose: (each at: 3);
									menu: aMenu.
								resultHomeButtons add: button.].			
	^ resultHomeButtons
	! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'kh 2/18/2011 23:33:29.685'!
requestLooseScreenObjects
	"Comprehensive initialization routine for you-loose-screen"
	
	| resultLoadScreenObjects loadScreenObjects |
	
	resultLoadScreenObjects := Array new: 4.

	loadScreenObjects := (OrderedCollection newFrom: {  ({#ImageEarth . 190@120 . 3}).
															({#ImageBackgroundHome . 0@0 . 1}).
															({#ImageStarSky . 0@0 . 2}).
															({#ImageLooseScreen. 0@0. 4})}).

	loadScreenObjects do: [:each |  |aDecoration|
									    aDecoration := PgiDecorationObject new.
									    aDecoration appearance:  ((self requestAppearance: (each at: 1)) at:1) asMorph;
													position: (each at: 2).
									     resultLoadScreenObjects at: (each at: 3) put: aDecoration.].
											
	^ resultLoadScreenObjects! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'kh 2/18/2011 23:28:51.453'!
requestSuccessButtonsFor: aMenu

	| resultHomeButtons homeButtons |
	
	resultHomeButtons := OrderedCollection new.
		
	homeButtons := (OrderedCollection newFrom: { ({#ImageContinueButton . 304@360 . #Continue}).
									  				  ({#ImageHomeButton . 304@460 . #HomeMenu})}).
		
	homeButtons do: [:each| |button| 
							  button:= PgiButton  new.
								button
									image: ((self requestAppearance: (each first)) first);
									position: (each at: 2);
									purpose: (each at: 3);
									menu: aMenu.
								resultHomeButtons add: button.].			
	^ resultHomeButtons
	! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'kh 2/18/2011 23:33:40.565'!
requestSuccessScreenObjects
	"Comprehensive initialization routine for success-screen"
	
	| resultLoadScreenObjects loadScreenObjects |
	
	resultLoadScreenObjects := Array new: 4.
	
	loadScreenObjects := (OrderedCollection newFrom: {  ({#ImageEarth . 190@120 . 3}).
															({#ImageBackgroundHome . 0@0 . 1}).
															({#ImageStarSky . 0@0 . 2}).
															({#ImageSuccessScreen. 0@0. 4})}).

	loadScreenObjects do: [:each |  |aDecoration|
									    aDecoration := PgiDecorationObject new.
									    aDecoration appearance:  ((self requestAppearance: (each at: 1)) at:1) asMorph;
													position: (each at: 2).
									     resultLoadScreenObjects at: (each at: 3) put: aDecoration.].
											
	^ resultLoadScreenObjects! !

!PgiRessourceManager methodsFor: 'request data' stamp: 'kh 2/18/2011 23:33:48.155'!
requestToolButtonsFor: aMenu
	"Comprehensive initialization routine for level tool buttons"

	| buttons resultButtons buttonsY buttonWidth offset index  |
	
	buttonWidth := 73.
	buttonsY := 0.
	offset := 0.
	index := 0.
	resultButtons := Dictionary new.	

	buttons :=  (OrderedCollection newFrom: {  ({#ImageButtonTool1 . #Walker}).
								 				 ({#ImageButtonTool2 . #Blocker}).
												 ({#ImageButtonTool3 . #Digger}).
												 ({#ImageButtonTool4 . #Miner}).
			 									 ({#ImageButtonTool5 . #Glider}).
												 ({#ImageButtonTool6 . #Jetpack}).
												 ({#ImageButtonTool7 . #Builder}).}).
	buttons do: [:each| |button |
					button := PgiToolButton new.					
					button
						image: ((self requestAppearance: (each at: 1)) first);
						position: (offset+(index*buttonWidth))@buttonsY;
						purpose: (each at: 2);
						menu: aMenu;
						marker: ((self requestAppearance: #overlayToolButton) first) asMorph.
					index := index+1.
					resultButtons at: button purpose put: button].
					
	^ resultButtons! !

!PgiRessourceManager methodsFor: 'accessing' stamp: 'kh 2/11/2011 16:36:28.573'!
requestToolUsagesFor: aLevelNumber

	^ cachedToolUsages! !

Object subclass: #PgiSarCreator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Setup'!

!PgiSarCreator methodsFor: 'as yet unclassified' stamp: 'tm 1/18/2011 22:32:03.906'!
assemble: aTargetFile
	"write everything the game needs to a SAR archive"
	
	ZipArchive new
		addString: self grabSource as: self packageName, '.mcz';
		addString: self preamble as: 'install/preamble';
		writeToFileNamed: aTargetFile.
	

	! !

!PgiSarCreator methodsFor: 'as yet unclassified' stamp: 'tm 1/18/2011 22:17:05.201'!
grabSource
	"extract source from current working copy. Will show a commit window."

	| stream |
	stream := RWBinaryOrTextStream on: (String new).
	((MCWorkingCopy forPackage: (MCPackage new name: self packageName)) newVersion) fileOutOn: stream.
	^stream contents! !

!PgiSarCreator methodsFor: 'as yet unclassified' stamp: 'tm 1/18/2011 21:29:39.966'!
packageName
	"answer the name of the package being processed"

	^'Pgi'! !

!PgiSarCreator methodsFor: 'as yet unclassified' stamp: 'tm 1/18/2011 22:30:34.677'!
preamble
	"comment stating purpose of message"

	^'self fileInMonticelloZipVersionNamed: ''', self packageName, '.mcz''.'! !

Object subclass: #PgiScroller
	instanceVariableNames: 'scrollSensors objectToScroll responsive game'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiScroller commentStamp: 'RS 2/11/2011 11:17:42.871' prior: 0!
A PgiScroller is controlling its Scrollsensors and moves its assigned scrollable accordingly.

Instance Variables
	downScroller:		<PgiScroller>
	game:		<PgiGame>
	leftScroller:		<PgiScroller>
	responsive:		<Boolean>
	rightScroller:		<PgiScroller>
	scrollable:		<PgiStage>
	scrollers:		<OrderedCollection>
	upScroller:		<PgiScroller>
	world:		<PgiWorld>

downScroller
	- xxxxx

game
	- xxxxx

leftScroller
	- xxxxx

responsive
	- xxxxx

rightScroller
	- xxxxx

scrollable
	- xxxxx

scrollers
	- xxxxx

upScroller
	- xxxxx

world
	- xxxxx
!

!PgiScroller class methodsFor: 'accessing' stamp: 'kh 2/17/2011 19:18:45.246'!
distance

	^15! !

!PgiScroller class methodsFor: 'instance creation' stamp: 'kh 2/18/2011 14:21:51.279'!
newPgiScrollerOn: aGame withExtent: aPoint at: aPosition

	| scroller up down left right sensorWidth verticalSensorsHight |
	
	sensorWidth := 40.
	verticalSensorsHight := 440.
	
	scroller := PgiScroller new. 
	scroller game: aGame;
			objectToScroll: aGame scrollablePart.

	up := PgiScrollSensor new.
	up extent: aPoint x @ sensorWidth;
	     topLeft: aPosition;
	     movementVector: 0 @ self distance.
	
	down := PgiScrollSensor new.
	down extent:  (aPoint x) @ sensorWidth;
		   bottomLeft: aPosition +  (0 @ aPoint y);
		   movementVector: 0 @ (-1 * self distance) .
		
	left := PgiScrollSensor new.
	left    extent: sensorWidth @ verticalSensorsHight;
		   topLeft: aPosition + (0 @ sensorWidth);
		   movementVector: self distance @ 0 .
		
	right := PgiScrollSensor new.
	right  extent: sensorWidth @ verticalSensorsHight;
		   topRight: aPosition + (aPoint x @ sensorWidth);
		   movementVector: (-1 * self distance ) @ 0 .
			
	scroller scrollSensors: {up . down . left . right}.
	
	scroller scrollSensors do: [:scrollSensor | scroller game addMorph: scrollSensor.
											   scrollSensor openInWorld;
															beSticky].
	
	^scroller! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 2/17/2011 23:28:52.403'!
abandon

	self scrollSensors do: [:scrollSensor | scrollSensor abandon.]! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 2/18/2011 00:20:37.639'!
calculateShiftedPosition

	| movement |
	
	movement := self scrollSensors inject: (0@0) into: [: sum :scrollsensor | sum + scrollsensor movement].
	
	^ self objectToScroll position + movement! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:20:04.872'!
game

	^ game! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:20:12.346'!
game: aPgiGame

	game := aPgiGame! !

!PgiScroller methodsFor: 'initialize-release' stamp: 'kh 2/17/2011 23:10:14.886'!
initialize 
	
	super initialize.
	self responsive: true.! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 2/18/2011 00:04:41.765'!
isOnStage: aRectangle

	^ ((aRectangle topLeft <= self game position) 
	    and: (aRectangle bottomRight >= (self game position + self game screenSize)))
! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 2/17/2011 23:35:59.629'!
objectToScroll

	^ objectToScroll! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 2/17/2011 23:36:16.684'!
objectToScroll: aPasteUpMorph

	objectToScroll := aPasteUpMorph! !

!PgiScroller methodsFor: 'stepping' stamp: 'kh 2/18/2011 00:21:27.345'!
pgiStep
	
	| newPosition objectToScrollShifted |

	newPosition := self calculateShiftedPosition.
	objectToScrollShifted := newPosition extent: ( self objectToScroll width)@(self objectToScroll height).
	
	(self isOnStage: objectToScrollShifted) ifTrue: [self objectToScroll position: newPosition].! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:20:26.245'!
responsive

	^ responsive! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 2/18/2011 00:05:32.475'!
responsive: aBoolean
	"start or stop checking the scrollSensors"
	
	responsive := aBoolean.
	
! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 2/17/2011 23:36:45.32'!
scrollSensors

	^ scrollSensors! !

!PgiScroller methodsFor: 'accessing' stamp: 'kh 2/17/2011 23:37:11.518'!
scrollSensors: aCollection

	scrollSensors := aCollection! !

Object subclass: #PgiStage
	instanceVariableNames: 'visualPart gameState size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiStage commentStamp: 'RS 2/11/2011 11:23:40.884' prior: 0!
A PgiStage is the Logical structure of the area where the actual game objects are drawn.

Instance Variables
	size:		<Object>
	state:		<Object>
	visualPart:		<Object>

size
	- is the size of the area.

state
	- reference to the game state that initiated this Stage.

visualPart
	- the pasteupmorph where all LevelObjects and DecorationObejcts are actually drawn on.
!

PgiStage subclass: #PgiFinalizeStage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiFinalizeStage commentStamp: 'rs 2/18/2011 21:45:55.176' prior: 0!
A PgiFinalizeStage is the stage, that consists all the visuals to ask the user how to go on after he has quit a level.

Instance Variables
!

!PgiFinalizeStage methodsFor: 'initialize' stamp: 'pre 2/19/2011 03:12:31.866'!
initializeStage

	| loadScreenObjects |
	
	super initializeStage.
	
	(self gameState successful == true) 
		ifTrue: [loadScreenObjects := self gameState ressourceManager requestSuccessScreenObjects]
		ifFalse: [loadScreenObjects := self gameState ressourceManager requestLooseScreenObjects].
										
	loadScreenObjects do: [:each | self addElement: each appearance at: each position]. 	
! !

PgiStage subclass: #PgiHomeStage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiHomeStage commentStamp: 'rs 2/19/2011 00:08:52.937' prior: 0!
A PgiHomeStage is Logical structure of the area where the home menu is drawn.

Instance Variables
!

!PgiHomeStage methodsFor: 'creation and cleanup' stamp: 'rs 2/19/2011 00:29:35.559'!
abandon

	super abandon.! !

!PgiHomeStage methodsFor: 'accessing' stamp: 'rs 2/18/2011 22:43:51.368'!
initializeStage

	| homeScreenObjects|
	
	super initializeStage.
	
	homeScreenObjects := self gameState ressourceManager requestHomeScreenObjects.	
	homeScreenObjects do: [:each | self addElement: each appearance at: each position]. 
	
	! !

PgiStage subclass: #PgiLoadingStage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiLoadingStage commentStamp: 'rs 2/18/2011 21:50:32.784' prior: 0!
A PgiLoadingStage is containing all the elements that the user is presented with while the level is loading.

Instance Variables
!

!PgiLoadingStage methodsFor: 'initialize' stamp: 'kh 2/18/2011 14:25:11.276'!
initializeStage

	| loadScreenObjects |
	
	super initializeStage.
	
	loadScreenObjects := self gameState ressourceManager requestLoadScreenObjects.	
	loadScreenObjects do: [:each | self addElement: each appearance at: each position]. 	
! !

!PgiStage methodsFor: 'creation and cleanup' stamp: 'kh 2/17/2011 17:48:15.581'!
abandon

	self visualPart abandon.
! !

!PgiStage methodsFor: 'accessing' stamp: 'rs 2/16/2011 15:53:35.763'!
addElement: aMorph at: aPoint

	aMorph beSticky;
			position: ((self visualPart) topLeft + aPoint).
			
	self visualPart addMorph: aMorph.! !

!PgiStage methodsFor: 'accessing' stamp: 'kh 2/18/2011 14:22:06.005'!
gameState

	^ gameState! !

!PgiStage methodsFor: 'accessing' stamp: 'kh 2/18/2011 14:22:21.811'!
gameState: aPgiGameState

	gameState := aPgiGameState! !

!PgiStage methodsFor: 'initialize-release' stamp: 'kh 2/17/2011 17:29:07.697'!
initialize 

	super initialize.
	self visualPart: PasteUpMorph new.
	self visualPart borderWidth: 0;
				     beSticky.! !

!PgiStage methodsFor: 'creation and cleanup' stamp: 'kh 2/18/2011 14:24:23.331'!
initializeStage 

	self visualPart extent: self gameState extent;
				    position: self gameState position.! !

!PgiStage methodsFor: 'creation and cleanup' stamp: 'RS 2/11/2011 16:19:14.863'!
launch 

	self gameState game addMorph: visualPart.! !

!PgiStage methodsFor: 'accessing' stamp: 'lsb 2/3/2011 20:08:26.59'!
size

	^size! !

!PgiStage methodsFor: 'accessing' stamp: 'lsb 2/3/2011 20:01:44.947'!
size: aPoint

	size := aPoint.
! !

!PgiStage methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:22:31.455'!
visualPart

	^ visualPart! !

!PgiStage methodsFor: 'accessing' stamp: 'kh 2/18/2011 14:22:35.844'!
visualPart: aMorph

	visualPart := aMorph! !

!PgiStage methodsFor: 'creation and cleanup' stamp: 'lsb 2/3/2011 20:01:37.491'!
worldSize: aPoint
	
	self size: aPoint.
	visualPart width: (aPoint x);
			   height: (aPoint y).
	! !

PgiStage subclass: #PgiWorld
	instanceVariableNames: 'allObjects allPenguins collisionObjects deadlyObjects penguinsToSpawn penguinsToRescue goal penguinsSpawned penguinsRescued levelNumber factory gravity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiWorld commentStamp: 'pre 2/19/2011 01:45:35.844' prior: 0!
A PgiWorld is the environment in which all PgiWorldObject live. This is the play stage and it cares about all its objects..

Instance Variables
	allObjects:		<Set>
	allPenguins:		<Set>
	collisionObjects:		<Set>
	deadlyObjects:		<Set>
	factory:		<PgiFactory>
	goal:		<PgiGoal>
	gravity:		<Number>
	levelNumber:		<Number>
	penguinsRescued:		<Number>
	penguinsSpawned:		<Number>
	penguinsToRescue:		<Number>
	penguinsToSpawn:		<Number>

allObjects
	- a Set that stores all Objects living in the world.

allPenguins
	- the horde of penguins running around. For all penguin operations like finalize.

collisionObjects
	- the Set of objects penguins should collide with.

deadlyObjects
	- the Set of objects which will kill penguins.

factory
	- the PgiFactory providing our world with all kinds of PgiWorldObjects.

goal
	- the Goal in the World

gravity
	- a Number determining the strength of the gravity force in PenguinNewton also called Pixel.

levelNumber
	- the current level in the world.

penguinsRescued
	- how many penguins are already rescued.

penguinsSpawned
	- how many penguins are already spawned.

penguinsToRescue
	- how many penguins does one have to save in this level.

penguinsToSpawn
	- how many penguins should live in this world.
!

!PgiWorld methodsFor: 'accessing' stamp: 'kh 2/17/2011 17:55:42.818'!
abandon

	super abandon.
! !

!PgiWorld methodsFor: 'accessing' stamp: 'pre 12/20/2010 14:39:32.369'!
allObjects
	"Answer the value of allObjects"

	^ allObjects! !

!PgiWorld methodsFor: 'accessing' stamp: 'pre 12/20/2010 14:39:32.468'!
allObjects: anObject
	"Set the value of allObjects"

	allObjects := anObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'pre 1/13/2011 20:11:53.678'!
allPenguins
	"Answer the value of allPenguins"

	^ allPenguins! !

!PgiWorld methodsFor: 'accessing' stamp: 'pre 1/13/2011 20:11:53.688'!
allPenguins: anObject
	"Set the value of allPenguins"

	allPenguins := anObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'pre 12/20/2010 14:39:32.5'!
collisionObjects
	"Answer the value of collisionObjects"

	^ collisionObjects! !

!PgiWorld methodsFor: 'accessing' stamp: 'pre 12/20/2010 14:39:32.534'!
collisionObjects: anObject
	"Set the value of collisionObjects"

	collisionObjects := anObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'lsb 2/6/2011 03:15:10.943'!
deadlyObjects
	"Answer the value of deadlyObjects"

	^ deadlyObjects! !

!PgiWorld methodsFor: 'accessing' stamp: 'lsb 2/6/2011 03:15:11.023'!
deadlyObjects: anObject
	"Set the value of deadlyObjects"

	deadlyObjects := anObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'RS 1/9/2011 22:32:42.206'!
factory
	"Answer the value of factory"

	^ factory! !

!PgiWorld methodsFor: 'accessing' stamp: 'RS 1/9/2011 22:32:42.235'!
factory: anObject
	"Set the value of factory"

	factory := anObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'lsb 12/24/2010 17:21:19.5'!
goal
	"Answer the value of goal"

	^ goal! !

!PgiWorld methodsFor: 'accessing' stamp: 'lsb 12/24/2010 17:21:19.603'!
goal: anObject
	"Set the value of goal"

	goal := anObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'pre 1/13/2011 19:11:49.39'!
gravity
	"Answer the value of gravity"

	^ gravity! !

!PgiWorld methodsFor: 'accessing' stamp: 'pre 1/13/2011 19:11:49.404'!
gravity: anObject
	"Set the value of gravity"

	gravity := anObject! !

!PgiWorld methodsFor: 'initialize-release' stamp: 'lsb 2/18/2011 18:13:27.037'!
initialize 
	super initialize.
	self allObjects: Set new;
		 collisionObjects: Set new;
		 allPenguins: Set new;
		 deadlyObjects: Set new;
		 penguinsRescued: 0; 
		 penguinsSpawned: 0;		
	       gravity: 10.
	
! !

!PgiWorld methodsFor: 'accessing' stamp: 'kh 12/26/2010 17:42:52.84'!
levelNumber
	"Answer the value of levelNumber"

	^ levelNumber! !

!PgiWorld methodsFor: 'accessing' stamp: 'kh 12/26/2010 17:42:52.913'!
levelNumber: anObject
	"Set the value of levelNumber"

	levelNumber := anObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:24:54.401'!
penguinsRescued
	"Answer the value of penguinsRescued"

	^ penguinsRescued! !

!PgiWorld methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:24:54.437'!
penguinsRescued: anObject
	"Set the value of penguinsRescued"

	penguinsRescued := anObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'kh 12/26/2010 16:27:35.289'!
penguinsSpawned
	"Answer the value of penguinsSpawned"

	^ penguinsSpawned! !

!PgiWorld methodsFor: 'accessing' stamp: 'kh 12/26/2010 16:27:35.322'!
penguinsSpawned: anObject
	"Set the value of penguinsSpawned"

	penguinsSpawned := anObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:24:54.321'!
penguinsToRescue
	"Answer the value of penguinsToRescue"

	^ penguinsToRescue! !

!PgiWorld methodsFor: 'accessing' stamp: 'kh 2/16/2011 05:24:54.37'!
penguinsToRescue: anObject
	"Set the value of penguinsToRescue"

	penguinsToRescue := anObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'pre 12/20/2010 14:39:32.664'!
penguinsToSpawn
	"Answer the value of penguinsToSpawn"

	^ penguinsToSpawn! !

!PgiWorld methodsFor: 'accessing' stamp: 'pre 12/20/2010 14:39:32.7'!
penguinsToSpawn: anObject
	"Set the value of penguinsToSpawn"

	penguinsToSpawn := anObject! !

!PgiWorld methodsFor: 'administration' stamp: 'kh 1/12/2011 12:07:09.335'!
registerCollisionObject: aWorldObject

	self collisionObjects add: aWorldObject! !

!PgiWorld methodsFor: 'accessing' stamp: 'lsb 2/10/2011 22:42:05.191'!
registerDeadlyObject: aDeadlyObject

	self deadlyObjects add: aDeadlyObject! !

!PgiWorld methodsFor: 'administration' stamp: 'RS 2/11/2011 15:41:03.994'!
registerPenguin: aPenguin

	self allPenguins add: aPenguin.! !

!PgiWorld methodsFor: 'administration' stamp: 'lsb 2/16/2011 02:07:19.363'!
registerWorldObject: aWorldObject

	self allObjects add: aWorldObject.! !

!PgiWorld methodsFor: 'administration' stamp: 'kh 2/16/2011 05:15:05.116'!
reportRescuedPenguin

	self penguinsRescued: (self penguinsRescued + 1).
	self gameState updateNumberOfRescuedPenguins.! !

!PgiWorld methodsFor: 'administration' stamp: 'RS 2/11/2011 16:22:01.565'!
reportSpawnedPenguin

	self penguinsSpawned: (self penguinsSpawned + 1).
	self gameState updateNumberOfSpawnedPenguins.! !

!PgiWorld methodsFor: 'administration' stamp: 'pre 1/13/2011 19:58:26.436'!
unregisterCollisionObject: aWorldObject

	self collisionObjects remove: aWorldObject ifAbsent: [].
! !

!PgiWorld methodsFor: 'administration' stamp: 'kh 2/17/2011 17:43:27.573'!
unregisterPenguin: aPenguin

	self allPenguins remove: aPenguin ifAbsent: [].
	self allObjects remove: aPenguin ifAbsent: [].
	self gameState unsubscribe: aPenguin.! !

!PgiWorld methodsFor: 'administration' stamp: 'kh 2/17/2011 17:43:32.113'!
unregisterWorldObject: aWorldObject

	self allObjects remove: aWorldObject ifAbsent: []! !

Object subclass: #PgiStepper
	instanceVariableNames: 'toBeStepped world cycle stepTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiStepper commentStamp: 'rs 2/18/2011 21:51:13.829' prior: 0!
A PgiStepper is the central stepping unit to which all Objects that want to be stepped can subscribe. they can additionally supply a step frequency which determines how often the Object will be stepped at a time.

Instance Variables
	cycle:		<Object>
	stepTime:		<Object>
	toBeStepped:		<Bag>
	world:		<PgiWorld>

cycle
	- xxxxx

stepTime
	- xxxxx

toBeStepped
	- Set of objects that are supposed to be stepped.

world
	- xxxxx
!

!PgiStepper methodsFor: 'creation and cleanup' stamp: 'pre 1/18/2011 18:33:18.264'!
abandon

	| w |
	w := self world.
	w ifNotNil: [w stopStepping: self selector: #step].! !

!PgiStepper methodsFor: 'accessing' stamp: 'RS 2/5/2011 18:16:13.884'!
baseStepTime

	^10! !

!PgiStepper methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:24:33.617'!
cycle

	^ cycle! !

!PgiStepper methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:24:38.689'!
cycle: anObject

	cycle := anObject! !

!PgiStepper methodsFor: 'initialize-release' stamp: 'rs 2/18/2011 23:51:50.427'!
initialize

	super initialize.
	self cycle: 0;
		stepTime: 10.
	self world: (self currentWorld);
		 toBeStepped: Dictionary new.! !

!PgiStepper methodsFor: 'accessing' stamp: 'pre 2/13/2011 18:07:15.653'!
maxCycle

	^10000! !

!PgiStepper methodsFor: 'stepping' stamp: 'kh 1/19/2011 02:24:19.471'!
pause 

	| w |
	w := self world.
	w ifNotNil: [w stopStepping: self selector: #step].! !

!PgiStepper methodsFor: 'initialize-release' stamp: 'kh 1/18/2011 22:22:05.344'!
resume
	self stepTime: (self baseStepTime).
	self startStepping ! !

!PgiStepper methodsFor: 'stepping' stamp: 'pre 1/18/2011 17:48'!
shouldGetStepsFrom: aWorld
	^self world == aWorld! !

!PgiStepper methodsFor: 'initialize-release' stamp: 'lsb 2/12/2011 17:19:25.601'!
speedUp 
	self resume.
	self stepTime: (((1/5)*(self baseStepTime )) rounded).! !

!PgiStepper methodsFor: 'stepping' stamp: 'RS 12/28/2010 13:55:28.355'!
startStepping
	"Start getting sent the 'step' message."
	self startStepping: #step at: Time millisecondClockValue arguments: nil stepTime: nil.! !

!PgiStepper methodsFor: 'stepping' stamp: 'RS 12/28/2010 13:56:25.187'!
startStepping: aSelector at: scheduledTime arguments: args stepTime: stepTime
	"Start stepping the receiver"
	| w |
	w := self world.
	w ifNotNil: [
		w startStepping: self at: scheduledTime selector: aSelector arguments: args stepTime: stepTime.
		self changed].! !

!PgiStepper methodsFor: 'stepping' stamp: 'lsb 2/16/2011 02:29:11.404'!
step

	self cycle: (((self cycle = (self maxCycle)) ifTrue: [1] ifFalse: [self cycle + 1])).
	((self toBeStepped keys) select: 
	[ :each |  ((self cycle) \\ (toBeStepped at: each)) = 0  ]) 
			do: 
			[:each | each pgiStep].! !

!PgiStepper methodsFor: 'stepping' stamp: 'kh 1/18/2011 21:52:42.662'!
stepTime

	^stepTime! !

!PgiStepper methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:24:42.222'!
stepTime: anObject

	stepTime := anObject! !

!PgiStepper methodsFor: 'stepping' stamp: 'lsb 2/3/2011 21:55:10.039'!
subscribe: aSteppable every: numberOfSteps

	(numberOfSteps > (self maxCycle )) 
		ifTrue: [ Error new signal: ('Step frequency is invalid. Should not be greater than ' + (self maxCycle asString ))].
	self toBeStepped at: aSteppable put: numberOfSteps.! !

!PgiStepper methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:24:46.431'!
toBeStepped

	^ toBeStepped! !

!PgiStepper methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:24:50.507'!
toBeStepped: anArrayOfObjects

	toBeStepped := anArrayOfObjects.! !

!PgiStepper methodsFor: 'stepping' stamp: 'lsb 2/4/2011 16:17:29.333'!
unsubscribe: anObject
	self toBeStepped removeKey: anObject ifAbsent: [Error new signal:'Object wurde nicht gesteppt'].! !

!PgiStepper methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:24:55.6'!
world

	^ world! !

!PgiStepper methodsFor: 'accessing' stamp: 'kh 1/19/2011 02:25:03.728'!
world: aPgiWorld

	world := aPgiWorld! !

Object subclass: #PgiTestObject
	instanceVariableNames: 'stepped'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Test'!

!PgiTestObject methodsFor: 'as yet unclassified' stamp: 'lsb 2/4/2011 17:02:29.987'!
initialize 

	self stepped: false.! !

!PgiTestObject methodsFor: 'as yet unclassified' stamp: 'lsb 2/4/2011 17:02:42.282'!
pgiStep

	self stepped: true.! !

!PgiTestObject methodsFor: 'accessing' stamp: 'lsb 2/4/2011 17:01:44.843'!
stepped
	"Answer the value of stepped"

	^ stepped! !

!PgiTestObject methodsFor: 'accessing' stamp: 'lsb 2/4/2011 17:01:44.91'!
stepped: anObject
	"Set the value of stepped"

	stepped := anObject! !

Object subclass: #PgiWorldBuilder
	instanceVariableNames: 'buildingWorld gameState resourceManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-Managing'!

!PgiWorldBuilder commentStamp: 'rs 2/19/2011 00:26:34.777' prior: 0!
A PgiWorldBuilder is composing the different parts of a level by delegating instantaition to the worlds Factory and sticking the parts together.

Instance Variables
	buildingWorld:		<aPgiWorld>
	gameState:		<aPgiGamestate>
	resourceManager:		<aPgiResourceManager>

buildingWorld
	- the World, e.g. level, that the Builder is composing together, provides acess to the Factory.

gameState
	- the games state, providing all the nessecary infomration to instantiate the level.

resourceManager
	- the data source, from which the LevelBuilders requests the level Specification. He also provides the worlds factory with this referance, to supply all the needed resource Objects. 
!

!PgiWorldBuilder methodsFor: 'building world' stamp: 'rs 2/16/2011 15:19:38.217'!
buildWorld: aLevelNumber
	"The central method for building a World. Every single step is envoked here one after another."
	
	| aLevel |
	
	aLevel := (self resourceManager requestLevel: aLevelNumber).
	
	self 	initializeBuildingWorld: aLevel;
			placeDecoratorMorphs: (aLevel decorationObjects);
			createWorldObjects: (aLevel levelObjects );
			placeWell: (aLevel penguinWell );
			placeGoal: (aLevel penguinGoal ).
	
	^buildingWorld! !

!PgiWorldBuilder methodsFor: 'accessing' stamp: 'kh 2/18/2011 14:27:56.284'!
buildingWorld

	^ buildingWorld! !

!PgiWorldBuilder methodsFor: 'accessing' stamp: 'kh 2/18/2011 14:28:00.016'!
buildingWorld: anObject

	buildingWorld := anObject! !

!PgiWorldBuilder methodsFor: 'building world' stamp: 'RS 2/12/2011 00:09:34.313'!
createWorldObjects: levelObjectCollection 
	"This will place all the neat level objects and register them as collision	Objects. "
	
	levelObjectCollection
		do: [:aLevelObject | buildingWorld factory build: aLevelObject].! !

!PgiWorldBuilder methodsFor: 'accessing' stamp: 'kh 2/18/2011 14:28:04.133'!
gameState

	^ gameState! !

!PgiWorldBuilder methodsFor: 'accessing' stamp: 'kh 2/18/2011 14:28:11.308'!
gameState: aPgiGameState

	gameState := aPgiGameState! !

!PgiWorldBuilder methodsFor: 'building world' stamp: 'kh 2/18/2011 00:46:47.564'!
initializeBuildingWorld: aLevel
	"This method creates an empty World and sets the basic world information."

	buildingWorld := PgiWorld new.

	buildingWorld visualPart position: (self gameState position + aLevel startPosition).
	buildingWorld worldSize: (aLevel worldSize);
					gameState: (self gameState);
					levelNumber: aLevel levelNumber;
					penguinsToSpawn: (aLevel penguinsToSpawn);
					penguinsSpawned: 0;
					penguinsToRescue: (aLevel penguinsToRescue);	
					penguinsRescued: 0;
					factory: (PgiFactory newOn: buildingWorld withRecourceManager: self resourceManager).

					! !

!PgiWorldBuilder methodsFor: 'building world' stamp: 'pre 2/19/2011 03:13:06.891'!
placeDecoratorMorphs: aDecorationsCollection
	"This method places all the Objects that wont be collisionable."
	"TODO: We could think of a layer structure for them via sortedCollection"

	aDecorationsCollection do: [ :aDecoration | 
		buildingWorld visualPart addMorph: (buildingWorld factory build: aDecoration)].! !

!PgiWorldBuilder methodsFor: 'building world' stamp: 'RS 2/12/2011 17:04:34.319'!
placeGoal: aGoal
	
	| newGoal |
	
	newGoal := buildingWorld factory build: aGoal.
	
      self gameState stepper subscribe: newGoal every: 70. ! !

!PgiWorldBuilder methodsFor: 'building world' stamp: 'RS 2/11/2011 21:00:53.602'!
placeWell: aWell
	
	| tmpWell |	
		
	tmpWell  := buildingWorld factory build: aWell.
	
	tmpWell penguinBuilder: buildingWorld factory.
	
	self gameState stepper subscribe: tmpWell every: 50. 
! !

!PgiWorldBuilder methodsFor: 'accessing' stamp: 'kh 2/18/2011 14:27:51.265'!
resourceManager

	^ resourceManager! !

!PgiWorldBuilder methodsFor: 'accessing' stamp: 'kh 2/18/2011 14:27:46.608'!
resourceManager: anObject

	resourceManager := anObject! !

Object subclass: #PgiWorldObject
	instanceVariableNames: 'appearance world position shape morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic'!

!PgiWorldObject commentStamp: 'RS 2/11/2011 12:09:30.12' prior: 0!
A PgiWorldObject is the class for all logical representations of the Objects in the level.

Instance Variables
	appearance:		<symbol>
	morph:		<PgiWorldObjectMorph / PgiTexturedMorph>
	position:		<Point>
	shape:		<PolygonMorph>
	world:		<PgiWorld>

appearance
	- xxxxx

morph
	- xxxxx

position
	- xxxxx

shape
	- xxxxx

world
	- xxxxx
!

PgiWorldObject subclass: #PgiPenguin
	instanceVariableNames: 'state rescued collisionMethod'
	classVariableNames: 'ToolMapping'
	poolDictionaries: ''
	category: 'Pgi-LevelLogic'!

!PgiPenguin commentStamp: 'pre 2/19/2011 02:42:35.815' prior: 0!
A PgiPenguin is the logical component of a Penguin. It has a state which determins its behavior. Furthermore it forwards collision requests to the CollisionDetectionMethod.

Instance Variables
	collisionMethod: <PgiCollisionDetectionMethod>
	rescued:  <Boolean>
	state:		<PgiPenguinState>

collisionMethod
	- The collisionMethod which the penguin calls to determine his collisions.

rescued 
	- shows whether the penguin was already rescued so it isnt rescued several times.

state
	- the penguins state. This is essential for the penguins behavior. It determines what it will do during the next step.

!

!PgiPenguin class methodsFor: 'class initialization' stamp: 'rs 2/18/2011 17:24:32.166'!
initialize 

	super initialize.
	ToolMapping := (Dictionary newFrom: 
						{#Walker -> PgiWalking.
						 #Blocker -> PgiBlocking.
						 #Digger -> PgiWalkingDigger.
						 #Miner -> PgiMining.
						 #Glider -> PgiParachuterWalking.
						 #Jetpack -> PgiJetPackUp.
						 # Builder -> PgiBuildingBridge.
						}
					  ).
				
! !

!PgiPenguin class methodsFor: 'levelobject-creation' stamp: 'pre 2/13/2011 17:37:43.017'!
levelObject
	^ (PgiLevelObject new) edges: {0@0 . 24@0 . 24@35 . 0@35 };
							type: #Penguin;
							appearance: #PenguinWalkRight;
							position: 0@0! !

!PgiPenguin class methodsFor: 'levelobject-creation' stamp: 'pre 2/13/2011 17:35:18.457'!
levelObjectWithPosition: aPoint

	| resultLevelObject |
	
	resultLevelObject := self levelObject.
	resultLevelObject position: (resultLevelObject position) + aPoint.

	^resultLevelObject! !

!PgiPenguin class methodsFor: 'tool-mapping' stamp: 'kh 1/13/2011 18:41:57.446'!
lookUpStateClassFor: currentTool

	^ ToolMapping at: currentTool! !

!PgiPenguin methodsFor: 'creation and cleanup' stamp: 'pre 2/19/2011 10:57:12.582'!
abandon
	self morph abandon.
	self world unregisterPenguin: self.
	! !

!PgiPenguin methodsFor: 'collision' stamp: 'pre 2/19/2011 02:20:39.155'!
calculateMovementInDistance: aNumber
	
	self collisionMethod computationDistance: aNumber.
						
	^self collisionMethod compute! !

!PgiPenguin methodsFor: 'state-penguin-morph' stamp: 'pre 1/9/2011 15:27:23.375'!
changeAppearance: anAppearanceSymbol
	self appearance: anAppearanceSymbol.
	self morph changeAppearance: anAppearanceSymbol.! !

!PgiPenguin methodsFor: 'state-penguin-morph' stamp: 'pre 2/18/2011 21:39:58.5'!
changeStateTo: aState
	"Set a new state, performing necessary updates on both sides."
	
	(self state) ifNotNil: [ self state leave].

	aState penguin: self.
	self state: aState.
	
	self state enter.
! !

!PgiPenguin methodsFor: 'accessing' stamp: 'pre 2/19/2011 02:20:11.918'!
collisionMethod
	"Answer the value of collisionMethod"

	^ collisionMethod! !

!PgiPenguin methodsFor: 'accessing' stamp: 'pre 2/19/2011 02:20:11.965'!
collisionMethod: anObject
	"Set the value of collisionMethod"

	collisionMethod := anObject! !

!PgiPenguin methodsFor: 'collision' stamp: 'pre 2/19/2011 02:35:30.376'!
collisionPoint: aSymbol inDirection: aNumber

	^ self collisionMethod collisionPoint: aSymbol inDirection: aNumber
! !

!PgiPenguin methodsFor: 'accessing' stamp: 'rs 2/18/2011 23:27:12.879'!
getEdgeFrom: aPoint to: aDirection

	^ ((aPoint x + (self width * 0.5 *(aDirection sign))) rounded)@((aPoint y) rounded)! !

!PgiPenguin methodsFor: 'accessing' stamp: 'pre 2/19/2011 03:27:01.489'!
gravityPoint: aSymbol

	| resultingPoint |
	
	(aSymbol = #left) ifTrue: [ resultingPoint := (self bottomLeft + (self gravityVector )) ].
	(aSymbol = #right) ifTrue: [ resultingPoint := (self bottomRight + (self gravityVector ))].
	
	^resultingPoint! !

!PgiPenguin methodsFor: 'accessing' stamp: 'pre 2/19/2011 03:24:18.656'!
gravityVector 
	^0@(self world gravity)! !

!PgiPenguin methodsFor: 'accessing' stamp: 'RS 2/11/2011 18:56:06.796'!
handleClickEvent 

	| currentTool |
	
	currentTool := self world gameState selectedTool.
	
	(currentTool ~= #None) ifTrue: [
		(self state changeTo: (PgiPenguin lookUpStateClassFor: currentTool)) ifTrue: 
				[self world gameState toolHasBeenApplied: currentTool.]].
! !

!PgiPenguin methodsFor: 'initialize-release' stamp: 'pre 2/19/2011 02:36:10.052'!
initialize 
	
	super initialize.
	
	self state: (PgiWalking new).
	(self state) direction: (3@0);
				penguin: self.
				
	self collisionMethod: PgiCollisionDetectionMethod new.
	self collisionMethod penguin: self.
				
	self
		 shape: ((PolygonMorph new setVertices: {0@0 . 24@0 . 24@35 . 0@35 }) borderWidth: 0);
		 position: 0@0;
		 rescued: false.
.
	! !

!PgiPenguin methodsFor: 'state-penguin-morph' stamp: 'pre 1/9/2011 15:59:21.413'!
moveBy: aDirection
	self position: (self position + aDirection ).
	self morph moveBy: aDirection.! !

!PgiPenguin methodsFor: 'stepping' stamp: 'pre 2/19/2011 01:49:34.787'!
pgiStep
	"Invoke penguin move according to current strategy. If Penguins leave the world, they are abandoned. 
	Bias is a displacement so that this happens out of sight of the player. "
	| bias |
	
	bias := 15@15.
	
	(( ((0@0 - bias) <= self position) and: ((self world size + bias) > self position) ) not) 
	ifTrue: [ self abandon.
			^self ].
		
	self state proceed.
	
	self morph update.
	(self rescued) ifFalse: [
		(self reachedGoal)
			ifTrue: [
					self rescued: true.
					self world goal arrivedPenguin: self]]! !

!PgiPenguin methodsFor: 'state-penguin-morph' stamp: 'pre 2/19/2011 03:20:58.742'!
reachedGoal 
	"Check whether the Pinguin has reached the goal. A little bias is permitted.."
	
	^self world goal didReachGoal: self

	
	! !

!PgiPenguin methodsFor: 'accessing' stamp: 'kh 1/12/2011 12:16:07.936'!
rescued
	"Answer the value of rescued"

	^ rescued! !

!PgiPenguin methodsFor: 'accessing' stamp: 'kh 1/12/2011 12:16:07.996'!
rescued: anObject
	"Set the value of rescued"

	rescued := anObject! !

!PgiPenguin methodsFor: 'accessing' stamp: 'TM 12/26/2010 18:39:53.34'!
state
	"Answer the value of state"

	^ state! !

!PgiPenguin methodsFor: 'accessing' stamp: 'TM 12/26/2010 18:39:53.527'!
state: anObject
	"Set the value of state"

	state := anObject! !

PgiWorldObject subclass: #PgiPenguinGoal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic'!

!PgiPenguinGoal commentStamp: 'rs 2/19/2011 00:16:38.339' prior: 0!
A PgiPenguinGoal is a the object, that the Penguins have to reach to be saved!!

Instance Variables
!

!PgiPenguinGoal methodsFor: 'functionality' stamp: 'pre 2/19/2011 11:27:19.933'!
arrivedPenguin: aPenguin

	self world reportRescuedPenguin.
	aPenguin abandon.! !

!PgiPenguinGoal methodsFor: 'functionality' stamp: 'pre 2/19/2011 11:23:31.13'!
didReachGoal: aPenguin

	^((self shape bounds scaleBy: 0.8) translateBy: (self position)) containsPoint: (aPenguin position)
	! !

!PgiPenguinGoal methodsFor: 'accessing' stamp: 'rs 2/13/2011 01:16:47.326'!
handleClickEvent
	
	^self! !

!PgiPenguinGoal methodsFor: 'stepping' stamp: 'RS 2/12/2011 16:56:46.189'!
pgiStep

	self morph update.! !

PgiWorldObject subclass: #PgiPenguinWell
	instanceVariableNames: 'penguinBuilder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pgi-LevelLogic'!

!PgiPenguinWell commentStamp: 'rs 2/19/2011 00:17:33.809' prior: 0!
A PgiPenguinWell is the Object that spawns all the penguins.

Instance Variables
	penguinBuilder:		<PgiFactory>

penguinBuilder
	- reference to the worlds Factory that the well resides in.
!

!PgiPenguinWell methodsFor: 'accessing' stamp: 'rs 2/13/2011 01:16:41.991'!
handleClickEvent

	^self! !

!PgiPenguinWell methodsFor: 'accessing' stamp: 'lsb 1/6/2011 17:15:06.35'!
penguinBuilder
	"Answer the value of penguinBuilder"

	^ penguinBuilder! !

!PgiPenguinWell methodsFor: 'accessing' stamp: 'lsb 1/6/2011 17:15:06.426'!
penguinBuilder: anObject
	"Set the value of penguinBuilder"

	penguinBuilder := anObject! !

!PgiPenguinWell methodsFor: 'stepping' stamp: 'pre 2/13/2011 17:42:51.636'!
pgiStep
	
	| penguinObject penguin|
	
	((self world penguinsSpawned) <( self world penguinsToSpawn) ) ifTrue: [
		penguin := PgiPenguin levelObjectWithPosition: (self bottomLeft - (0@(self height / 2))).
		
		"Carefull, order matters:"
		self world reportSpawnedPenguin. 
		penguinObject  := (self penguinBuilder build: penguin).
		self registerPenguin: penguinObject.]! !

!PgiPenguinWell methodsFor: 'functionality' stamp: 'RS 2/11/2011 18:54:37.199'!
registerPenguin: aPenguinObject
	"If You register the Penguin in more Collections, don't forget to update PgiPenguin -> abandon to
	enable proper deletion."	
	
	(self world) registerPenguin: aPenguinObject. 
	(self world) gameState stepper subscribe: aPenguinObject every: 7! !

!PgiWorldObject class methodsFor: 'levelobject-creation' stamp: 'pre 2/13/2011 17:45:50.323'!
levelObject
	^ PgiLevelObject new appearance: #none;
					    edges: {};
					    position: 0@0;
					    type: #Standard! !

!PgiWorldObject class methodsFor: 'levelobject-creation' stamp: 'pre 2/13/2011 17:46:43.232'!
levelObjectWithPosition: aPoint
	| resultLevelObject |
	
	resultLevelObject := self levelObject.
	resultLevelObject position: aPoint.
	
	^resultLevelObject! !

!PgiWorldObject class methodsFor: 'instance creation' stamp: 'tm 1/7/2011 00:10:47.884'!
within: aWorld
	"Create a world object and set given argument"

	| newWorldObject |
	
	newWorldObject := self basicNew.
	newWorldObject 
		world: aWorld;
		initialize.
	
	^newWorldObject! !

!PgiWorldObject methodsFor: 'creation and cleanup' stamp: 'kh 2/17/2011 17:43:40.69'!
abandon 

	self shape abandon.
	self morph abandon.
	! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 12/21/2010 13:33:55.218'!
appearance
	"Answer the value of appearance"

	^ appearance! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 12/21/2010 13:33:55.302'!
appearance: anObject
	"Set the value of appearance"

	appearance := anObject! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 2/13/2011 17:40:57.882'!
bottomLeft

	^((self topLeft) + (0@(self height)))! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 2/13/2011 17:41:03.523'!
bottomMiddle

	^self bottomLeft + ((((0.5 * (self width)) rounded)@0))! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 2/13/2011 17:41:08.257'!
bottomRight

	^(self topRight + (0@(self height)))! !

!PgiWorldObject methodsFor: 'geometry' stamp: 'kh 2/17/2011 17:43:51.856'!
closestSegmentTo: aPoint

	^ self shape closestSegmentTo: (aPoint - (self position))! !

!PgiWorldObject methodsFor: 'geometry' stamp: 'RS 2/5/2011 21:22:19.861'!
containsPoint: aPoint

	^self shape containsPoint: aPoint - self position-(1@1)! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'kh 2/17/2011 17:44:04.689'!
extent

	^self shape extent! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'kh 2/17/2011 17:44:01.292'!
filledForm

	^self shape filledForm ! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'kh 1/13/2011 17:16:34.162'!
handleClickEvent 

	^self! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'kh 2/17/2011 17:44:12.268'!
height

	^self shape height! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 1/18/2011 17:48'!
midPoint
	"Returns the midPoint of PgiWorldObjects grafikal representation."
	| width height |
	
	width := ((self shape width) / 2) rounded.
	height :=((self shape height) / 2) rounded.
	
	^ (self position + (width@height))
	! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'lsb 12/16/2010 22:20:54.107'!
morph
	"Answer the value of morph"

	^ morph! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'lsb 12/16/2010 22:20:54.162'!
morph: anObject
	"Set the value of morph"

	morph := anObject! !

!PgiWorldObject methodsFor: 'additional Data' stamp: 'lsb 2/16/2011 16:44:04.159'!
pgiStep 

	self subclassResponsibility.! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'lsb 12/16/2010 22:20:53.865'!
position
	"Answer the value of position"

	^ position! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'lsb 12/16/2010 22:20:53.927'!
position: anObject
	"Set the value of position"

	position := anObject! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 2/6/2011 14:22:04.992'!
setVertices: aVerticeCollections
	| shiftedVertices |
	shiftedVertices := OrderedCollection new.
	aVerticeCollections do: [:each | shiftedVertices add: (each - (self position)) ].
	self shape setVertices: ( shiftedVertices asArray  ).! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'lsb 12/16/2010 22:20:54'!
shape
	"Answer the value of shape"

	^ shape! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'lsb 12/16/2010 22:20:54.05'!
shape: anObject
	"Set the value of shape"

	shape := anObject! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 2/13/2011 17:40:15.664'!
topLeft 

	^(self position )! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 2/13/2011 17:41:26.785'!
topMiddle

	^ (self topLeft + (((0.5 * (self width)) rounded)@0))! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 2/13/2011 17:41:32.246'!
topRight

	^((self topLeft ) + ((self width)@0))! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 2/19/2011 02:02:34.678'!
updateShape: aVerticesCollection withShift: aShiftVector
	
	self setVertices: aVerticesCollection.
	self morph updateShape: (self shape) withShift: aShiftVector.! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 2/6/2011 14:26:53.018'!
vertices
	| shiftedVertices |
	shiftedVertices := OrderedCollection new.
	(self shape vertices) do: [ :each | shiftedVertices add: (each + (self position))].
	^shiftedVertices ! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'pre 2/6/2011 14:24:53.554'!
width
	^self shape width! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'lsb 12/16/2010 22:20:53.65'!
world
	"Answer the value of world"

	^ world! !

!PgiWorldObject methodsFor: 'accessing' stamp: 'lsb 12/16/2010 22:20:53.767'!
world: anObject
	"Set the value of world"

	world := anObject! !
PgiResource initialize!
PgiPenguin initialize!
